import Foundation
import Async
import UIKit
import PlaytubeSDK
import AVKit
import Contacts

class AppInstance {
    
    static let instance = AppInstance()
    
    var userId: Int? = nil
    var sessionId: String? = nil
    var cookie: String? = nil
    var addCount:Int? = 0
    var userType = 0
    var contacts = [FetchedContact]()
    var notInterestedData = [NotInterestVideoModel.Datum]()
    var userProfile: ChannelModel?
    var siteSetting: Site_settings?
    
    func getUserSession() -> Bool {
        log.verbose("getUserSession = \(UserDefaults.standard.getUserSessions(Key: Local.USER_SESSION.User_Session))")
        let localUserSessionData = UserDefaults.standard.getUserSessions(Key: Local.USER_SESSION.User_Session)
        if localUserSessionData.isEmpty {
            return false
        } else {
            self.userId = (localUserSessionData[Local.USER_SESSION.user_id]  as! Int)
            self.sessionId = localUserSessionData[Local.USER_SESSION.session_id] as? String ?? ""
            self.cookie = localUserSessionData[Local.USER_SESSION.cookie] as? String ?? ""
            return true
        }
    }
    
    func fetchUserProfile(completion: @escaping (Bool) -> ()) {
        let status = AppInstance.instance.getUserSession()
        if status {
            let userId = AppInstance.instance.userId ?? 0
            let sessionId = AppInstance.instance.sessionId ?? ""
            Async.background {
                Async.background {
                    MyChannelManager.instance.getChannelInfo(User_id: userId, Session_Token: sessionId, Channel_id: userId, completionBlock: { (success, sessionError, error) in
                        if success != nil {
                            Async.main {
                                AppInstance.instance.userProfile = success
                                completion(true)
                                log.debug("success")
                            }
                        } else if sessionError != nil {
                            Async.main {
                                log.error("sessionErrror = \(sessionError?.errors!.error_text ?? "")")
                                completion(false)
                            }
                        } else {
                            Async.main {
                                log.error("error = \(error?.localizedDescription ?? "")")
                                completion(false)
                            }
                        }
                    })
                }
            }
        } else {
            log.error(InterNetError)
        }
    }
    
    func getNotInterestedData(data: [VideoDetail]) -> [VideoDetail] {
        var array: [VideoDetail] = []
        for i in data {
            let result = AppInstance.instance.notInterestedData.contains(where: {$0.video_id == i.id})
            if !result {
                array.append(i)
            }
        }
        return array
    }
    
    func fetchContacts() {
        print("Attempting to fetch contacts")
        let store = CNContactStore()
        store.requestAccess(for: .contacts) { (granted, error) in
            if let error = error {
                print("failed to request access", error)
                return
            }
            if granted {
                print("access granted")
                let keys = [CNContactGivenNameKey, CNContactFamilyNameKey, CNContactPhoneNumbersKey]
                let request = CNContactFetchRequest(keysToFetch: keys as [CNKeyDescriptor])
                do {
                    try store.enumerateContacts(with: request, usingBlock: { (contact, stopPointer) in
                        print(contact.givenName)
                        self.contacts.append(FetchedContact(firstName: contact.givenName, lastName: contact.familyName, telephone: contact.phoneNumbers.first?.value.stringValue ?? ""))
                    })
                } catch let error {
                    print("Failed to enumerate contact", error)
                }
            } else {
                print("access denied")
            }
        }
    }
    
    func getNotInterestedDataAPI() {
        if Connectivity.isConnectedToNetwork() {
            NotInterestManager.instance.getNotInterestDataAPI { success, sessionError, error in
                if success != nil {
                    self.notInterestedData = success?.data ?? []
                    log.verbose("Not Interest Data = \(self.notInterestedData.count)")
                } else if sessionError != nil {
                    Async.main {
                        log.error("responseError = \(sessionError?.errors!.error_text ?? "")")
                    }
                } else {
                    log.error("error = \(error?.localizedDescription ?? "")")
                }
            }
        } else {
            log.error(InterNetError)
        }
    }
    
    func getWatchLaterData() -> [VideoDetail] {
        let allData =  UserDefaults.standard.getSharedVideos(Key: Local.WATCH_LATER.watch_Later)
        if allData.isEmpty {
            return []
        } else {
            log.verbose("all data = \(allData)")
            var watchLaterVideos: [VideoDetail] = []
            for data in allData {
                let videoObject = try? PropertyListDecoder().decode(VideoDetail.self ,from: data)
                if videoObject != nil {
                    log.verbose("videoObject = \(videoObject?.id ?? 0)")
                    watchLaterVideos.append(videoObject!)
                } else {
                    log.verbose("Nil values cannot be append in Array!")
                }
            }
            return watchLaterVideos
        }
    }
    
}
