import UIKit
import SDWebImage
import Async
import Toast_Swift
import PlaytubeSDK

protocol ArticleSectionOneTableItemDelegate {
    func handleUserProfileButtonTap(_ sender: UIButton)
}

class ArticleSectionOneTableItem: UITableViewCell {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var thumbnailImage: UIImageView!
    @IBOutlet weak var disLikeCountLabel: UILabel!
    @IBOutlet weak var likeCountLabel: UILabel!
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var lblViews: UILabel!
    @IBOutlet weak var lblUsername: UILabel!
    @IBOutlet weak var lblSubscribers: UILabel!
    @IBOutlet weak var imgProfile: UIImageView!
    @IBOutlet weak var lblDescSecond: UILabel!
    @IBOutlet weak var imgLike: UIImageView!
    @IBOutlet weak var imgDislike: UIImageView!
    
    // MARK: - Properties
    
    var articleData: Article?
    var delegate: ArticleSectionOneTableItemDelegate?
    
    // MARK: - Initialize Functions
    
    override func awakeFromNib() {
        super.awakeFromNib()
  
    }
    
    override func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)
        
        // Configure the view for the selected state
    }
    
    // MARK: - Selectors
    
    // User Profile Button Action
    @IBAction func userProfileButtonAction(_ sender: UIButton) {
        self.delegate?.handleUserProfileButtonTap(sender)
    }
    
    // Like Button Action
    @IBAction func likeButtonAction(_ sender: UIButton) {
        self.likeDislikeArticle(postID: self.articleData?.id ?? 0, type: "like")
    }
    
    // DisLike Button Action
    @IBAction func disLikeButtonAction(_ sender: UIButton) {
        self.likeDislikeArticle(postID: self.articleData?.id ?? 0, type: "dislike")
    }
    
    // MARK: - Helper functions
    
    func bind(_ object: Article) {
        self.titleLabel.text = object.title?.htmlAttributedString
        self.lblDescSecond.text = (object.description?.htmlAttributedString ?? "")  + "\n" + (object.text ?? "")
        self.lblViews.text = "\(object.views ?? "") Views | \(object.shared ?? 0) Shares | \(object.comments_count ?? 0) Comments | \(object.text_time ?? "")"
        self.likeCountLabel.text = "\(object.likes ?? 0)"
        self.disLikeCountLabel.text = "\(object.dislikes ?? 0)"
        let thumbnailImage = URL(string: object.image ?? "")
        self.thumbnailImage.sd_setShowActivityIndicatorView(true)
        self.thumbnailImage.sd_setIndicatorStyle(.medium)
        DispatchQueue.global(qos: .userInteractive).async {
            self.thumbnailImage.sd_setImage(with: thumbnailImage, placeholderImage: R.image.maxresdefault())
        }
        self.lblUsername.text = object.user_data?.username ?? ""
        switch object.user_data?.subscribe_count {
        case .integer(let value):
            self.lblSubscribers.text = "\(value) Subscribers"
        case .string(let value):
            self.lblSubscribers.text = "\(value) Subscribers"
        default:
            break
        }
        let profileImage = URL(string: object.user_data?.avatar ?? "")
        self.imgProfile.sd_setShowActivityIndicatorView(true)
        self.imgProfile.sd_setIndicatorStyle(.medium)
        DispatchQueue.global(qos: .userInteractive).async {
            self.imgProfile.sd_setImage(with: profileImage, placeholderImage: R.image.maxresdefault())
        }
        if object.liked == 1 {
            self.imgLike.image = R.image.like_blue1()
            self.imgDislike.image = R.image.dislike_gray()
        } else if object.disliked == 1 {
            self.imgLike.image = R.image.like_gray()
            self.imgDislike.image = R.image.dislike_blue1()
        } else {
            self.imgLike.image = R.image.like_gray()
            self.imgDislike.image = R.image.dislike_gray()
        }
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension ArticleSectionOneTableItem {
    
    func likeDislikeArticle(postID: Int, type: String) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                ArticlesManager.instance.likeDislikeArticles(user_id: userID, session_token: sessionID, post_id: postID, type: type, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            if type == "like" {
                                if success?.code == 1 {
                                    self.imgLike.image = R.image.like_blue1()
                                    self.likeCountLabel.text = "\(success?.likes ?? 0)"
                                    self.articleData?.liked = 1
                                    obj_appDelegate.window?.rootViewController?.view.makeToast(NSLocalizedString("Liked", comment: "Liked"))
                                    if self.articleData?.disliked == 1 {
                                        self.imgDislike.image = R.image.dislike_gray()
                                        self.articleData?.disliked = 0
                                        self.disLikeCountLabel.text = "\(success?.dislikes ?? 0)"
                                    }
                                } else {
                                    self.imgLike.image = R.image.like_gray()
                                    self.likeCountLabel.text = "\(success?.likes ?? 0)"
                                    self.articleData?.liked = 0
                                }
                            } else {
                                if success?.code == 1 {
                                    obj_appDelegate.window?.rootViewController?.view.makeToast(NSLocalizedString("Disliked", comment: "Disliked"))
                                    self.articleData?.disliked = 1
                                    self.imgDislike.image = R.image.dislike_blue1()
                                    self.disLikeCountLabel.text = "\(success?.dislikes ?? 0)"
                                    if self.articleData?.liked == 1 {
                                        self.articleData?.liked = 0
                                        self.imgLike.image = R.image.like_gray()
                                        self.likeCountLabel.text = "\(success?.likes ?? 0)"
                                    }
                                } else {
                                    self.imgDislike.image = R.image.dislike_gray()
                                    self.articleData?.disliked = 0
                                    self.disLikeCountLabel.text = "\(success?.dislikes ?? 0)"
                                }
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            obj_appDelegate.window?.rootViewController?.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        Async.main {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            obj_appDelegate.window?.rootViewController?.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                })
            }
        } else {
            obj_appDelegate.window?.rootViewController?.view.makeToast(InterNetError)
        }
    }
    
}
