
import UIKit
import Async
import PlaytubeSDK
import FittedSheets
import UIView_Shimmer

class ArticlesSectionFiveTableItem: UITableViewCell, ShimmeringViewProtocol {
    
    @IBOutlet weak var lblUsername: UILabel!
    @IBOutlet weak var commentTextLabel: UILabel!
    @IBOutlet weak var profileLeadingConst: NSLayoutConstraint!
    @IBOutlet weak var likeCountLabel: UILabel!
    @IBOutlet weak var dislikeBtn: UIButton!
    @IBOutlet weak var likeBtn: UIButton!
    @IBOutlet weak var btnReplies: UIButton!
    @IBOutlet weak var profileImage: UIImageView!
    
    var shimmeringAnimatedItems: [UIView] {
        [
            lblUsername,
            commentTextLabel,
            likeCountLabel,
            dislikeBtn,
            likeBtn,
            btnReplies,
            profileImage
        ]
    }
    
    var commentData: Comment?
    var articlesReplyData: ReplyComment?
    var articleReplyVC: ArticleCommentReplyVC?
    var playerCommentVC: PlayerCommentVC?
    var likeCount:Int? = 0
    var dislikeCount:Int? = 0
    
    override func awakeFromNib() {
        super.awakeFromNib()
        
    }
    
    override func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)
        
    }
    
    @IBAction func likePressed(_ sender: Any) {
        if AppInstance.instance.getUserSession() {
            if self.commentData != nil {
                self.likeComment(commentID: self.commentData?.id ?? 0)
            } else if self.articlesReplyData != nil {
                self.likeReplyComment(commentID: self.articlesReplyData?.id ?? 0)
            }
        } else {
            let vc = R.storyboard.popups.loginPopupVC()
            obj_appDelegate.window?.rootViewController?.present(vc!, animated: true, completion: nil)
        }
    }
    
    @IBAction func sharePressed(_ sender: Any) {
        if AppInstance.instance.getUserSession() {
            let viewController = R.storyboard.articles.articleCommentReplyVC()
            viewController?.commentData = self.commentData
            let controller = SheetViewController(controller:viewController!, sizes: [.fixed(420), .intrinsic])
            self.playerCommentVC?.present(controller, animated: true, completion: nil)
        } else {
            let vc = R.storyboard.popups.loginPopupVC()
            obj_appDelegate.window?.rootViewController?.present(vc!, animated: true, completion: nil)
        }
    }
    
    @IBAction func dislikePressed(_ sender: Any) {
        if AppInstance.instance.getUserSession() {
            if self.commentData != nil {
                self.disLikeComment(commentID: self.commentData?.id ?? 0)
                
            } else if self.articlesReplyData != nil {
                self.disLikeReplyComment(commentID: self.articlesReplyData?.id ?? 0)
            }
        } else {
            let vc = R.storyboard.popups.loginPopupVC()
            obj_appDelegate.window?.rootViewController?.present(vc!, animated: true, completion: nil)
        }
    }
    
    func bind(_ object: Comment?, articleData: ReplyComment?) {
        if object != nil{
            self.btnReplies.isHidden = false
            self.commentData = object
            self.likeCount = object?.likes ?? 0
            self.dislikeCount = object?.disLikes ?? 0
            self.commentTextLabel.text = object?.text?.htmlAttributedString ?? ""
            self.likeCountLabel.text = "\(object?.likes ?? 0)"
            self.btnReplies.setTitle("\(object?.repliesCount ?? 0) Replies", for: .normal)
            let profileImage = URL(string: object?.commentUserData?.avatar ?? "")
            self.lblUsername.text = "\(object?.commentUserData?.username ?? "") | \(object?.textTime ?? "")"
            self.profileImage.sd_setImage(with: profileImage , placeholderImage:R.image.maxresdefault())
            if object?.isLikedComment == 1 {
                self.likeBtn.setImage(R.image.like_blue1(), for: .normal)
                self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
            } else if object?.isDislikedComment == 1 {
                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                self.dislikeBtn.setImage(R.image.dislike_blue1(), for: .normal)
            } else {
                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
            }
        } else if articleData != nil {
            self.articlesReplyData = articleData
            self.likeCount = articleData?.replyLikes ?? 0
            self.dislikeCount = articleData?.replyDislikes ?? 0
            self.commentTextLabel.text = articleData?.text?.htmlAttributedString ?? ""
            self.likeCountLabel.text = "\(articleData?.replyLikes ?? 0)"
            self.btnReplies.isHidden = true
            //self.dislikeCountLabel.text = "\(articleData?.replyDislikes ?? 0)"
            let profileImage = URL(string: articleData?.replyUserData?.avatar ?? "")
            self.lblUsername.text = "\(articleData?.replyUserData?.username ?? "") | \(articleData?.textTime ?? "")"
            self.profileImage.sd_setImage(with: profileImage , placeholderImage:R.image.maxresdefault())
            if articleData?.isLikedReply == 1 {
                self.likeBtn.setImage(R.image.like_blue1(), for: .normal)
                self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
            } else if articleData?.isDislikedReply == 1 {
                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                self.dislikeBtn.setImage(R.image.dislike_blue1(), for: .normal)
            } else {
                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
            }
        }
    }
    
    func likeComment(commentID: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                PlayVideoManager.instance.likeComments(User_id: userID, Session_Token: sessionID, Type: "like", Comment_Id: commentID) { (success, sessionError, error) in
                    if success != nil{
                        Async.main {
                            log.verbose("success \(success?.liked ?? 0) ")
                            if success?.liked == 1 {
                                self.likeBtn.setImage(R.image.like_blue1(), for: .normal)
                                log.verbose("success = \(success?.successType ?? "")")
                                let likes = self.commentData?.likes ?? 0
                                self.commentData?.likes = likes + 1
                                self.commentData?.isLikedComment = 1
                                self.likeCountLabel.text = "\(self.commentData?.likes ?? 0)"
                                if self.commentData?.isDislikedComment == 1 {
                                    self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
                                    self.commentData?.isDislikedComment = 0
                                }
                                self.playerCommentVC!.view.makeToast(NSLocalizedString("Liked", comment: "Liked"))
                            } else {
                                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                                log.verbose("success = \(success?.successType ?? "")")
                                self.commentData?.isLikedComment = 0
                                let likes = self.commentData?.likes ?? 0
                                self.commentData?.likes = likes - 1
                                self.likeCountLabel.text = "\(self.commentData?.likes ?? 0)"
                            }
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        self.playerCommentVC!.view.makeToast(sessionError?.errors?.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.playerCommentVC!.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.playerCommentVC!.view.makeToast(InterNetError)
        }
    }
    
    func disLikeComment(commentID: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                PlayVideoManager.instance.dislikeComments(User_id: userID, Session_Token: sessionID, Type: "dislike", Comment_Id: commentID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            log.verbose("success \(success?.dislike ?? 0) ")
                            if success?.dislike == 1 {
                                self.dislikeBtn.setImage(R.image.dislike_blue1(), for: .normal)
                                self.commentData?.isDislikedComment = 1
                                if self.commentData?.isLikedComment == 1 {
                                    self.commentData?.isLikedComment = 0
                                    let likes = self.commentData?.likes ?? 0
                                    self.commentData?.likes = likes - 1
                                    self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                                    self.likeCountLabel.text = "\(self.commentData?.likes ?? 0)"
                                }
                                self.playerCommentVC!.view.makeToast(NSLocalizedString("Disliked", comment: "Disliked"))
                            } else {
                                self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
                                self.commentData?.isDislikedComment = 0
                            }
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        self.playerCommentVC!.view.makeToast(sessionError?.errors?.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.playerCommentVC!.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.playerCommentVC!.view.makeToast(InterNetError)
        }
    }
    
    func likeReplyComment(commentID: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                PlayVideoManager.instance.likeCommentReplies(User_id: userID, Session_Token: sessionID, Type: "like", Reply_Id: commentID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            log.verbose("success \(success?.liked ?? 0) ")
                            if success?.liked == 1 {
                                self.likeBtn.setImage(R.image.like_blue1(), for: .normal)
                                log.verbose("success = \(success?.successType ?? "")")
                                let likes = self.articlesReplyData?.replyLikes ?? 0
                                self.articlesReplyData?.replyLikes = likes + 1
                                self.articlesReplyData?.isLikedReply = 1
                                self.likeCountLabel.text = "\(self.articlesReplyData?.replyLikes ?? 0)"
                                if self.articlesReplyData?.isDislikedReply == 1 {
                                    self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
                                    self.articlesReplyData?.isDislikedReply = 0
                                }
                                self.articleReplyVC!.view.makeToast(NSLocalizedString("Liked", comment: "Liked"))
                            } else {
                                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                                log.verbose("success = \(success?.successType ?? "")")
                                self.articlesReplyData?.isLikedReply = 0
                                let likes = self.articlesReplyData?.replyLikes ?? 0
                                self.articlesReplyData?.replyLikes = likes - 1
                                self.likeCountLabel.text = "\(self.articlesReplyData?.replyLikes ?? 0)"
                            }
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        self.articleReplyVC!.view.makeToast(sessionError?.errors?.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.articleReplyVC!.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.articleReplyVC!.view.makeToast(InterNetError)
        }
    }
    
    func disLikeReplyComment(commentID: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                PlayVideoManager.instance.dislikeCommentReplies(User_id: userID, Session_Token: sessionID, Type: "dislike", Reply_id: commentID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            log.verbose("success \(success?.dislike ?? 0) ")
                            if success?.dislike == 1 {
                                self.dislikeBtn.setImage(R.image.dislike_blue1(), for: .normal)
                                self.articlesReplyData?.isDislikedReply = 1
                                if self.articlesReplyData?.isLikedReply == 1 {
                                    self.articlesReplyData?.isLikedReply = 0
                                    let likes = self.articlesReplyData?.replyLikes ?? 0
                                    self.articlesReplyData?.replyLikes = likes - 1
                                    self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                                    self.likeCountLabel.text = "\(self.articlesReplyData?.replyLikes ?? 0)"
                                }
                                self.articleReplyVC!.view.makeToast(NSLocalizedString("Disliked", comment: "Disliked"))
                            } else {
                                self.dislikeBtn.setImage(R.image.dislike_gray(), for: .normal)
                                self.articlesReplyData?.isDislikedReply = 0
                            }
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        self.articleReplyVC!.view.makeToast(sessionError?.errors?.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.articleReplyVC!.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.articleReplyVC!.view.makeToast(InterNetError)
        }
    }
    
}
