import UIKit
import WebKit
import SDWebImage
import UIView_Shimmer

class NotificationTableItem: UITableViewCell, ShimmeringViewProtocol {
    
    @IBOutlet weak var notificationDot: UIImageView!
    @IBOutlet weak var profileImage: UIImageView!
    @IBOutlet weak var timeLabel: UILabel!
    @IBOutlet weak var titleLabel: UILabel!    
    @IBOutlet weak var thumbnailImage: UIImageView!
    
    var shimmeringAnimatedItems: [UIView] {
        [
            notificationDot,
            profileImage,
            timeLabel,
            titleLabel,
            thumbnailImage
        ]
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
    }

    override func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)

    }
    
    func bind(_ object: NotificationsModel.Notification) {
        let profileImage = URL(string: object.userData?.avatar ?? "")
        self.profileImage.sd_setShowActivityIndicatorView(true)
        self.profileImage.sd_setIndicatorStyle(.medium)
        DispatchQueue.global(qos: .userInteractive).async {
            self.profileImage.sd_setImage(with: profileImage , placeholderImage:R.image.maxresdefault())
        }
        let userNameFont = UIFont(name: "TTCommons-DemiBold", size: 16.0)
        let title = ((object.userData?.name ?? "") + " " + (object.title ?? ""))
        self.titleLabel.attributedText = title.attributedString(subStr: object.userData?.name ?? "", color: .primary_UI_Primary, font: userNameFont)
        self.timeLabel.text = object.time ?? ""
        if let videoDetail = object.video?.videoClass {
            self.thumbnailImage.isHidden = false
            let thumbnailImage = URL(string: videoDetail.thumbnail ?? "")
            self.thumbnailImage.sd_setShowActivityIndicatorView(true)
            self.thumbnailImage.sd_setIndicatorStyle(.medium)
            DispatchQueue.global(qos: .userInteractive).async {
                self.thumbnailImage.sd_setImage(with: thumbnailImage, placeholderImage: R.image.maxresdefault())
            }
        } else {
            self.thumbnailImage.isHidden = true
        }
    }
    
}

extension String {
    
    func attributedString(subStr: String, color: UIColor, font: UIFont?) -> NSMutableAttributedString {
        let range = (self as NSString).range(of: subStr)
        let attributedString = NSMutableAttributedString(string:self)
        attributedString.addAttributes([
            NSAttributedString.Key.foregroundColor: color,
            NSAttributedString.Key.font: font
        ], range: range)
        attributedString.addAttribute(NSAttributedString.Key.foregroundColor, value: color , range: range)
        return attributedString
    }
}
