//
//  VideoDetailCell.swift
//  Playtube
//
//  Created by Ubaid Javaid on 5/23/20.
//  Copyright © 2020 Muhammad Haris Butt. All rights reserved.
//

import UIKit
import SDWebImage
import Async
import ActiveLabel
import PlaytubeSDK
import UIView_Shimmer
import Toast_Swift
import SafariServices

protocol VideoDetailCellDelegate {
    func showDescriptionButtonTapped()
}

class VideoDetailCell: UITableViewCell, ShimmeringViewProtocol {
    
    // MARK: - IBOutelets
    
    @IBOutlet weak var profileImage: UIImageView!
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var viewsLabel: UILabel!
    @IBOutlet weak var userName: UILabel!
    @IBOutlet weak var updownBtn: UIButton!
    @IBOutlet weak var likeBtn: UIButton!
    @IBOutlet weak var disLikeBtn: UIButton!
    @IBOutlet weak var shareBtn: UIButton!
    @IBOutlet weak var AddBtn: UIButton!
    @IBOutlet weak var SubscribeBtn: UIButton!
    @IBOutlet weak var publishedDate: UILabel!
    @IBOutlet weak var descriptionLabel: ActiveLabel!
    @IBOutlet weak var CategoryLabel: UILabel!
    @IBOutlet weak var likeCountLabel: UILabel!
    @IBOutlet weak var donateLabel: UILabel!
    @IBOutlet weak var editLabel: UILabel!
    @IBOutlet weak var disLikeCountLabel: UILabel!
    @IBOutlet weak var shareLabel: UILabel!
    @IBOutlet weak var subscriberCount: UILabel!
    @IBOutlet weak var addLabel: UILabel!
    @IBOutlet weak var categoryLbl: UILabel!
    @IBOutlet weak var btnDonate: UIButton!
    @IBOutlet weak var detailView: UIView!
    @IBOutlet weak var editView: UIView!
    @IBOutlet weak var donateView: UIView!
    
    // MARK: - Properties
    
    var videoLoaction: String? =  nil
    var videoId: Int? = nil
    var tabBarController: TabbarController!
    var object: VideoDetail?
    var isShowDescription: Bool = false
    var shimmeringAnimatedItems: [UIView] {
        [
            titleLabel,
            viewsLabel,
            likeCountLabel,
            disLikeCountLabel,
            donateLabel,
            editLabel,
            shareLabel,
            addLabel,
            profileImage,
            userName,
            subscriberCount,
            SubscribeBtn,
            publishedDate,
            descriptionLabel,
            categoryLbl,
            CategoryLabel            
        ]
    }
    var delegate: VideoDetailCellDelegate?
    
    // MARK: - View Life Cycles
    
    override func awakeFromNib() {
        super.awakeFromNib()
        
    }
    
    override func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)
        
        // Configure the view for the selected state
    }
    
    // MARK: - Selectors
    
    // Like Button Action
    @IBAction func likeButtonAction(_ sender: UIButton) {
        if AppInstance.instance.getUserSession() {
            self.like()
        } else {
            let newVC = R.storyboard.popups.loginPopupVC()
            self.tabBarController?.present(newVC!, animated: true)
        }
    }
    
    // Dis Like Button Action
    @IBAction func disLikeButtonAction(_ sender: UIButton) {
        if AppInstance.instance.getUserSession() {
            self.disLike()
        } else {
            let newVC = R.storyboard.popups.loginPopupVC()
            self.tabBarController?.present(newVC!, animated: true, completion: nil)
        }
    }
    
    // Share Button Action
    @IBAction func shareButtonAction(_ sender: UIButton) {
        self.shareVideoLink()
    }
    
    // Add To Button Action
    @IBAction func addToButtonAction(_ sender: UIButton) {
        let popupVC = R.storyboard.popups.addToPopupVC()
        popupVC?.delegate = self
        self.tabBarController?.present(popupVC!, animated: true)
    }
    
    // Donate Button Action
    @IBAction func donateButtonAction(_ sender: UIButton) {
        let email = (self.object?.owner?.donation_paypal_email?.addingPercentEncoding(withAllowedCharacters: .urlHostAllowed) ?? "")
        let name = (self.object?.owner?.name?.addingPercentEncoding(withAllowedCharacters: .urlHostAllowed) ?? "")
        let urlString = "https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=" + email + "&lc=US&item_name=Donation+to+" + name + "&no_note=0&cn=&currency_code=USD&bn=PP-DonationsBF:btn_donateCC_LG.gif:NonHosted"
        print("urlString >>>>> ", urlString)
        if let url = URL(string: urlString) {
            let safariViewController = SFSafariViewController(url: url)
            safariViewController.delegate = self
            self.tabBarController?.present(safariViewController, animated: true, completion: nil)
        }
    }
    
    // Edit Button Action
    @IBAction func editButtonAction(_ sender: UIButton) {
        self.tabBarController?.minimizeVideoPlayer()
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.5) {
            self.tabBarController?.handleDismissVideoPlayer()
        }
        if let newVC = R.storyboard.loggedUser.editVideoVC() {
            newVC.object = self.object
            self.tabBarController?.navigationController?.pushViewController(newVC, animated: true)
        }
    }
    
    // Subscribe Button Action
    @IBAction func subscribeButtonAction(_ sender: UIButton) {
        if AppInstance.instance.getUserSession() {
            if self.object?.owner?.am_i_subscribed == 0 {
                if self.object?.owner?.subscriber_price != "0" {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.titleText  = "Purchase Required"
                    warningPopupVC?.messageText = "This channel is paid, You must pay to subscribe"
                    warningPopupVC?.okText = "PURCHASE"
                    self.tabBarController?.present(warningPopupVC!, animated: true, completion: nil)
                    warningPopupVC?.okButton.tag = 1002
                } else {
                    self.subscribeChannel()
                }
            } else {
                self.subscribeChannel()
            }
        } else {
            let warningPopupVC = R.storyboard.popups.warningPopupVC()
            warningPopupVC?.delegate = self
            warningPopupVC?.okText = "YES"
            warningPopupVC?.cancelText = "NO"
            warningPopupVC?.titleText  = "Warning"
            warningPopupVC?.messageText = "Please sign in to subscribe to the channel"
            self.tabBarController?.present(warningPopupVC!, animated: true, completion: nil)
            warningPopupVC?.okButton.tag = 1001
        }
    }
    
    // Up Down Button Action
    @IBAction func upDownButtonAction(_ sender: UIButton) {
        self.isShowDescription = !self.isShowDescription
        if self.isShowDescription {
            self.detailView.isHidden = false
            self.titleLabel.numberOfLines = 0
            self.descriptionLabel.numberOfLines = 0
            self.updownBtn.setImage(UIImage(named: "up_new"), for: .normal)
        } else {
            self.detailView.isHidden = true
            self.titleLabel.numberOfLines = 1
            self.descriptionLabel.numberOfLines = 1
            self.updownBtn.setImage(UIImage(named: "down_new"), for: .normal)
        }
        self.layoutIfNeeded()
        self.delegate?.showDescriptionButtonTapped()
    }
    
    // MARK: - Helper Functions
    
    func bind(object: VideoDetail) {
        self.object = object
        self.videoLoaction = object.video_location ?? ""
        self.videoId = object.id ?? 0
        let url = URL(string: object.owner?.avatar ?? "")
        self.profileImage.sd_setShowActivityIndicatorView(true)
        self.profileImage.sd_setIndicatorStyle(.medium)
        DispatchQueue.global(qos: .userInteractive).async {
            self.profileImage.sd_setImage(with: url , placeholderImage: R.image.maxresdefault())
        }
        self.titleLabel.text = object.title?.htmlAttributedString ?? ""
        if object.owner?.username == "" || object.owner?.username == nil {
            self.userName.text = "\(object.owner?.first_name ?? "") \(object.owner?.last_name ?? "")"
        } else {
            self.userName.text = object.owner?.username ?? ""
        }
        self.viewsLabel.text = "\(object.category_name?.htmlAttributedString ?? "") | \(object.views ?? 0) Views | \(object.time_ago ?? "")"
        self.publishedDate.text = "\("Published on ")\(object.time_date ?? "")"
        self.descriptionLabel.text = object.description?.removeContinuousBRTags().stringByDecodingURL.htmlAttributedString
        self.likeCountLabel.text =  "\(object.likes ?? 0)"
        self.disLikeCountLabel.text = "\(object.dislikes ?? 0)"
        self.editView.isHidden = AppInstance.instance.userId != object.owner?.id
        self.donateView.isHidden = (object.owner?.donation_paypal_email == nil || object.owner?.donation_paypal_email == "")
        self.categoryLbl.text = NSLocalizedString("Category", comment: "Category")
        self.CategoryLabel.text = object.category_name ?? ""
        switch object.owner?.subscribe_count {
        case .integer(let value):
            self.subscriberCount.text = "\(value) Subscribers"
        case .string(let value):
            self.subscriberCount.text = "\(value) Subscribers"
        default:
            break
        }
        if AppInstance.instance.getUserSession() {
            self.descriptionLabel.handleURLTap { (url) in
                UIApplication.shared.open(url, options: [:], completionHandler: nil)
            }
            if self.object?.owner?.am_i_subscribed == 0 {
                if self.object?.owner?.subscriber_price != "0" {
                    self.SubscribeBtn.setTitle("+ Subscribe $" + (self.object?.owner?.subscriber_price ?? "") , for: .normal)
                } else {
                    self.SubscribeBtn.setTitle("+ Subscribe", for: .normal)
                }
            } else {
                self.SubscribeBtn.setTitle("✓ Subscribed", for: .normal)
            }
            if object.is_liked == 1 {
                self.likeBtn.setImage(R.image.like_blue1(), for: .normal)
                self.disLikeBtn.setImage(R.image.dislike_gray(), for: .normal)
            } else if object.is_disliked == 1 {
                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                self.disLikeBtn.setImage(R.image.dislike_blue1(), for: .normal)
            } else {
                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                self.disLikeBtn.setImage(R.image.dislike_gray(), for: .normal)
            }
        }
    }
    
    func shareVideoLink() {
        // text to share
        let url = self.object?.url ?? ""
        print(url)
        // set up activity view controller
        let textToShare = [ url ]
        let activityViewController = UIActivityViewController(activityItems: textToShare, applicationActivities: nil)
        activityViewController.popoverPresentationController?.sourceView = self
        // exclude some activity types from the list (optional,)
        activityViewController.excludedActivityTypes = [
            UIActivity.ActivityType.airDrop,
            UIActivity.ActivityType.postToFacebook,
            UIActivity.ActivityType.assignToContact,
            UIActivity.ActivityType.mail,
            UIActivity.ActivityType.postToTwitter,
            UIActivity.ActivityType.message,
            UIActivity.ActivityType.postToFlickr,
            UIActivity.ActivityType.postToVimeo,
            UIActivity.ActivityType.postToTencentWeibo,
            UIActivity.ActivityType.init(rawValue: "net.whatsapp.WhatsApp.ShareExtension"),
            UIActivity.ActivityType.init(rawValue: "com.google.Gmail.ShareExtension"),
            UIActivity.ActivityType.init(rawValue: "com.toyopagroup.picaboo.share"),
            UIActivity.ActivityType.init(rawValue: "com.tinyspeck.chatlyio.share")
        ]
        activityViewController.completionWithItemsHandler = { activity, completed, items, error in
            if !completed {
                return
            } else {
                if self.object != nil {
                    log.verbose("Check = \(UserDefaults.standard.getSharedVideos(Key: Local.SHARED_VIDEOS.shared_videos))")
                    let objectToEncode = self.object
                    let data = try? PropertyListEncoder().encode(objectToEncode)
                    var getSharedVideosData = UserDefaults.standard.getSharedVideos(Key: Local.SHARED_VIDEOS.shared_videos)
                    if UserDefaults.standard.getSharedVideos(Key: Local.SHARED_VIDEOS.shared_videos).contains(data!) {
                        self.tabBarController?.view.makeToast(NSLocalizedString("Already added in shared videos", comment: "Already added in shared videos"))
                    } else {
                        getSharedVideosData.append(data!)
                        UserDefaults.standard.setSharedVideos(value: getSharedVideosData, ForKey: Local.SHARED_VIDEOS.shared_videos)
                        self.tabBarController?.view.makeToast(NSLocalizedString("Added to shared videos", comment: "Added to shared videos"))
                    }
                }
            }
        }
        // present the view controller
        self.tabBarController?.present(activityViewController, animated: true, completion: nil)
    }
    
}

// MARK: Api Call
extension VideoDetailCell {
    
    func subscribeChannel() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let channelID = self.object?.owner?.id ?? 0
            Async.background {
                PlayVideoManager.instance.subUnsubChannel(User_id: userID, Session_Token: sessionID, Channel_Id: channelID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            if success?.code == 0 {
                                self.tabBarController?.view.makeToast("Unsubscribed Successfully")
                            } else {
                                self.tabBarController?.view.makeToast(NSLocalizedString("Subscribed Successfully", comment: "Subscribed Successfully"))
                            }
                            self.fetchUserProfile(Channel_id: self.object?.owner?.id ?? 0)
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors!.error_text ?? "")")
                        self.tabBarController?.view.makeToast(sessionError?.errors!.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.tabBarController?.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.tabBarController?.view.makeToast("InterNetError")
        }
    }
    
    func disLike() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let channelID = self.videoId ?? 0
            Async.background {
                PlayVideoManager.instance.likeDislikeVideos(User_id: userID, Session_Token: sessionID, Video_Id: channelID, Like_Type: "dislike", completionBlock: { (success, sessionError, error) in
                    Async.main {
                        if success?.success_type == "added_dislike"{
                            self.disLikeBtn.setImage(R.image.dislike_blue1(), for: .normal)
                            self.tabBarController?.view.makeToast(NSLocalizedString("Disliked", comment: "Disliked"))
                            let dislikes = self.object?.dislikes ?? 0
                            self.object?.dislikes = dislikes + 1
                            self.object?.is_disliked = 1
                            self.disLikeCountLabel.text = "\(self.object?.dislikes ?? 0)"
                            if self.object?.is_liked == 1 {
                                self.object?.is_liked = 0
                                let likes = self.object?.likes ?? 0
                                self.object?.likes = likes - 1
                                self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                                self.likeCountLabel.text = "\(self.object?.likes ?? 0)"
                            }
                        } else {
                            self.disLikeBtn.setImage(R.image.dislike_gray(), for: .normal)
                            log.verbose("success = \(success?.success_type ?? "")")
                            let dislikes = self.object?.dislikes ?? 0
                            self.object?.dislikes = dislikes - 1
                            self.object?.is_disliked = 0
                            self.disLikeCountLabel.text = "\(self.object?.dislikes ?? 0)"
                        }
                    }
                })
            }
        } else {
            self.tabBarController!.view.makeToast("InterNetError")
        }
    }
    
    func like() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let channelID = self.videoId ?? 0
            Async.background {
                PlayVideoManager.instance.likeDislikeVideos(User_id: userID, Session_Token: sessionID, Video_Id: channelID, Like_Type: "like", completionBlock: { (success, sessionError, error) in
                    Async.main {
                        if success?.success_type == "added_like"{
                            self.likeBtn.setImage(R.image.like_blue1(), for: .normal)
                            log.verbose("success = \(success?.success_type ?? "")")
                            self.tabBarController?.view.makeToast(NSLocalizedString("Liked", comment: "Liked"))
                            let likes = self.object?.likes ?? 0
                            self.object?.likes = likes + 1
                            self.object?.is_liked = 1
                            self.likeCountLabel.text = "\(self.object?.likes ?? 0)"
                            if self.object?.is_disliked == 1 {
                                self.disLikeBtn.setImage(R.image.dislike_gray(), for: .normal)
                                let dislikes = self.object?.dislikes ?? 0
                                self.object?.dislikes = dislikes - 1
                                self.object?.is_disliked = 0
                                self.disLikeCountLabel.text = "\(self.object?.dislikes ?? 0)"
                            }
                        } else {
                            self.likeBtn.setImage(R.image.like_gray(), for: .normal)
                            log.verbose("success = \(success?.success_type ?? "")")
                            self.object?.is_liked = 0
                            let likes = self.object?.likes ?? 0
                            self.object?.likes = likes - 1
                            self.likeCountLabel.text = "\(self.object?.likes ?? 0)"
                        }
                    }
                })
            }
        } else {
            self.tabBarController?.view.makeToast("InterNetError")
        }
    }
    
    func getPlaylist() {
        if Connectivity.isConnectedToNetwork() {
            self.tabBarController?.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                PlaylistManager.instance.getPlaylist(User_id: userID, Session_Token: sessionID, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.tabBarController?.dismissProgressDialog {
                                let newVC = R.storyboard.popups.playlistsPopupVC()
                                newVC?.videoID = self.videoId ?? 0
                                newVC?.playlistArray = success?.myAllPlaylists ?? []
                                newVC?.delegate = self.tabBarController!
                                self.tabBarController!.present(newVC!, animated: true, completion: nil)
                            }
                        }
                    } else if sessionError != nil {
                        self.tabBarController?.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.tabBarController?.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.tabBarController?.dismissProgressDialog {
                            log.verbose("Error = \(error?.localizedDescription ?? "")")
                            self.tabBarController?.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                })
            }
        } else {
            self.tabBarController?.dismissProgressDialog {
                self.tabBarController?.view.makeToast("InterNetError")
            }
        }
    }
    
    func payUsingWalletApi(pay_Type: String, id: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.tabBarController.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                WalletManager.instance.payUsingWalletApi(user_id: userID, session_token: sessionID, pay_Type: pay_Type, id: id) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.tabBarController.dismissProgressDialog {
                                log.verbose("success \(success?.success ?? "")")
                                self.fetchUserProfile(Channel_id: self.object?.owner?.id ?? 0)
                            }
                        }
                    } else if sessionError != nil {
                        self.tabBarController.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.tabBarController?.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.tabBarController.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.tabBarController?.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.tabBarController?.view.makeToast(InterNetError)
        }
    }
    
    func fetchUserProfile(Channel_id: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userId = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                MyChannelManager.instance.getChannelInfo(User_id: userId, Session_Token: sessionID, Channel_id: Channel_id, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.object?.owner = success?.data
                            if let object = self.object {
                                self.bind(object: object)
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.tabBarController?.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        Async.main {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.tabBarController?.view.makeToast(error?.localizedDescription)
                        }
                    }
                })
            }
        } else {
            self.tabBarController?.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: WarningPopupVCDelegate Methods
extension VideoDetailCell: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        if sender.tag == 1001 {
            let vc = R.storyboard.auth.loginVC()
            self.tabBarController?.navigationController?.pushViewController(vc!, animated: true)
        }
        
        if sender.tag == 1002 {
            if self.object?.owner?.am_i_subscribed == 0 && self.object?.owner?.subscriber_price != "0" {
                let amount = ((self.object?.owner?.subscriber_price ?? "") as NSString).doubleValue
                AppInstance.instance.fetchUserProfile { success in
                    if success {
                        let walletBalance = AppInstance.instance.userProfile?.data?.wallet?.rounded() ?? 0.0
                        if amount < walletBalance {
                            self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.object?.owner?.id ?? 0)
                        } else {
                            let warningPopupVC = R.storyboard.popups.warningPopupVC()
                            warningPopupVC?.delegate = self
                            warningPopupVC?.titleText  = "Wallet"
                            warningPopupVC?.messageText = "Sorry, You do not have enough money please top up your wallet"
                            warningPopupVC?.okText = "ADD WALLET"
                            self.tabBarController?.present(warningPopupVC!, animated: true, completion: nil)
                            warningPopupVC?.okButton.tag = 1003
                        }
                    }
                }
            }
            
        }
        if sender.tag == 1003 {
            let newVC = R.storyboard.settings.walletVC()
            newVC?.isPurchase = true
            newVC?.delegate = self
            self.tabBarController?.navigationController?.pushViewController(newVC!, animated: true)
        }
    }
    
}

// MARK: WalletVCDelegate
extension VideoDetailCell: WalletVCDelegate {
    
    func handleAddMoneyInWalletForPurchase() {
        self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.object?.owner?.id ?? 0)
    }
    
}

// MARK: AddToPopupVCDelegate Methods
extension VideoDetailCell: AddToPopupVCDelegate {
    
    func handleSaveToPlaylistTap(_ sender: UIButton) {
        if AppInstance.instance.getUserSession() {
         self.getPlaylist()
         } else {
         let newVC = R.storyboard.popups.loginPopupVC()
         self.tabBarController?.present(newVC!, animated: true, completion: nil)
         }
    }
    
    func handleSaveToWatchLaterTap(_ sender: UIButton) {
        if AppInstance.instance.getUserSession() {
            self.setWatchLater()
        } else {
            let warningPopupVC = R.storyboard.popups.warningPopupVC()
            warningPopupVC?.delegate = self
            warningPopupVC?.okText = "YES"
            warningPopupVC?.cancelText = "NO"
            warningPopupVC?.titleText  = "Warning"
            warningPopupVC?.messageText = "Please sign in to add to WatchLater videos"
            self.tabBarController?.present(warningPopupVC!, animated: true, completion: nil)
            warningPopupVC?.okButton.tag = 1001
        }
    }
    
    func setWatchLater() {
        if object != nil {
            let objectToEncode = object
            let data = try? PropertyListEncoder().encode(objectToEncode)
            var getWatchLaterData = UserDefaults.standard.getWatchLater(Key: Local.WATCH_LATER.watch_Later)
            if AppInstance.instance.getWatchLaterData().contains(where: { $0.id == object?.id }) {
                self.tabBarController?.view.makeToast(NSLocalizedString("Already added in watch later", comment: "Already added in watch later"))
            } else {
                getWatchLaterData.append(data!)
                UserDefaults.standard.setWatchLater(value: getWatchLaterData, ForKey: Local.WATCH_LATER.watch_Later)
                self.tabBarController?.view.makeToast(NSLocalizedString("Added to watch later", comment: "Added to watch later"))
            }
        }
    }
    
}

// MARK: SFSafariViewControllerDelegate Methods
extension VideoDetailCell: SFSafariViewControllerDelegate {
    
    func safariViewControllerDidFinish(_ controller: SFSafariViewController) {
        controller.dismiss(animated: true, completion: nil)
    }
    
}
