//
//  PopularCell.swift
//  Playtube
//
//  Created by iMac on 25/05/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit
import UIView_Shimmer
import Async
import Toast_Swift

protocol PopularCellDelegate {
    func handleSubscribeButtonTap(indexPath: IndexPath)
}

class PopularCell: UITableViewCell, ShimmeringViewProtocol {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var userName: UILabel!
    @IBOutlet weak var subscriberLabel: UILabel!
    @IBOutlet weak var profileImage: RoundImage!
    @IBOutlet weak var subscribeButton: UIButton!
    @IBOutlet weak var verifiedBadge: UIImageView!
    
    // MARK: - Properties
    
    var shimmeringAnimatedItems: [UIView] {
        [
            userName,
            subscriberLabel,
            profileImage,
            subscribeButton,
            verifiedBadge
        ]
    }
    var delegate: PopularCellDelegate?
    var indexPath = IndexPath()
    
    // MARK: - Initialize View

    override func awakeFromNib() {
        super.awakeFromNib()
        // Initialization code
    }

    override func setSelected(_ selected: Bool, animated: Bool) {
        super.setSelected(selected, animated: animated)

        // Configure the view for the selected state
    }
    
    // MARK: - Selectors
    
    // Subscribe Button Action
    @IBAction func subscribeButtonAction(_ sender: UIButton) {
        self.delegate?.handleSubscribeButtonTap(indexPath: self.indexPath)
    }
    
    // MARK: - Helper Functions
    
    func bind(_ object: Owner) {
        self.userName.text = object.name
        let profileImage = URL(string: object.avatar ?? "")
        self.profileImage.sd_setShowActivityIndicatorView(true)
        self.profileImage.sd_setIndicatorStyle(.medium)
        DispatchQueue.global(qos: .userInteractive).async {
            self.profileImage.sd_setImage(with: profileImage , placeholderImage:R.image.maxresdefault())
        }
        switch object.subscribe_count {
        case .integer(let value):
            self.subscriberLabel.text = "\(value) Subscribers"
        case .string(let value):
            self.subscriberLabel.text = "\(value) Subscribers"
        default:
            break
        }
        self.verifiedBadge.isHidden = (object.verified ?? 0) != 1
        if object.am_i_subscribed == 0 {
            if object.subscriber_price != "0" {
                self.subscribeButton.setTitle("Subscribe " + "$" + (object.subscriber_price ?? ""), for: .normal)
            } else {
                self.subscribeButton.setTitle("Subscribe", for: .normal)
            }
            self.subscribeButton.backgroundColor = UIColor(named: "Primary_UI_Primary")?.withAlphaComponent(0.1)
            self.subscribeButton.setTitleColor(UIColor(named: "Primary_UI_Primary"), for: .normal)
        } else {
            self.subscribeButton.setTitle("Subscribed", for: .normal)
            self.subscribeButton.backgroundColor = UIColor(named: "Fill_Colors_Secondary")
            self.subscribeButton.setTitleColor(UIColor(named: "Label_Colors_Secondary"), for: .normal)
        }
    }
    
}
