//
//  TwoFactorVC.swift
//  Playtube
//
//  Created by Muhammad Haris Butt on 17/08/2021.
//  Copyright © 2021 Muhammad Haris Butt. All rights reserved.
//

import UIKit
import PlaytubeSDK
import Async
import Toast_Swift

class TwoFactorVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var verifyCodeTextField: UITextField!
    
    // MARK: - Properties
    
    var userID: Int? = 0
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // Verify Button Action
    @IBAction func verifyButtonAction(_ sender: UIButton) {
        if (self.verifyCodeTextField.text?.isEmpty ?? false) {
            self.view.makeToast(NSLocalizedString("Please enter Code.", comment: "Please enter Code."))
        } else {
            self.verifyTwoFactor()
        }
    }
    
    // MARK: - Helper Functions
    
    private func verifyTwoFactor() {
        if appDelegate.isInternetConnected {
            if (self.verifyCodeTextField.text?.isEmpty ?? false) {
                let securityAlertVC = R.storyboard.popups.securityPopupVC()
                securityAlertVC?.titleText  = "Security"
                securityAlertVC?.errorText = NSLocalizedString("Please enter Code.", comment: "Please enter Code.")
                self.present(securityAlertVC!, animated: true, completion: nil)
            } else {
                self.showProgressDialog(text: "Loading...")
                let userID = self.userID ?? 0
                let code = self.verifyCodeTextField.text ?? ""
                Async.background({
                    UserManager.instance.twoFactor(userID: userID, code: Int(code)!) { (success, sessionError, error) in
                        if success != nil {
                            Async.main {
                                self.dismissProgressDialog{
                                    log.verbose("Success = \(success?.data?.sessionID ?? "")")
                                    log.verbose("Success = \(success?.data?.userID ?? 0)")
                                    AppInstance.instance.getUserSession()
                                    AppInstance.instance.fetchUserProfile { (success) in
                                        if (success) {
                                            AppInstance.instance.userType = 1
                                            let vc = R.storyboard.loggedUser.tabBarNav()
                                            self.appDelegate.window?.rootViewController = vc
                                        } else {
                                            print(false)
                                        }
                                    }
                                }
                            }
                        } else if sessionError != nil {
                            Async.main{
                                self.dismissProgressDialog {
                                    log.verbose("session Error = \(sessionError?.errors?.error_text ?? "")")
                                    let securityAlertVC = R.storyboard.popups.securityPopupVC()
                                    securityAlertVC?.titleText  = "Security"
                                    securityAlertVC?.errorText = sessionError?.errors?.error_text ?? ""
                                    self.present(securityAlertVC!, animated: true, completion: nil)
                                }
                            }
                        } else {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.verbose("error = \(error?.localizedDescription ?? "")")
                                    let securityAlertVC = R.storyboard.popups.securityPopupVC()
                                    securityAlertVC?.titleText  = "Security"
                                    securityAlertVC?.errorText = error?.localizedDescription ?? ""
                                    self.present(securityAlertVC!, animated: true, completion: nil)
                                }
                            }
                        }
                    }
                    
                })
            }
        } else {
            self.dismissProgressDialog {
                let securityAlertVC = R.storyboard.popups.securityPopupVC()
                securityAlertVC?.titleText  = "Internet Error"
                securityAlertVC?.errorText = InterNetError
                self.present(securityAlertVC!, animated: true, completion: nil)
                log.error("internetError - \(InterNetError)")
            }
        }
    }
    
}
