import UIKit
import Async
import PlaytubeSDK
import UIView_Shimmer
import Toast_Swift

class ArticleCommentReplyVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var commentTextField: UITextField!
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var noReplyLabel: UILabel!
    
    // MARK: - Properties
    
    var isLoading = true
    var commentData: Comment? = nil
    private var repliesArray: [ReplyComment] = []
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    @IBAction func sendPressed(_ sender: Any) {
        self.addReply()
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.tableView.separatorStyle = .none
        self.tableView.register(UINib(resource: R.nib.articlesSectionFiveTableItem), forCellReuseIdentifier: R.reuseIdentifier.articlesSectionFiveTableItem.identifier)
        log.verbose("commentData?.videoID = \(commentData?.videoID ?? 0)")
        self.isLoading = true
        self.tableView.reloadData()
        self.fetchCommentReplies()
    }
    
    private func fetchCommentReplies() {
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        let commentID = commentData?.id ?? 0
        Async.background {
            PlayVideoManager.instance.fetchReplies(user_id: userID, session_Token: sessionID, comment_Id: commentID, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.repliesArray = []
                            self.repliesArray = success?.data ?? []
                            self.isLoading = false
                            if self.repliesArray.isEmpty {
                                self.tableView.isHidden = false
                                self.noReplyLabel.isHidden = false
                            } else {
                                self.tableView.isHidden = false
                                self.noReplyLabel.isHidden = true
                            }
                            self.tableView.reloadData()
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            log.error("SessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text)
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                }
            })
        }
    }
    
    private func addReply() {
        self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        let commentText = self.commentTextField.text ?? ""
        Async.background {
            PlayVideoManager.instance.addCommentReply(User_id: userID, Session_Token: sessionID, VideoId: self.commentData?.videoID ?? 0, Comment_Id: self.commentData?.id ?? 0, Type: "reply", Comment_Text: commentText, completionBlock: { (success, sessionError, error) in
                if success != nil{
                    Async.main {
                        self.dismissProgressDialog {
                            self.fetchCommentReplies()
                            self.commentTextField.text = nil
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            log.error("SessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text)
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                }
            })
        }
    }
    
}

extension ArticleCommentReplyVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if isLoading {
            return 5
        } else {
            if self.repliesArray.count == 0 {
                return 1
            }
            return self.repliesArray.count + 1
        }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if isLoading {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articlesSectionFiveTableItem.identifier) as! ArticlesSectionFiveTableItem
            cell.profileLeadingConst.constant = indexPath.row == 0 ? 15.0 : 40.0
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.articlesSectionFiveTableItem.identifier) as! ArticlesSectionFiveTableItem
            cell.articleReplyVC = self
            if indexPath.row != 0 {
                let object = self.repliesArray[indexPath.row-1]
                cell.bind(nil, articleData: object)
            } else {
                cell.bind(self.commentData!, articleData: nil)
                cell.btnReplies.isEnabled = false
                cell.likeBtn.isEnabled = false
                cell.dislikeBtn.isEnabled = false
            }
            cell.profileLeadingConst.constant = indexPath.row == 0 ? 15.0 : 40.0
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        cell.setTemplateWithSubviews(isLoading, animate: true, viewBackgroundColor: .systemBackground)
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
}
