//
//  ChatScreenVC.swift
//  Playtube
//
//  Created by Muhammad Haris Butt on 1/11/20.
//  Copyright © 2020 Muhammad Haris Butt. All rights reserved.
//

import UIKit
import Async
import PlaytubeSDK
import DropDown
import Toast_Swift

class ChatScreenVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var messageTextField: UITextField!
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var lblTitle: UILabel!
    @IBOutlet weak var btnClear: UIButton!
    
    // MARK: - Properties
    
    let dropdown = DropDown()
    var recipentID: Int? = 0
    var username: String?  = ""
    var UserData: ChatUser?
    var userImg : String? = ""
    private var messageArray: [MessageModel] = []
    private var timer: Timer? = nil
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        self.tabBarController?.tabBar.isHidden = true
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        self.tabBarController?.tabBar.isHidden = false
        self.timer?.invalidate()
    }
    
    // MARK: - Selectors
    
    // Send Button Action
    @IBAction func sendButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.sendMessage()
    }
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
    
    // Clear Button Action
    @IBAction func clearButtonAction(_ sender: UIButton) {
        self.dropdown.show()
    }
    
    @objc func update() {
        self.fetchUserChats()
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.setupUI()
        self.customizeDropDownFunc()
        self.fetchUserChats()
    }
    
    private func setupUI() {
        self.lblTitle.text = self.username
        self.tableView.separatorStyle = .none
        tableView.register(UINib(resource: R.nib.chatLeftTableItem), forCellReuseIdentifier: R.reuseIdentifier.chatLeftTableItem.identifier)
        tableView.register(UINib(resource: R.nib.chatRightTableItem), forCellReuseIdentifier: R.reuseIdentifier.chatRightTableItem.identifier)
        timer = Timer.scheduledTimer(timeInterval: 5, target: self, selector: #selector(self.update), userInfo: nil, repeats: true)
    }
    
    private  func customizeDropDownFunc() {
        dropdown.dataSource = ["Clear chat"]
        dropdown.anchorView = self.btnClear
        dropdown.selectionAction = { [unowned self] (index: Int, item: String) in
            if index == 0 {
                self.cleartChat()
            }
            log.verbose("Selected item: \(item) at index: \(index)")
        }
        dropdown.direction = .any
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension ChatScreenVC {
    
    private func fetchUserChats() {
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        let receipentID = self.recipentID ?? 0
        Async.background {
            ChatManager.instance.getUserChats(User_id: userID, Session_Token: sessionID, First_id: 0, Last_id: 0, Recipient_id:receipentID , Limit: 0, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            log.debug("userList = \(success?.data?.messages ?? [])")
                            self.messageArray = []
                            self.messageArray = success?.data?.messages ?? []
                            self.tableView.reloadData()
                            if self.messageArray.count != 0 {
                                let indexPath = NSIndexPath(item: ((self.messageArray.count) - 1), section: 0)
                                self.tableView.scrollToRow(at: indexPath as IndexPath, at: UITableView.ScrollPosition.bottom, animated: true)
                            }
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text)
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.error("error = \(error?.localizedDescription ?? "")")
                        }
                    }
                }
            })
        }
    }
    
    private func sendMessage() {
        let hashID = Int(arc4random_uniform(UInt32(100000)))
        let messageText = self.messageTextField.text ?? ""
        let recipientId = self.recipentID ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        let userID = AppInstance.instance.userId ?? 0
        Async.background {
            ChatManager.instance.sendUserMessage(User_id: userID, Session_Token: sessionID, Recipient_id: recipientId, Hash_id: hashID, Text: messageText, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.messageTextField.text = ""
                            self.view.resignFirstResponder()
                            self.fetchUserChats()
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.error("error = \(error?.localizedDescription ?? "")")
                        }
                    }
                }
            })
        }
    }
    
    private func cleartChat() {
        let recipientId = self.recipentID ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        let userID = AppInstance.instance.userId ?? 0
        Async.background {
            ChatManager.instance.removeUserChats(User_id: userID, Session_Token: sessionID, Recipient_id: recipientId, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.navigationController?.popViewController(animated: true)
                            self.timer?.invalidate()
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.error("error = \(error?.localizedDescription ?? "")")
                        }
                    }
                }
            })
        }
    }
    
}

// MARK: TableView Setup
extension ChatScreenVC: UITableViewDataSource, UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.messageArray.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if self.messageArray.count == 0 {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.chatRightTableItem.identifier) as! ChatRightTableItem
            return cell
        }
        let object = self.messageArray[indexPath.row]
        if object.position == "right" {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.chatRightTableItem.identifier) as! ChatRightTableItem
            cell.bind(object)
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.chatLeftTableItem.identifier) as! ChatLeftTableItem
            cell.userData = self.UserData
            cell.userImg = self.userImg
            cell.bind(object)
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
}
