//
//  EditVideoVC.swift
//  Playtube
//
//  Created by iMac on 11/12/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit
import Async
import PlaytubeSDK
import Toast_Swift

class EditVideoVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var cancelButton: UIButton!
    @IBOutlet weak var videoImage: UIImageView!
    @IBOutlet weak var videoTitleTextField: UITextField!
    @IBOutlet weak var descriptionTextView: UITextView!
    @IBOutlet weak var descriptionTextViewHeight: NSLayoutConstraint!
    @IBOutlet weak var tagsTextView: UITextView!
    @IBOutlet weak var tagsTextViewHeight: NSLayoutConstraint!
    @IBOutlet weak var categoryTextField: UITextField!
    @IBOutlet weak var privacyTextField: UITextField!
    @IBOutlet weak var ageRestrictionTextField: UITextField!
    
    // MARK: - Properties
    
    var object: VideoDetail?
    var mediaData: Data? = nil
    private let imagePickerController = UIImagePickerController()
    var privacy = 0
    var age_restriction = 0
    var category_id = ""
    
    // MARK: - View Life Cycles

    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // Cancel Button Action
    @IBAction func cancelButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.videoImage.image = UIImage(named: "maxresdefault")
        self.cancelButton.isHidden = true
        self.mediaData = nil
    }
    
    // Select Image Button Action
    @IBAction func selectImageButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        let alert = UIAlertController(title: "", message: "Select Source", preferredStyle: .actionSheet)
        let camera = UIAlertAction(title: "Camera", style: .default) { (action) in
            self.imagePickerController.delegate = self
            self.imagePickerController.allowsEditing = true
            if UIImagePickerController.isSourceTypeAvailable(.camera) {
                self.imagePickerController.sourceType = .camera
            } else {
                self.view.makeToast("Camera is not availabel...")
                return
            }
            self.present(self.imagePickerController, animated: true, completion: nil)
        }
        let gallery = UIAlertAction(title: "Gallery", style: .default) { (action) in
            self.imagePickerController.delegate = self
            self.imagePickerController.allowsEditing = true
            self.imagePickerController.sourceType = .photoLibrary
            self.present(self.imagePickerController, animated: true, completion: nil)
        }
        let cancel = UIAlertAction(title: "Cancel", style: .destructive, handler: nil)
        alert.addAction(camera)
        alert.addAction(gallery)
        alert.addAction(cancel)
        self.present(alert, animated: true, completion: nil)
    }
    
    // Category Button Action
    @IBAction func categoryButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        var categories: [CategoryStruct] = []
        let category = (AppInstance.instance.siteSetting?.categories?.getStringAnyData() ?? [:]).sorted { element1, element2 in
            element1.key.localizedStandardCompare(element2.key) == .orderedAscending
        }
        for (key, value) in category {
            let cat1 = CategoryStruct(cat_Name: value.htmlAttributedString ?? "", cate_id: key)
            categories.append(cat1)
        }
        if let newVC = R.storyboard.popups.categoryPopupVC() {
            newVC.delegate = self
            newVC.categories = categories
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Privacy Button Action
    @IBAction func privacyButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if let newVC = R.storyboard.popups.privacyPopupVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Age Restriction Button Action
    @IBAction func ageRestrictionButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if let newVC = R.storyboard.popups.ageRestrictionPopupVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Publish Button Action
    @IBAction func publishButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if self.mediaData == nil {
            self.view.makeToast("Please select image")
            return
        }
        if (self.videoTitleTextField.text?.isEmpty ?? false) {
            self.view.makeToast("Please write video title.")
            return
        }
        if self.descriptionTextView.text.isEmpty {
            self.view.makeToast("Please write video description.")
            return
        }
        if self.tagsTextView.text.isEmpty {
            self.view.makeToast("Please write tags.")
            return
        }
        self.editVideoApi(video_id: self.object?.id ?? 0, title: self.videoTitleTextField.text ?? "", description: self.descriptionTextView.text, tags: self.tagsTextView.text, privacy: self.privacy, category_id: self.category_id, age_restriction: self.age_restriction, thumbnail: self.mediaData)
    }
    
    // Delete Video Button Action
    @IBAction func deleteVideoButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.deleteVideoApi(id: self.object?.id ?? 0)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.textFieldSetUp()
        self.textViewSetup()
        self.setData()
    }
    
    func textFieldSetUp() {
        self.videoTitleTextField.attributedPlaceholder = NSAttributedString(
            string: "Video Title",
            attributes: [NSAttributedString.Key.foregroundColor: UIColor(named: "Label_Colors_Secondary") as Any]
        )
        self.categoryTextField.attributedPlaceholder = NSAttributedString(
            string: "Category",
            attributes: [NSAttributedString.Key.foregroundColor: UIColor(named: "Label_Colors_Secondary") as Any]
        )
        self.privacyTextField.attributedPlaceholder = NSAttributedString(
            string: "Privacy",
            attributes: [NSAttributedString.Key.foregroundColor: UIColor(named: "Label_Colors_Secondary") as Any]
        )
        self.ageRestrictionTextField.attributedPlaceholder = NSAttributedString(
            string: "Age Restriction",
            attributes: [NSAttributedString.Key.foregroundColor: UIColor(named: "Label_Colors_Secondary") as Any]
        )
    }
    
    func textViewSetup() {
        self.descriptionTextView.textContainer.lineFragmentPadding = 0
        self.descriptionTextView.textContainerInset = .zero
        self.descriptionTextView.contentInset = .zero
        self.descriptionTextView.addPlaceholder("Description", with: UIColor(named: "Label_Colors_Secondary")!)
        self.descriptionTextView.delegate = self
        
        self.tagsTextView.textContainer.lineFragmentPadding = 0
        self.tagsTextView.textContainerInset = .zero
        self.tagsTextView.contentInset = .zero
        self.tagsTextView.addPlaceholder("Tags", with: UIColor(named: "Label_Colors_Secondary")!)
        self.tagsTextView.delegate = self
    }
    
    func setData() {
        let url = URL(string: self.object?.thumbnail ?? "")
        self.videoImage.sd_setImage(with: url) { image, error, cache, url in
            if let image = image {
                self.mediaData = image.pngData()
                self.cancelButton.isHidden = false
            } else {
                self.videoImage.image = UIImage(named: "maxresdefault")
                self.cancelButton.isHidden = true
                self.mediaData = nil
            }
        }
        self.videoTitleTextField.text = self.object?.title
        self.descriptionTextView.text = self.object?.description?.stringByDecodingURL.htmlAttributedString
        self.tagsTextView.text = self.object?.tags
        self.category_id = self.object?.category_id?.stringValue ?? ""
        self.categoryTextField.text = self.object?.category_name?.htmlAttributedString ?? ""
        self.privacy = self.object?.privacy ?? 0
        self.privacyTextField.text = self.getPrivacy(privacy: self.privacy)
        self.age_restriction = self.object?.age_restriction ?? 0
        self.ageRestrictionTextField.text = self.getAgeRestriction(ageRestriction: self.age_restriction)
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
            self.descriptionTextViewHeight.constant = self.setTextViewHeight(textView: self.descriptionTextView)
            self.tagsTextViewHeight.constant = self.setTextViewHeight(textView: self.tagsTextView)
        }
    }
    
    func getPrivacy(privacy: Int) -> String {
        switch privacy {
        case 0:
            return "Public"
        case 1:
            return "Private"
        case 2:
            return "Unlisted"
        default:
            return ""
        }
    }
    
    func getAgeRestriction(ageRestriction: Int) -> String {
        switch ageRestriction {
        case 1:
            return "All ages can view this video"
        case 2:
            return "Only +18"
        default:
            return ""
        }
    }
    
    func setTextViewHeight(textView: UITextView) -> Double {
        textView.sizeToFit()
        if textView.text.isEmpty || textView.contentSize.height <= 100 {
            return 100
        } else {
            return textView.contentSize.height
        }
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension EditVideoVC {
    
    func editVideoApi(video_id: Int, title: String, description: String, tags: String, privacy: Int, category_id: String, age_restriction: Int, thumbnail: Data?) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let user_id = AppInstance.instance.userId ?? 0
            let session_token = AppInstance.instance.sessionId ?? ""
            Async.background {
                VideoManager.instance.editVideoApi(user_id: user_id, session_token: session_token, video_id: video_id, title: title, description: description, tags: tags, privacy: privacy, category_id: category_id, age_restriction: age_restriction, thumbnail: thumbnail) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.view.makeToast(success?.message)
                                self.navigationController?.popViewController(animated: true)
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func deleteVideoApi(id: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let user_id = AppInstance.instance.userId ?? 0
            let session_token = AppInstance.instance.sessionId ?? ""
            Async.background {
                VideoManager.instance.deleteVideoApi(user_id: user_id, session_token: session_token, id: id) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.view.makeToast(success?.message)
                                self.navigationController?.popViewController(animated: true)
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: UIImagePickerControllerDelegate
extension EditVideoVC: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
        let image = info[UIImagePickerController.InfoKey.originalImage] as! UIImage
        self.videoImage.image = image
        self.mediaData = image.pngData()
        self.cancelButton.isHidden = false
        self.dismiss(animated: true, completion: nil)
    }
    
}

// MARK: CategoryPopupVCDelegate
extension EditVideoVC: CategoryPopupVCDelegate {
    
    func handleCategorySelection(category: CategoryStruct) {
        self.category_id = category.cate_id
        self.categoryTextField.text = category.cat_Name
    }
    
}

// MARK: PrivacyPopupVCDelegate
extension EditVideoVC: PrivacyPopupVCDelegate {
    
    func handlePrivacyTap(privacy: Int) {
        self.privacy = privacy
        self.privacyTextField.text = self.getPrivacy(privacy: self.privacy)
    }
    
}

// MARK: AgeRestrictionPopupVCDelegate
extension EditVideoVC: AgeRestrictionPopupVCDelegate {
    
    func handleAgeRestrictionTap(ageRestriction: Int) {
        self.age_restriction = ageRestriction
        self.ageRestrictionTextField.text = self.getAgeRestriction(ageRestriction: self.age_restriction)
    }
    
}

// MARK: UITextViewDelegate
extension EditVideoVC: UITextViewDelegate {
    
    func textViewDidChange(_ textView: UITextView) {
        switch textView {
        case self.descriptionTextView:
            self.descriptionTextViewHeight.constant = self.setTextViewHeight(textView: self.descriptionTextView)
        case self.tagsTextView:
            self.tagsTextViewHeight.constant = self.setTextViewHeight(textView: self.tagsTextView)
        default:
            break
        }
    }
    
}
