//
//  FavouriteCategoryController.swift
//  Playtube
//
//  Created by Ubaid Javaid on 9/10/20.
//  Copyright © 2020 Muhammad Haris Butt. All rights reserved.
//

import UIKit

class FavouriteCategoryController: UIViewController {
    
    // MARK: - IBOutlets

    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var tableViewHeight: NSLayoutConstraint!
    
    // MARK: - Properties
    
    var fav_cat: [CategoryStruct] = []
    var delegete : FavCategoryDelegate?
    var cat_ids = [String]()
    var cate_name = [String]()
    var selectedIndex = [Int]()
    var selectIndex: Int? = nil
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
     
        self.initialConfig()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        if let categories = AppInstance.instance.siteSetting?.categories?.getStringAnyData() {
            for (key,value) in categories {
                let cat1 = CategoryStruct(cat_Name: value, cate_id: key)
                self.fav_cat.append(cat1)
            }
            self.tableView.reloadData()
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.01) {
                if self.tableView.contentSize.height > (self.view.frame.height * 0.7) {
                    self.tableViewHeight.constant = (self.view.frame.height * 0.7)
                } else {
                    self.tableViewHeight.constant = self.tableView.contentSize.height
                }
            }
        }
    }
    
    // MARK: - Selectors
    
    // Close Button Action
    @IBAction func closeButtonAction(_ sender: UIButton) {
        self.dismiss(animated: true) {
            self.delegete?.fav_cat(id: self.cat_ids, name: self.cate_name)
        }
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.view.backgroundColor = .black.withAlphaComponent(0.4)
        self.tableViewSetup()
    }
    
    // TableView Setup
    func tableViewSetup() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.register(UINib(resource: R.nib.favouriteCatCell), forCellReuseIdentifier: R.reuseIdentifier.favCatCell.identifier)
        self.tableView.separatorStyle = .none
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension FavouriteCategoryController: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.fav_cat.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let index = self.fav_cat[indexPath.row]
        let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.favCatCell.identifier) as! FavouriteCatCell
        cell.CatName.text = index.cat_Name
        if (self.selectIndex == indexPath.row) {
            if (self.selectedIndex.contains(indexPath.row)) {
                self.selectedIndex.removeAll(where: {$0 == indexPath.row})
                self.cate_name.removeAll(where: {$0 == index.cat_Name})
                self.cat_ids.removeAll(where: {$0 == index.cate_id})
                cell.checkImage.image = UIImage(named: "checkbox-1")
                cell.checkImage.tintColor = UIColor(named: "Label_Colors_Secondary")
            } else {
                self.selectedIndex.append(indexPath.row)
                self.cate_name.append(index.cat_Name)
                self.cat_ids.append(index.cate_id)
                cell.checkImage.image = UIImage(named: "ticks")
                cell.checkImage.tintColor = UIColor(named: "Primary_UI_Primary")
            }
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.selectIndex = indexPath.row
        self.tableView.reloadData()
    }
    
}
