//
//  LiveStreamVC.swift
//  Playtube
//
//  Created by iMac on 25/01/24.
//  Copyright © 2024 ScriptSun. All rights reserved.
//

import UIKit
import Async
import Toast_Swift
import AgoraRtcKit
import PlaytubeSDK
import SDWebImage

protocol LiveStreamVCDelegate {
    func handleGoLiveButtonTap()
}

class LiveStreamVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var audienceView: UIView!
    @IBOutlet weak var broadcasterView: UIView!
    @IBOutlet weak var liveView: UIView!
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var optionsView: UIView!
    @IBOutlet weak var timeLabel: UILabel!
    @IBOutlet weak var broadcasterViewsLabel: UILabel!
    @IBOutlet weak var audienceViewsLabel: UILabel!
    @IBOutlet weak var userImageView: UIImageView!
    @IBOutlet weak var commentTextField: EmojiTextField!
    @IBOutlet weak var emojiButton: UIButton!
    @IBOutlet weak var moreButton: UIButton!
    @IBOutlet weak var shareButton: UIButton!
    @IBOutlet weak var audienceEyeImageView: UIImageView!
    @IBOutlet weak var broadcasterEyeImageView: UIImageView!
    
    // MARK: - Properties
    
    var agoraKit: AgoraRtcEngineKit!
    var userRole: AgoraClientRole = .broadcaster
    var beautyOptions: AgoraBeautyOptions = {
        let bOpt = AgoraBeautyOptions()
        bOpt.smoothnessLevel = 1
        bOpt.rednessLevel = 0.1
        return bOpt
    }()
    private var seconds = 0
    private var commentTimer: Timer?
    private var secTimer: Timer?
    var post_id = 0
    var videoDetail: VideoDetail?
    var viewsCount = ""
    private var commentsArray: [Comment] = []
    var delegate: LiveStreamVCDelegate?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Close Button Action
    @IBAction func closeButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if sender.tag == 1001 {
            let warningPopupVC = R.storyboard.popups.warningPopupVC()
            warningPopupVC?.delegate = self
            warningPopupVC?.titleText  = "Ready to stop live streaming?"
            warningPopupVC?.messageText = "Tap 'Yes' to end your stream or 'No' to continue"
            warningPopupVC?.okText = "YES"
            warningPopupVC?.cancelText = "NO"
            self.present(warningPopupVC!, animated: true, completion: nil)
            warningPopupVC?.okButton.tag = sender.tag
        } else {
            let warningPopupVC = R.storyboard.popups.warningPopupVC()
            warningPopupVC?.delegate = self
            warningPopupVC?.titleText  = "Ready to stop watching?"
            warningPopupVC?.messageText = "Tap 'Yes' to exit live stream or 'No' to keep watching"
            warningPopupVC?.okText = "YES"
            warningPopupVC?.cancelText = "NO"
            self.present(warningPopupVC!, animated: true, completion: nil)
            warningPopupVC?.okButton.tag = sender.tag
        }
    }
    
    // Camera Button Action
    @IBAction func cameraButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.agoraKit?.switchCamera()
    }
    
    // Magic Button Action
    @IBAction func magicButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        sender.isSelected.toggle()
        self.agoraKit.setVoiceBeautifierPreset(
            sender.isSelected ? .timbreTransformationClear : .presetOff
        )
        self.agoraKit.setBeautyEffectOptions(sender.isSelected, options: self.beautyOptions)
        sender.tintColor = sender.isSelected ? .primary_UI_Primary : .primary_UI_Tertiary
    }
    
    // Video Button Action
    @IBAction func videoButtonAction(_ sender: UIButton) {
        sender.isSelected.toggle()
        self.agoraKit.muteLocalVideoStream(sender.isSelected)
        self.liveView.isHidden = sender.isSelected
    }
    
    // Mic Button Action
    @IBAction func micButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        sender.isSelected.toggle()
        self.agoraKit.muteLocalAudioStream(sender.isSelected)
    }
    
    // Send Button Action
    @IBAction func sendButtonAction(_ sender: UIButton) {
        if !(self.commentTextField.text?.isEmpty ?? false) {
            self.sendComment(videoID: self.post_id)
        }
    }
    
    // Emoji Button Action
    @IBAction func emojiButtonAction(_ sender: UIButton) {
        self.commentTextField.isEmojiKeyboard = !(self.commentTextField.isEmojiKeyboard)
        self.commentTextField.becomeFirstResponder()
        self.commentTextField.reloadInputViews()
        self.emojiButton.setImage(self.commentTextField.isEmojiKeyboard ? R.image.icon_keyboard_vector() : R.image.emoji_smile(), for: .normal)
    }
    
    // More Button Action
    @IBAction func moreButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if let newVC = R.storyboard.popups.liveStreamPopupVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Share Button Action
    @IBAction func shareButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.shareVideo()
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.setupUI()
        self.setupTableView()
        self.initializeAgoraEngine()
        if self.userRole == .broadcaster {
            self.createLive()
        } else {
            self.setData()
            self.joinChannel()
        }
    }
    
    // Set Data
    func setData() {
        let userImage = URL(string: self.videoDetail?.owner?.avatar ?? "")
        self.userImageView.sd_setShowActivityIndicatorView(true)
        self.userImageView.sd_setIndicatorStyle(.medium)
        DispatchQueue.global(qos: .userInteractive).async {
            self.userImageView.sd_setImage(with: userImage , placeholderImage: R.image.maxresdefault())
        }
    }
    
    // Setup UI
    func setupUI() {
        self.audienceView.isHidden = self.userRole == .broadcaster
        self.broadcasterView.isHidden = self.userRole == .audience
        self.optionsView.isHidden = self.userRole == .audience
        self.moreButton.isHidden = self.userRole == .broadcaster
        self.shareButton.isHidden = self.userRole == .broadcaster
        self.commentTextField.delegate = self
    }
    
    func initializeAgoraEngine() {
        let config = AgoraRtcEngineConfig()
        config.appId = AgoraKeyCenter.AppId
        config.channelProfile = .liveBroadcasting
        self.agoraKit = AgoraRtcEngineKit.sharedEngine(with: config, delegate: self)
        self.agoraKit.setClientRole(self.userRole)
        self.agoraKit.setDefaultAudioRouteToSpeakerphone(true)
        self.agoraKit.enableVideo()
    }
    
    // Join Channel
    func joinChannel() {
        let option = AgoraRtcChannelMediaOptions()
        option.publishCameraTrack = self.userRole == .broadcaster
        option.publishMicrophoneTrack = self.userRole == .broadcaster
        option.clientRoleType = self.userRole
        AgoraNetworkManager.shared.generateToken(channelName: "\(self.post_id)") { token in
            let result = self.agoraKit.joinChannel(byToken: token, channelId: "\(self.post_id)", uid: 0, mediaOptions: option)
            if result != 0 {
                print("joinChannel call failed: \(result), please check your params")
            }
        }
    }
    
    func leaveChannel() {
        self.agoraKit.setupLocalVideo(nil)
        self.agoraKit.leaveChannel(nil)
        if self.userRole == .broadcaster {
            self.agoraKit.stopPreview()
            self.secTimer?.invalidate()
            if let newVC = R.storyboard.live.endLiveStreamVC() {
                newVC.commnet = "\(commentsArray.count)"
                newVC.duration = timeLabel.text ?? "0"
                newVC.views = self.viewsCount
                newVC.delegate = self
                newVC.modalPresentationStyle = .overCurrentContext
                self.present(newVC, animated: true, completion: nil)
            }
        } else {
            if let newVC = R.storyboard.live.endAudienceLiveStreamVC() {
                newVC.videoDetail = self.videoDetail
                newVC.delegate = self
                newVC.modalPresentationStyle = .overCurrentContext
                self.present(newVC, animated: true, completion: nil)
            }
        }
        self.commentTimer?.invalidate()
        AgoraRtcEngineKit.destroy()
    }
    
    func setupLocalVideo() {
        let videoCanvas = AgoraRtcVideoCanvas()
        videoCanvas.uid = 0
        videoCanvas.renderMode = .hidden
        videoCanvas.view = localVideoCanvas()
        self.agoraKit.setupLocalVideo(videoCanvas)
        self.agoraKit.startPreview()
        self.agoraKit.enableAudio()
    }
    
    func localVideoCanvas() -> UIView {
        return self.liveView
    }
    
    func remoteVideoCanvas() -> UIView {
        return self.liveView
    }
    
    func setupTableView() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.separatorStyle = .none
        self.tableView.register(UINib(resource: R.nib.liveTableViewCell), forCellReuseIdentifier: R.reuseIdentifier.liveTableViewCell.identifier)
    }
    
    private func setTimers() {
        if self.userRole == .broadcaster {
            self.secTimer =  Timer.scheduledTimer(
                timeInterval: 1.0, target: self, selector: #selector(self.secondTimer),
                userInfo: nil, repeats: true)
        }
        self.commentTimer = Timer.scheduledTimer(
            timeInterval: 1.0, target: self, selector: #selector(fetchComments),
            userInfo: nil, repeats: true)
    }
    
    @objc private func secondTimer() {
        self.seconds += 1
        self.timeLabel.text = self.getSecondsString()
    }
    
    private func getSecondsString() -> String {
        let hours = self.seconds / 3600
        let minutes = self.seconds / 60 % 60
        let seconds = self.seconds % 60
        return String(format:"%02i:%02i:%02i", hours, minutes, seconds)
    }
    
    @objc private func fetchComments() {
        if self.userRole == .broadcaster {
            self.getLiveComments(postId: self.post_id, page: API.Params.live)
        } else {
            self.getLiveComments(postId: self.post_id, page: API.Params.watch)
        }
    }
    
    func shareVideo() {
        // text to share
        let someText: String = "Share Video"
        let url = self.videoDetail?.url ?? ""
        print(url)
        // set up activity view controller
        let textToShare = [ someText, url ]
        let activityViewController = UIActivityViewController(activityItems: textToShare, applicationActivities: nil)
        activityViewController.popoverPresentationController?.sourceView = self.view
        // exclude some activity types from the list (optional,)
        activityViewController.excludedActivityTypes = [
            UIActivity.ActivityType.airDrop,
            UIActivity.ActivityType.postToFacebook,
            UIActivity.ActivityType.assignToContact,
            UIActivity.ActivityType.mail,
            UIActivity.ActivityType.postToTwitter,
            UIActivity.ActivityType.message,
            UIActivity.ActivityType.postToFlickr,
            UIActivity.ActivityType.postToVimeo,
            UIActivity.ActivityType.postToTencentWeibo,
            UIActivity.ActivityType.init(rawValue: "net.whatsapp.WhatsApp.ShareExtension"),
            UIActivity.ActivityType.init(rawValue: "com.google.Gmail.ShareExtension"),
            UIActivity.ActivityType.init(rawValue: "com.toyopagroup.picaboo.share"),
            UIActivity.ActivityType.init(rawValue: "com.tinyspeck.chatlyio.share")
        ]
        activityViewController.completionWithItemsHandler = { activity, completed, items, error in
            if !completed {
                return
            } else {
                self.view.makeToast("Live stream shared successfully....")
            }
        }
        // present the view controller
        self.present(activityViewController, animated: true, completion: nil)
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension LiveStreamVC {
    
    private func createLive() {
        let number = UInt32.random(in: UInt32.min...UInt32.max)
        let channelName = "\(number)"
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        Async.background {
            GoLiveManager.instance.createLiveStream(streamName: channelName, user_id: userID, type: "create", session_token: sessionID) { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.post_id = success?.post_id ?? 0
                            self.setupLocalVideo()
                            self.joinChannel()
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text)
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.error("error = \(error?.localizedDescription ?? "")")
                        }
                    }
                }
            }
        }
    }
    
    private func getLiveComments(postId: Int, page: String) {
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        GoLiveManager.instance.checkLiveComments(userid: userID, Session_Token: sessionID, page: page, postID: self.post_id) { (success, sessionError, error) in
            if success != nil {
                self.viewsCount = "\(success?.count ?? 0)"
                if self.userRole == .broadcaster {
                    self.broadcasterEyeImageView.image = (success?.count ?? 0) < 1 ? UIImage(named: "eye_hide") : UIImage(named: "eye_show")
                    self.broadcasterEyeImageView.tintColor = (success?.count ?? 0) < 1 ? UIColor.hexStringToUIColor(hex: "#737884") : .primary_UI_Tertiary
                    self.broadcasterViewsLabel.text = "\(success?.count ?? 0) Views"
                } else {
                    self.audienceEyeImageView.image = (success?.count ?? 0) < 1 ? UIImage(named: "eye_hide") : UIImage(named: "eye_show")
                    self.audienceEyeImageView.tintColor = (success?.count ?? 0) < 1 ? UIColor.hexStringToUIColor(hex: "#737884") : .primary_UI_Tertiary
                    self.audienceViewsLabel.text = "\(success?.count ?? 0) Views"
                }
                self.fetchVideoComments(videoID: self.post_id)
            } else if sessionError != nil {
                Async.main {
                    self.dismissProgressDialog {
                        self.view.makeToast(sessionError?.errors?.error_text)
                        log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                    }
                }
            } else {
                Async.main {
                    self.dismissProgressDialog {
                        self.view.makeToast(error?.localizedDescription)
                        log.error("error = \(error?.localizedDescription ?? "")")
                    }
                }
            }
        }
    }
    
    func sendComment(videoID: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let commentText = self.commentTextField.text ?? ""
            Async.background {
                PlayVideoManager.instance.addComments(User_id: userID, Session_Token: sessionID, VideoId: videoID, Comment_Text: commentText, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.commentTextField.text  = ""
                                self.commentTextField.resignFirstResponder()
                                log.verbose("success \(success?.message ?? "") ")
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func fetchVideoComments(videoID: Int) {
        Async.background {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            PlayVideoManager.instance.fetchComments(User_id: userID, Session_Token: sessionID, VideoId: videoID, completionBlock: { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            if self.commentsArray.count != success?.data?.count {
                                self.commentsArray = success?.data ?? []
                                self.commentsArray.sort (by: { ($0.time ?? 0) < ($1.time ?? 0) })
                                self.tableView.reloadData()
                                if self.commentsArray.count != 0 {
                                    DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                                        self.tableView.scrollToLastRow(animated: false)
                                    }
                                }
                            }
                        }
                    }
                } else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text)
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                }
            })
        }
    }
    
    func deleteLive(postId: Int) {
        let sessionID = AppInstance.instance.sessionId ?? ""
        let userID = AppInstance.instance.userId ?? 0
        Async.background {
            GoLiveManager.instance.endStream(postID: postId, Session_Token: sessionID, type: "delete", userid: userID) { (success, sessionError, error) in
                if success != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.leaveChannel()
                        }
                    }
                }  else if sessionError != nil {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors?.error_text)
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        }
                    }
                } else {
                    Async.main {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription)
                            log.error("error = \(error?.localizedDescription ?? "")")
                        }
                    }
                }
            }
        }
    }
    
}

// MARK: TableView Setup
extension LiveStreamVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.commentsArray.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.liveTableViewCell.identifier, for: indexPath) as! LiveTableViewCell
        let rowPath = self.commentsArray[indexPath.row]
        cell.setData(comment: rowPath)
        return cell
    }
    
}

// MARK: AgoraRtcEngineDelegate Methods
extension LiveStreamVC: AgoraRtcEngineDelegate {
    
    func rtcEngine(_ engine: AgoraRtcEngineKit, didOccurWarning warningCode: AgoraWarningCode) {
        print("warning: \(warningCode.description)")
    }
    
    func rtcEngine(_ engine: AgoraRtcEngineKit, didJoinChannel channel: String, withUid uid: UInt, elapsed: Int) {
        print("Join \(channel) with uid \(uid) elapsed \(elapsed)ms")
        print("post_id >>>>>", self.post_id)
        self.setTimers()
    }
    
    func rtcEngine(_ engine: AgoraRtcEngineKit, didJoinedOfUid uid: UInt, elapsed: Int) {
        let videoCanvas = AgoraRtcVideoCanvas()
        videoCanvas.uid = uid
        videoCanvas.renderMode = .fit
        videoCanvas.view = remoteVideoCanvas()
        self.agoraKit.setupRemoteVideo(videoCanvas)
        print("remote user join: \(uid) \(elapsed)ms")
    }
    
    func rtcEngine(_ engine: AgoraRtcEngineKit, didOfflineOfUid uid: UInt, reason: AgoraUserOfflineReason) {
        self.dismiss(animated: true) {
            self.leaveChannel()
        }
    }
    
    func rtcEngine(_ engine: AgoraRtcEngineKit, didOccurError errorCode: AgoraErrorCode) {
        print(errorCode.description)
    }
    
}

// MARK: WarningPopupVCDelegate
extension LiveStreamVC: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        self.view.alpha = 0
        if sender.tag == 1001 {
            self.deleteLive(postId: self.post_id)
        } else {
            self.leaveChannel()
        }
    }
    
}

// MARK: EndLiveStreamVCDelegate
extension LiveStreamVC: EndLiveStreamVCDelegate {
    
    func handleDismissLiveStream() {
        self.dismiss(animated: true)
    }
    
}

// MARK: EndLiveStreamVCDelegate
extension LiveStreamVC: EndAudienceLiveStreamVCDelegate {
    
    func handleDismissEndAudienceLiveStream() {
        self.dismiss(animated: true)
    }
    
    func handleGoLiveButtonAction() {
        self.dismiss(animated: true) {
            self.delegate?.handleGoLiveButtonTap()
        }
    }
    
}

// MARK: UITextFieldDelegate Methods
extension LiveStreamVC: UITextFieldDelegate {
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        switch textField {
        case self.commentTextField:
            self.commentTextField.isEmojiKeyboard = false
            self.emojiButton.setImage(self.commentTextField.isEmojiKeyboard ? R.image.icon_keyboard_vector() : R.image.emoji_smile(), for: .normal)
        default:
            break
        }
    }
    
}

// MARK: LiveStreamPopupVCDelegate
extension LiveStreamVC: LiveStreamPopupVCDelegate {
    
    func handleCopyButtonTap() {
        if let url = self.videoDetail?.url {
            UIPasteboard.general.string = url
        }
    }
    
    func handleReportButtonTap() {
        if let newVC = R.storyboard.popups.liveStreamReportPopupVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
}

// MARK: LiveStreamReportPopupVCDelegate
extension LiveStreamVC: LiveStreamReportPopupVCDelegate {
    
    func handleReportReportButtonTap() {
        self.view.makeToast("Thak you, We have recived your report!")
    }
    
}
