//
//  NewUserChannelVC.swift
//  Playtube
//
//  Created by iMac on 25/05/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit
import Toast_Swift
import Async
import PlaytubeSDK

class NewUserChannelVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var scrollView: UIScrollView!
    @IBOutlet weak var containerView: UIView!
    @IBOutlet weak var searchButton: UIButton!
    @IBOutlet weak var chatButton: UIButton!
    @IBOutlet weak var settingButton: UIButton!
    
    // MARK: - Properties

    var navigationView = UIView()
    var headerView = UserChannelHeaderView()
    lazy var headerLabel: UILabel = {
        let label = UILabel()
        label.translatesAutoresizingMaskIntoConstraints = false
        label.text = "TopLeft"
        label.textColor = .white
        return label
    }()
    var channelData: Owner?
    var pageMenu: CAPSPageMenu!
    var userChannelVideosVC: UserChannelVideosVC = UserChannelVideosVC.userChannelVideosVC()
    var userShortsVC: UserShortsVC = UserShortsVC.userShortsVC()
    var userChannelPlaylistVC: UserChannelPlaylistVC = UserChannelPlaylistVC.userChannelPlaylistVC()
    var aboutVC: AboutVC = AboutVC.aboutVC()
    var userActivitiesVC: UserActivitiesVC = UserActivitiesVC.userActivitiesVC()
    var controllerArray: [UIViewController] = []
    var childScrollView = UIScrollView()
    var isStatusBarHidden: Bool = false {
        didSet {
            if oldValue != self.isStatusBarHidden {
                self.setNeedsStatusBarAppearanceUpdate()
            }
        }
    }
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
    
    // Search Button Action
    @IBAction func searchButtonAction(_ sender: UIButton) {
        let newVC = R.storyboard.loggedUser.searchVC()
        self.navigationController?.pushViewController(newVC!, animated: true)
    }
    
    // Chat Button Action
    @IBAction func chatButtonAction(_ sender: UIButton) {
        let vc = R.storyboard.chat.chatScreenVC()
        vc?.recipentID = channelData?.id ?? 0
        vc!.username = channelData?.username ?? ""
        vc!.userImg = channelData?.avatar ?? ""
        self.navigationController?.pushViewController(vc!, animated: true)
    }
    
    // Setting Button Action
    @IBAction func settingButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        let newVC = R.storyboard.settings.settingVC()
        self.tabBarController?.navigationController?.pushViewController(newVC!, animated: true)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.searchButton.isHidden = AppInstance.instance.userId == self.channelData?.id
        self.chatButton.isHidden = AppInstance.instance.userId == self.channelData?.id
        self.settingButton.isHidden = AppInstance.instance.userId != self.channelData?.id
        self.setupHeaderView()
        self.setupPageMenu()
        self.fetchUserProfile(Channel_id: self.channelData?.id ?? 0)
    }

    // Setup Header View
    func setupHeaderView() {
        self.scrollView.delegate = self
        self.headerView = UserChannelHeaderView(frame: CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width, height: 349))
        self.headerView.delegate = self
        self.headerView.channelData = self.channelData
        self.headerView.setData()
        self.scrollView.stickyHeader.view = headerView
        self.scrollView.stickyHeader.height = headerView.frame.height
        self.scrollView.stickyHeader.minimumHeight = 48
        
        navigationView.frame = CGRect(x: 0, y: 0, width: headerView.frame.width, height: headerView.frame.height)
        navigationView.backgroundColor = UIColor(named: "Primary_UI_Primary")
        navigationView.alpha = 0
        self.scrollView.stickyHeader.view = navigationView
        
        self.navigationView.addSubview(headerLabel)
        headerLabel.text = channelData?.username ?? ""
        headerLabel.font =  UIFont(name: "TTCommons-Medium", size: 18.0)
        headerLabel.bottomAnchor.constraint(equalTo: navigationView.bottomAnchor, constant: -10.0).isActive = true
        headerLabel.leadingAnchor.constraint(equalTo: navigationView.leadingAnchor, constant: 48.0).isActive = true
        headerLabel.heightAnchor.constraint(equalToConstant: 24.0).isActive = true
    }
    
    // setup Pagemenu
    func setupPageMenu() {
        self.userChannelVideosVC.parentContorller = self
        self.userChannelVideosVC.channelId = self.channelData?.id ?? 0
        self.userChannelVideosVC.delegate = self
        self.controllerArray.append(self.userChannelVideosVC)
        
        self.userShortsVC.parentContorller = self
        self.userShortsVC.channelId = self.channelData?.id ?? 0
        self.userShortsVC.delegate = self
        self.controllerArray.append(self.userShortsVC)
        
        self.userChannelPlaylistVC.parentContorller = self
        self.userChannelPlaylistVC.delegate = self
        self.userChannelPlaylistVC.channelID = self.channelData?.id ?? 0
        self.controllerArray.append(self.userChannelPlaylistVC)
        
        self.userActivitiesVC.parentContorller = self
        self.userActivitiesVC.delegate = self
        self.userActivitiesVC.channelID = self.channelData?.id ?? 0
        self.controllerArray.append(self.userActivitiesVC)
        
        self.aboutVC.parentContorller = self
        self.aboutVC.channelID = self.channelData?.id ?? 0
        self.aboutVC.channelData = self.channelData!
        self.controllerArray.append(self.aboutVC)
        
        let parameters: [CAPSPageMenuOption] = [
            .menuHeight(00)
        ]
        self.pageMenu = CAPSPageMenu(viewControllers: self.controllerArray, frame: CGRect(x: 0.0, y: 0.0, width: self.containerView.bounds.width, height: self.containerView.bounds.height), pageMenuOptions: parameters)
        self.pageMenu.delegate = self
        self.containerView.addSubview(self.pageMenu.view!)
        self.pageMenu.moveToPage(0)
    }
    
    func setUserData() {
        self.headerView.channelData = self.channelData
        self.headerView.setData()
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension NewUserChannelVC {
    
    func fetchUserProfile(Channel_id: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userId = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                MyChannelManager.instance.getChannelInfo(User_id: userId, Session_Token: sessionID, Channel_id: Channel_id, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.channelData = success?.data
                            self.setUserData()
                        }
                    } else if sessionError != nil {
                        Async.main {
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        Async.main {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func subscribeChannel() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let channelID = self.channelData?.id ?? 0
            Async.background {
                PlayVideoManager.instance.subUnsubChannel(User_id: userID , Session_Token: sessionID, Channel_Id: channelID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            log.verbose("success \(success?.code ?? 0) ")
                            if success?.code == 0 {
                                self.view.makeToast("Unsubscribed Successfully")
                            } else {
                                self.view.makeToast(NSLocalizedString("Subscribed Successfully", comment: "Subscribed Successfully"))
                            }
                            self.fetchUserProfile(Channel_id: self.channelData?.id ?? 0)
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors!.error_text ?? "")")
                        self.view.makeToast(sessionError?.errors!.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func payUsingWalletApi(pay_Type: String, id: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                WalletManager.instance.payUsingWalletApi(user_id: userID, session_token: sessionID, pay_Type: pay_Type, id: id) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("success \(success?.success ?? "")")
                                self.fetchUserProfile(Channel_id: self.channelData?.id ?? 0)
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: UIScrollViewDelegate
extension NewUserChannelVC: UIScrollViewDelegate {
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        let offset = scrollView.contentOffset.y
        let changeStartOffset: CGFloat = -100.0
        let changeSpeed: CGFloat = 50.0
        self.navigationView.alpha = min(1.0, (offset - changeStartOffset) / changeSpeed)
        if self.scrollView.contentOffset.y > (self.scrollView.contentSize.height - self.scrollView.bounds.height) {
            self.childScrollView.isScrollEnabled = true
        } else {
            if self.scrollView.contentOffset.y <= 0 {
                self.childScrollView.isScrollEnabled = true
            } else {
                self.childScrollView.isScrollEnabled = false
            }
        }
    }
    
}

// MARK: UserChannelHeaderViewDelegate
extension NewUserChannelVC: UserChannelHeaderViewDelegate {
    
    func handleSubscribeButtonTap(sender: UIButton) {
        if AppInstance.instance.userId == self.channelData?.id {
            self.view.endEditing(true)
            let newVC = R.storyboard.settings.editChannelVC()
            self.navigationController?.pushViewController(newVC!, animated: true)
        } else {
            if self.channelData?.am_i_subscribed == 0 {
                if self.channelData?.subscriber_price != "0" {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.titleText  = "Purchase Required"
                    warningPopupVC?.messageText = "This channel is paid, You must pay to subscribe"
                    warningPopupVC?.okText = "PURCHASE"
                    self.present(warningPopupVC!, animated: true, completion: nil)
                    warningPopupVC?.okButton.tag = 1001
                } else {
                    self.subscribeChannel()
                }
            } else {
                self.subscribeChannel()
            }
        }
    }
    
    func handleDidSelectItemAt(indexPath: IndexPath) {
        if self.pageMenu.currentPageIndex != indexPath.item {
            self.pageMenu.moveToPage(indexPath.item)
        }
    }
    
}

// MARK: WarningPopupVCDelegate
extension NewUserChannelVC: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        if sender.tag == 1001 {
            if self.channelData?.am_i_subscribed == 0 && self.channelData?.subscriber_price != "0" {
                let amount = ((self.channelData?.subscriber_price ?? "") as NSString).doubleValue
                AppInstance.instance.fetchUserProfile { success in
                    if success {
                        let walletBalance = AppInstance.instance.userProfile?.data?.wallet?.rounded() ?? 0.0
                        if amount < walletBalance {
                            self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.channelData?.id ?? 0)
                        } else {
                            let warningPopupVC = R.storyboard.popups.warningPopupVC()
                            warningPopupVC?.delegate = self
                            warningPopupVC?.titleText  = "Wallet"
                            warningPopupVC?.messageText = "Sorry, You do not have enough money please top up your wallet"
                            warningPopupVC?.okText = "ADD WALLET"
                            self.present(warningPopupVC!, animated: true, completion: nil)
                            warningPopupVC?.okButton.tag = 1002
                        }
                    }
                }
            }
            
        }
        if sender.tag == 1002 {
            let newVC = R.storyboard.settings.walletVC()
            newVC?.isPurchase = true
            newVC?.delegate = self
            self.navigationController?.pushViewController(newVC!, animated: true)
        }
    }
    
}

// MARK: WalletVCDelegate
extension NewUserChannelVC: WalletVCDelegate {
    
    func handleAddMoneyInWalletForPurchase() {
        self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.channelData?.id ?? 0)
    }
    
}

// MARK: CAPSPageMenuDelegate
extension NewUserChannelVC: CAPSPageMenuDelegate {
    
    func willMoveToPage(_ controller: UIViewController, index: Int) {
        self.headerView.selectedIndex = index
        self.headerView.collectionView.scrollToItem(at: IndexPath(item: index, section: 0), at: .centeredHorizontally, animated: true)
        self.headerView.collectionView.reloadData()
    }
    
    func didMoveToPage(_ controller: UIViewController, index: Int) {
        self.headerView.selectedIndex = index
        self.headerView.collectionView.scrollToItem(at: IndexPath(item: index, section: 0), at: .centeredHorizontally, animated: true)
        self.headerView.collectionView.reloadData()
    }
    
}

// MARK: UserChannelVideosVCDelegate
extension NewUserChannelVC: UserChannelVideosVCDelegate, UserShortsVCDelegate, UserChannelPlaylistVCDelegate, UserActivitiesVCDelegate {
    
    func scrollViewDidScroll(scrollView: UIScrollView, tableView: UITableView) {
        self.childScrollView = scrollView
        if scrollView.contentOffset.y <= 0 {
            if self.scrollView.contentOffset.y >= -48.0 {
                tableView.isScrollEnabled = false
            } else {
                if tableView.contentSize.height <= tableView.bounds.height {
                    tableView.isScrollEnabled = false
                } else {
                    tableView.isScrollEnabled = true
                }
            }
        } else {
            if self.scrollView.contentOffset.y >= (self.scrollView.contentSize.height - self.scrollView.bounds.height) {
                tableView.isScrollEnabled = true
            } else {
                tableView.isScrollEnabled = false
            }
        }
    }
    
    func videoCount(count: Int) {
        self.headerView.videosLabel.text = "\(count)"
    }
    
}

// MARK: StatusBarHiddenDelegate
extension NewUserChannelVC: StatusBarHiddenDelegate {
    
    func handleUpdate(isStatusBarHidden: Bool) {
        self.isStatusBarHidden = isStatusBarHidden
    }
    
}
