//
//  MoviesFilterVC.swift
//  Playtube
//
//  Created by Abdul Moid on 25/03/2021.
//  Copyright © 2021 Muhammad Haris Butt. All rights reserved.
//

import UIKit
import Toast_Swift
import Cosmos

class MoviesFilterVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var applyButton: RoundButton!
    @IBOutlet weak var categoryTextField: UITextField!
    @IBOutlet weak var countryTextField: UITextField!
    @IBOutlet weak var releaseTextField: UITextField!
    @IBOutlet weak var ratingView: CosmosView!
    @IBOutlet weak var searchTermTextField: UITextField!
    @IBOutlet weak var backButton: UIButton!
    
    // MARK: - Properties
    
    var sheetHeight: CGFloat = 582
    var sheetBackgroundColor: UIColor? =  UIColor(named: "System_Backgrounds_Primary")
    var sheetCornerRadius: CGFloat = 22
    private var hasSetOriginPoint = false
    private var originPoint: CGPoint?
    var parentController: MoviesNewVC?
    
    // MARK: - View Life Cycles

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    override func viewDidLayoutSubviews() {
        if !hasSetOriginPoint {
            hasSetOriginPoint = true
            originPoint = view.frame.origin
        }
    }
    
    // MARK: - Selectors
    
    // Reset Button Action
    @IBAction func resetButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.searchTermTextField.text = ""
        self.countryTextField.text = ""
        self.categoryTextField.text = ""
        self.ratingView.rating = 0
        self.releaseTextField.text = ""
    }
    
    // Release Button Action
    @IBAction func releaseButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if let newVC = R.storyboard.popups.releaseAlertVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overFullScreen
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Country Button Action
    @IBAction func countryButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if let newVC = R.storyboard.popups.locationAlertVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overFullScreen
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Category Button Action
    @IBAction func categoryButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        var array: [String] = []
        for (_, value) in (AppInstance.instance.siteSetting?.movies_categories?.getStringAnyData() ?? [:]) {
            array.append(value)
        }
        if let newVC = R.storyboard.popups.moviesCategoryPopupVC() {
            newVC.delegate = self
            newVC.categories = array
            newVC.modalPresentationStyle = .overFullScreen
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Apply Filter Button Action
    @IBAction func applyFilterButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.parentController?.isLoading = true
        self.filterData()
        self.dismiss(animated: true, completion: nil)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        view.frame.size.height = sheetHeight + self.view.safeAreaBottom
        view.isUserInteractionEnabled = true
        view.backgroundColor = sheetBackgroundColor
        view.roundCorners(corners: [.topLeft, .topRight], radius: sheetCornerRadius)
        self.setPanGesture()
    }
    
    // Set Pan Gesture
    func setPanGesture() {
        let panGesture = UIPanGestureRecognizer(target: self, action: #selector(panGestureRecognizerAction))
        view.addGestureRecognizer(panGesture)
    }
    
    // Gesture Recognizer
    @objc func panGestureRecognizerAction(sender: UIPanGestureRecognizer) {
        let translation = sender.translation(in: view)
        // Not allowing the user to drag the view upward
        guard translation.y >= 0 else { return }
        view.frame.origin = CGPoint(
            x: 0,
            y: self.originPoint!.y + translation.y
        )
        if sender.state == .ended {
            self.dismiss(animated: true, completion: nil)
        }
    }
    
    func filterData() {
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        MovieManager.instance.filterMovies(User_id: "\(userID)", pageID: "1", keyword: searchTermTextField.text ?? "", country: countryTextField.text ?? "", category: categoryTextField.text ?? "", rating: ratingView.rating, release: releaseTextField.text ?? "", Session_Token: sessionID) { (success, sessionErr, Err) in
            if success != nil {
                self.parentController?.moviesArray.removeAll()
                guard let chanels = success?.channels else { return }
                self.parentController?.moviesArray = chanels
            } else if sessionErr != nil {
                self.parentController?.view.makeToast("Error filtering movies")
            } else {
                self.parentController?.view.makeToast("Error filtering movies")
            }
            self.dismiss(animated: true) {
                self.parentController?.checkarray()
                self.parentController?.isLoading = false
            }
        }
    }
    
}

// MARK: - Extensions

// MARK: MoviesCategoryPopupVCDelegate
extension MoviesFilterVC: MoviesCategoryPopupVCDelegate {
    
    func handleMoviesCategorySelection(category: String) {
        self.categoryTextField.text = category
    }
    
}

// MARK: LocationAlertVCDelegate Methods
extension MoviesFilterVC: LocationAlertVCDelegate {
    
    func handleCountrySelection(country: CountryModel) {
        self.countryTextField.text = country.name
    }
    
}

// MARK: LocationAlertVCDelegate Methods
extension MoviesFilterVC: ReleaseAlertVCDelegate {
    
    func handleReleaseSelection(release: String) {
        self.releaseTextField.text = release
    }
    
}
