import UIKit
import PlaytubeSDK
import GoogleMobileAds
import Async
import Toast_Swift

class NotificationVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var showStack: UIStackView!
    
    // MARK: - Properties
    
    var interstitial: GADInterstitialAd!
    private var notificationsArray: [NotificationsModel.Notification] = []
    var isStatusBarHidden: Bool = false {
        didSet {
            if oldValue != self.isStatusBarHidden {
                self.setNeedsStatusBarAppearanceUpdate()
            }
        }
    }
    private var isLoading = true {
        didSet {
            self.tableView.reloadData()
        }
    }
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.setupUI()
        self.isLoading = true
        self.getNotification()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
    
    // MARK: - Helper Functions
    
    private func setupUI() {
        if AppSettings.shouldShowAddMobBanner {
            let request = GADRequest()
            GADInterstitialAd.load(withAdUnitID: AppSettings.interestialAddUnitId, request: request, completionHandler: { (ad, error) in
                if let error = error {
                    print("Failed to load interstitial ad with error: \(error.localizedDescription)")
                    return
                }
                self.interstitial = ad
            })
        }
        self.tableView.separatorStyle = .none
        tableView.register(UINib(resource: R.nib.notificationTableItem), forCellReuseIdentifier: R.reuseIdentifier.notificationTableItem.identifier)
    }
    
    func CreateAd() -> GADInterstitialAd {
        GADInterstitialAd.load(withAdUnitID: AppSettings.interestialAddUnitId, request: GADRequest(), completionHandler: { (ad, error) in
            if let error = error {
                print("Failed to load interstitial ad with error: \(error.localizedDescription)")
                return
            }
            self.interstitial = ad
        })
        return  self.interstitial
    }
    
    private func getNotification() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                NotificationsManager.instance.getNotifications(User_id: userID, Session_Token: sessionID, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.debug("success")
                                self.notificationsArray = (success?.notifications)!
                                self.isLoading = false
                                if !self.notificationsArray.isEmpty {
                                    self.tableView.reloadData()
                                    self.showStack.isHidden = true
                                    self.tableView.isHidden = false
                                } else {
                                    self.showStack.isHidden = false
                                    self.tableView.isHidden = true
                                }
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.debug("sessionError = \(sessionError?.errors?.error_text ?? "")")
                                self.view.makeToast(sessionError?.errors?.error_text)
                                self.isLoading = false
                            }
                        }
                    } else {
                        Async.main {
                            self.dismissProgressDialog {
                                log.debug("error = \(error?.localizedDescription ?? "")")
                                self.view.makeToast(error?.localizedDescription)
                                self.isLoading = false
                            }
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension NotificationVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if self.isLoading {
            return 10
        } else {
            return self.notificationsArray.count
        }        
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if self.isLoading {
            let cell = tableView.dequeueReusableCell(withIdentifier:R.reuseIdentifier.notificationTableItem.identifier) as! NotificationTableItem
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier:R.reuseIdentifier.notificationTableItem.identifier) as! NotificationTableItem
            let object = self.notificationsArray[indexPath.row]
            cell.bind(object)
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if !self.isLoading {
            let notificationObject = self.notificationsArray[indexPath.row]
            if (notificationObject.title?.contains("added"))! || (notificationObject.title?.contains("disliked"))! || (notificationObject.title?.contains("liked"))! || (notificationObject.title?.contains("commented"))! || (notificationObject.title?.contains("commented"))! {
                if AppInstance.instance.addCount == AppSettings.interestialCount {
                    interstitial.present(fromRootViewController: self)
                    interstitial = CreateAd()
                    AppInstance.instance.addCount = 0
                }
                AppInstance.instance.addCount = AppInstance.instance.addCount! + 1
                if let videoObject = self.notificationsArray[indexPath.row].video?.videoClass {
                    let newVC = self.tabBarController as! TabbarController
                    newVC.statusBarHiddenDelegate = self
                    newVC.handleOpenVideoPlayer(for: videoObject)
                } else {
                    let vc = R.storyboard.library.newUserChannelVC()
                    vc?.channelData = self.notificationsArray[indexPath.row].userData
                    self.navigationController?.pushViewController(vc!, animated: true)
                }
            } else {
                if let newVC = R.storyboard.library.newUserChannelVC() {
                    newVC.channelData = self.notificationsArray[indexPath.row].userData
                    self.navigationController?.pushViewController(newVC, animated: true)
                }
            }
        }
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        cell.setTemplateWithSubviews(isLoading, animate: true, viewBackgroundColor: .systemBackground)
    }
    
}

// MARK: StatusBarHiddenDelegate
extension NotificationVC: StatusBarHiddenDelegate {
    
    func handleUpdate(isStatusBarHidden: Bool) {
        self.isStatusBarHidden = isStatusBarHidden
    }
    
}
