//
//  PopularChannelController.swift
//  Playtube
//
//  Created by Ubaid Javaid on 9/7/20.
//  Copyright © 2020 Muhammad Haris Butt. All rights reserved.
//

import UIKit
import PlaytubeSDK
import SDWebImage
import UIView_Shimmer
import FittedSheets
import Refreshable
import Toast_Swift
import Async

class PopularChannelController: BaseVC, ChannelFilterDelegate {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    
    // MARK: - Properties
    
    private static var pageSize = 15
    private var currentPage = 0
    private var lastPage = 0
    var channels: [Channels] = []
    var controller : SheetViewController?
    private var isLoading = true {
        didSet {
            tableView.reloadData()
        }
    }
    var type = "views"
    var sort = "all_time"
    var ownerObject: Owner?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        PopularChannelController.pageSize = 15
        self.currentPage = 0
        self.lastPage = 0
        self.isLoading = true
        self.getChannels()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
    
    // Filter Button Action
    @IBAction func filterButtonAction(_ sender: UIButton) {        
        if let newVC = R.storyboard.loggedUser.channelFilterVC() {
            newVC.sortStr = self.type
            newVC.timeStr = self.sort
            newVC.delegate = self
            newVC.modalPresentationStyle = .custom
            newVC.transitioningDelegate = self
            self.present(newVC, animated: true)
        }
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.registerCell()
    }
    
    // Register Cell
    func registerCell() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        if #available(iOS 10.0, *) {
            tableView.prefetchDataSource = self
        }
        self.tableView.tableFooterView = UIView()
        self.tableView.register(UINib(resource: R.nib.popularCell), forCellReuseIdentifier: R.reuseIdentifier.popularCell.identifier)
        self.tableView.addPullRefresh { [weak self] in
            PopularChannelController.pageSize = 15
            self?.currentPage = 0
            self?.lastPage = 0
            self?.isLoading = true
            self?.getChannels()
        }
    }
    
    private func getChannelsLoadMore(indx: Int) {
        if (Connectivity.isConnectedToNetwork()) {
            let targetCount = currentPage < 0 ? 0 : (currentPage + 1) * PopularChannelController.pageSize - 10
            guard indx == targetCount else {
                return
            }
            currentPage += 1
            let arrIds = NSMutableArray()
            for data in self.channels {
                if let user_data = data.user_data {
                    if let id = user_data.id {
                        arrIds.add(id)
                    }
                }
            }
            let strIds = arrIds.componentsJoined(by: ",")
            var lastCount = 0
            switch self.channels[self.channels.count-1].count {
            case .integer(let value as NSNumber):
                lastCount = value.intValue
            case .string(let value as NSString):
                lastCount = value.integerValue
            default:
                break
            }
            //self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            GetPopularPostManager.sharedInstance.getPopularPost(type: self.type, sortType: self.sort, limit: 15, lastCount: lastCount, channelIds: strIds) { (success, authError, error) in
                if (success != nil) {
                    self.lastPage += 1
                    DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
                        self.tableView.beginUpdates()
                        for i in success?.channels ?? [] {
                            self.tableView.insertRows(at: [IndexPath(row: self.channels.count, section: 0)], with: .automatic)
                            self.channels.append(i)
                        }
                        self.tableView.endUpdates()
                    }
                } else if (authError != nil) {
                    self.isLoading = false
                    self.view.makeToast(authError?.errors?.error_text)
                } else if (error != nil) {
                    self.isLoading = false
                    self.view.makeToast(error?.localizedDescription)
                }
            }
            
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    private func getChannels() {
        if (Connectivity.isConnectedToNetwork()) {
            //self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            GetPopularPostManager.sharedInstance.getPopularPost(type: self.type, sortType: self.sort, limit: 15, lastCount: 0, channelIds: "") { (success, authError, error) in
                if (success != nil) {
                    self.dismissProgressDialog {
                        self.channels = []
                        self.channels = success?.channels ?? []
                        self.isLoading = false
                        self.tableView.stopPullRefreshEver()
                    }
                } else if (authError != nil) {
                    self.dismissProgressDialog {
                        self.view.makeToast(authError?.errors?.error_text)
                    }
                } else if (error != nil) {
                    self.dismissProgressDialog {
                        self.view.makeToast(error?.localizedDescription)
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func filter(sort_by: String, time_by: String) {
        self.type = sort_by
        self.sort = time_by
        self.channels.removeAll()
        self.isLoading = true
        PopularChannelController.pageSize = 15
        self.currentPage = 0
        self.lastPage = 0
        self.getChannels()
    }
    
    func viewModel(at index: Int) -> Channels? {
        guard index >= 0 && index < self.channels.count else { return nil }
        self.getChannelsLoadMore(indx : index)
        return self.channels[index]
    }
    
}

// MARK: - Extensions

// MARK: Api call
extension PopularChannelController {
    
    func subscribeChannel() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let channelID = self.ownerObject?.id ?? 0
            Async.background {
                PlayVideoManager.instance.subUnsubChannel(User_id: userID, Session_Token: sessionID, Channel_Id: channelID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            if success?.code == 0 {
                                self.view.makeToast("Unsubscribed Successfully")
                            } else {
                                self.view.makeToast(NSLocalizedString("Subscribed Successfully", comment: "Subscribed Successfully"))
                            }
                            self.currentPage = 0
                            self.lastPage = 0
                            self.getChannels()
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors!.error_text ?? "")")
                        obj_appDelegate.window?.rootViewController?.view.makeToast(sessionError?.errors!.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        obj_appDelegate.window?.rootViewController?.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            obj_appDelegate.window?.rootViewController?.view.makeToast("InterNetError")
        }
    }
    
    func payUsingWalletApi(pay_Type: String, id: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                WalletManager.instance.payUsingWalletApi(user_id: userID, session_token: sessionID, pay_Type: pay_Type, id: id) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("success \(success?.success ?? "")")
                                self.currentPage = 0
                                self.lastPage = 0
                                self.getChannels()
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: TableView Setup
extension PopularChannelController: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if isLoading {
            return 10
        } else {
            return self.channels.count
        }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if isLoading {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.popularCell.identifier) as! PopularCell
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.popularCell.identifier) as! PopularCell
            if let index = self.viewModel(at: indexPath.row) {
                guard let object = index.user_data else { return cell }
                cell.bind(object)
            }
            cell.delegate = self
            cell.indexPath = indexPath
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if !isLoading {
            if let newVC = R.storyboard.library.newUserChannelVC() {
                newVC.channelData = self.channels[indexPath.row].user_data
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        }
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        cell.setTemplateWithSubviews(isLoading, animate: true, viewBackgroundColor: .systemBackground)
    }
    
}

// MARK: UITableViewDataSourcePrefetching
extension PopularChannelController: UITableViewDataSourcePrefetching {
    
    func tableView(_ tableView: UITableView, prefetchRowsAt indexPaths: [IndexPath]) {
        for indexPath in indexPaths {
            if self.channels.count == 0 {
                return
            }
            if let _ = self.viewModel(at: (indexPath as NSIndexPath).row) {
                print(String.init(format: "prefetchRowsAt #%i", indexPath.row))
            }
        }
    }
    
}

// MARK: PopularCellDelegate
extension PopularChannelController: PopularCellDelegate {
    
    func handleSubscribeButtonTap(indexPath: IndexPath) {
        self.ownerObject = self.viewModel(at: indexPath.row)?.user_data
        if self.ownerObject?.am_i_subscribed == 0 {
            if self.ownerObject?.subscriber_price != "0" {
                let warningPopupVC = R.storyboard.popups.warningPopupVC()
                warningPopupVC?.delegate = self
                warningPopupVC?.titleText  = "Purchase Required"
                warningPopupVC?.messageText = "This channel is paid, You must pay to subscribe"
                warningPopupVC?.okText = "PURCHASE"
                self.present(warningPopupVC!, animated: true, completion: nil)
                warningPopupVC?.okButton.tag = 1001
            } else {
                self.subscribeChannel()
            }
        } else {
            self.subscribeChannel()
        }
    }
    
}

// MARK: WarningPopupVCDelegate
extension PopularChannelController: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        if sender.tag == 1001 {
            if self.ownerObject?.am_i_subscribed == 0 && self.ownerObject?.subscriber_price != "0" {
                let amount = ((self.ownerObject?.subscriber_price ?? "") as NSString).doubleValue
                AppInstance.instance.fetchUserProfile { success in
                    if success {
                        let walletBalance = AppInstance.instance.userProfile?.data?.wallet?.rounded() ?? 0.0
                        if amount < walletBalance {
                            self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.ownerObject?.id ?? 0)
                        } else {
                            let warningPopupVC = R.storyboard.popups.warningPopupVC()
                            warningPopupVC?.delegate = self
                            warningPopupVC?.titleText  = "Wallet"
                            warningPopupVC?.messageText = "Sorry, You do not have enough money please top up your wallet"
                            warningPopupVC?.okText = "ADD WALLET"
                            self.present(warningPopupVC!, animated: true, completion: nil)
                            warningPopupVC?.okButton.tag = 1002
                        }
                    }
                }
            }
            
        }
        if sender.tag == 1002 {
            let newVC = R.storyboard.settings.walletVC()
            newVC?.isPurchase = true
            newVC?.delegate = self
            self.navigationController?.pushViewController(newVC!, animated: true)
        }
    }
    
}

// MARK: WalletVCDelegate
extension PopularChannelController: WalletVCDelegate {
    
    func handleAddMoneyInWalletForPurchase() {
        self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.ownerObject?.id ?? 0)
    }
    
}

// MARK: UIViewControllerTransitioningDelegate Methods
extension PopularChannelController: UIViewControllerTransitioningDelegate {
    
    func presentationController(forPresented presented: UIViewController, presenting: UIViewController?, source: UIViewController) -> UIPresentationController? {
        BottomSheetPresentationController(presentedViewController: presented, presenting: presenting)
    }
    
}
