//
//  PopularChannelFilterController.swift
//  Playtube
//
//  Created by Ubaid Javaid on 9/8/20.
//  Copyright © 2020 Muhammad Haris Butt. All rights reserved.
//

import UIKit

class PopularChannelFilterController: UIViewController {
    
    // MARK: - IBOutlets

    @IBOutlet weak var applyFilterButton: UIButton!
    @IBOutlet weak var subscribersButton: UIButton!
    @IBOutlet weak var viewsButton: UIButton!
    @IBOutlet weak var mostActiveButton: UIButton!
    @IBOutlet weak var todayButton: UIButton!
    @IBOutlet weak var thisWeekButton: UIButton!
    @IBOutlet weak var thisMonthButton: UIButton!
    @IBOutlet weak var thisYearButton: UIButton!
    @IBOutlet weak var allTimeButton: UIButton!
    
    // MARK: - Properties
    
    var delegate: ChannelFilterDelegate?
    var sortStr = "views"
    var timeStr = "today"
    var sheetHeight: CGFloat = 422
    var sheetBackgroundColor: UIColor? =  UIColor(named: "System_Backgrounds_Primary")
    var sheetCornerRadius: CGFloat = 22
    private var hasSetOriginPoint = false
    private var originPoint: CGPoint?
    
    // MARK: - View Life Cycles

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    override func viewDidLayoutSubviews() {
        if !hasSetOriginPoint {
            hasSetOriginPoint = true
            originPoint = view.frame.origin
        }
    }
    
    // MARK: - Selectors
    
    // Today Button Action
    @IBAction func todayButtonAction(_ sender: UIButton) {
        self.timeBackgroundClear()
        self.timeStr = "today"
        self.setSelectedButton(sender: sender)
    }
    
    // This Week Button Action
    @IBAction func thisWeekButtonAction(_ sender: UIButton) {
        self.timeBackgroundClear()
        self.timeStr = "this_week"
        self.setSelectedButton(sender: sender)
    }
    
    // This Month Button Action
    @IBAction func thisMonthButtonAction(_ sender: UIButton) {
        self.timeBackgroundClear()
        self.timeStr = "this_month"
        self.setSelectedButton(sender: sender)
    }
    
    // This Year Button Action
    @IBAction func thisYearButtonAction(_ sender: UIButton) {
        self.timeBackgroundClear()
        self.timeStr = "this_year"
        self.setSelectedButton(sender: sender)
    }
    
    // All Time Button Action
    @IBAction func allTimeButtonAction(_ sender: UIButton) {
        self.timeBackgroundClear()
        self.timeStr = "all_time"
        self.setSelectedButton(sender: sender)
    }
    
    // Reset Button Action
    @IBAction func resetButtonAction(_ sender: UIButton) {
        self.sortBackgroundClear()
        self.timeBackgroundClear()
        self.sortStr = "views"
        self.timeStr = "today"
        self.setSelectedButton(sender: self.viewsButton)
        self.setSelectedButton(sender: self.todayButton)
    }
    
    // Views Button Action
    @IBAction func viewsButtonAction(_ sender: UIButton) {
        self.sortBackgroundClear()
        self.sortStr = "views"
        self.setSelectedButton(sender: sender)
    }
    
    // Subscribers Button Action
    @IBAction func subscribersButtonAction(_ sender: UIButton) {
        self.sortBackgroundClear()
        self.sortStr = "subscribers"
        self.setSelectedButton(sender: sender)
    }
    
    // Most Active Button Action
    @IBAction func mostActiveButtonAction(_ sender: UIButton) {
        self.sortBackgroundClear()
        self.sortStr = "most_active"
        self.setSelectedButton(sender: sender)
    }
    
    // Apply Filter Button Action
    @IBAction func applyFilterButtonAction(_ sender: UIButton) {
        self.dismiss(animated: true) {
            self.delegate?.filter(sort_by: self.sortStr, time_by: self.timeStr)
        }
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        view.frame.size.height = sheetHeight + self.view.safeAreaBottom
        view.isUserInteractionEnabled = true
        view.backgroundColor = sheetBackgroundColor
        view.roundCorners(corners: [.topLeft, .topRight], radius: sheetCornerRadius)
        self.setPanGesture()
        self.setData()
    }
    
    func setData() {
        if self.sortStr == "views" {
            self.viewsButtonAction(self.viewsButton)
        } else if self.sortStr == "subscribers" {
            self.subscribersButtonAction(self.subscribersButton)
        } else if self.sortStr == "most_active" {
            self.mostActiveButtonAction(self.mostActiveButton)
        }
        
        if self.timeStr == "today" {
            self.todayButtonAction(self.todayButton)
        } else if self.timeStr == "this_week" {
            thisWeekButtonAction(self.thisWeekButton)
        } else if self.timeStr == "this_month" {
            self.thisMonthButtonAction(self.thisMonthButton)
        } else if self.timeStr == "this_year" {
            self.thisYearButtonAction(self.thisYearButton)
        } else if self.timeStr == "all_time" {
            self.allTimeButtonAction(self.allTimeButton)
        }
    }
    
    // Set Pan Gesture
    func setPanGesture() {
        let panGesture = UIPanGestureRecognizer(target: self, action: #selector(panGestureRecognizerAction))
        view.addGestureRecognizer(panGesture)
    }
    
    // Gesture Recognizer
    @objc func panGestureRecognizerAction(sender: UIPanGestureRecognizer) {
        let translation = sender.translation(in: view)
        // Not allowing the user to drag the view upward
        guard translation.y >= 0 else { return }
        view.frame.origin = CGPoint(
            x: 0,
            y: self.originPoint!.y + translation.y
        )
        if sender.state == .ended {
            self.dismiss(animated: true, completion: nil)
        }
    }
    
    func setSelectedButton(sender: UIButton) {
        sender.backgroundColor = UIColor(named: "Primary_UI_Primary")
        sender.setTitleColor(UIColor(named: "Primary_UI_Tertiary"), for: .normal)
    }
    
    func setUnSelectedButton(sender: UIButton) {
        sender.backgroundColor = UIColor(named: "Primary_UI_Primary")?.withAlphaComponent(0.1)
        sender.setTitleColor(UIColor(named: "Primary_UI_Primary"), for: .normal)
    }
    
    func sortBackgroundClear() {
        self.setUnSelectedButton(sender: self.viewsButton)
        self.setUnSelectedButton(sender: self.subscribersButton)
        self.setUnSelectedButton(sender: self.mostActiveButton)
    }
    
    func timeBackgroundClear() {
        self.setUnSelectedButton(sender: self.todayButton)
        self.setUnSelectedButton(sender: self.thisWeekButton)
        self.setUnSelectedButton(sender: self.thisMonthButton)
        self.setUnSelectedButton(sender: self.thisYearButton)
        self.setUnSelectedButton(sender: self.allTimeButton)
    }
    
}
