import UIKit
import Async
import Toast_Swift

class EditChannelVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var profileImage: UIImageView!
    @IBOutlet weak var coverImage: UIImageView!
    @IBOutlet weak var usernameTextField: UITextField!
    @IBOutlet weak var fullNameTextField: UITextField!
    @IBOutlet weak var emailTextField: UITextField!
    @IBOutlet weak var aboutTextField: UITextField!
    @IBOutlet weak var favouriteCategoryTextField: UITextField!
    @IBOutlet weak var genderTextField: UITextField!
    @IBOutlet weak var ageTextField: UITextField!
    @IBOutlet weak var countryTextField: UITextField!
    @IBOutlet weak var facebookTextField: UITextField!
    @IBOutlet weak var twitterTextField: UITextField!
    
    // MARK: - Properties
    
    private var genderString: String? = ""
    private var imagePicker = UIImagePickerController()
    private var imageStatus = false
    private var avatarImage: UIImage? = nil
    private var coverImageVar: UIImage? = nil
    private var cate_ids = [String]()
    var is_Setting = 0
    var age = ""
    var country_id = ""
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.setupUI()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // Gender Button Action
    @IBAction func genderButtonAction(_ sender: UIButton) {
        if let newVC = R.storyboard.popups.genderAlertVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Change Avatar Button Action
    @IBAction func changeAvatarButtonAction(_ sender: UIButton) {
        self.imageStatus = false
        if let newVC = R.storyboard.popups.imageAlertVC() {
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            newVC.delegate = self
            self.present(newVC, animated: true)
        }
    }
    
    // Change Cover Button Action
    @IBAction func changeCoverButtonAction(_ sender: UIButton) {
        self.imageStatus = true
        if let newVC = R.storyboard.popups.imageAlertVC() {
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            newVC.delegate = self
            self.present(newVC, animated: true)
        }
    }
    
    // Category Button Action
    @IBAction func categoryButtonAction(_ sender: UIButton) {
        if let newVC = R.storyboard.settings.favouriteCateVC() {
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            newVC.delegete = self
            self.present(newVC, animated: true)
        }
    }
    
    // Age Button Action
    @IBAction func ageButtonAction(_ sender: UIButton) {
        if let newVC = R.storyboard.popups.ageAlertVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Country Button Action
    @IBAction func countryButtonAction(_ sneder: UIButton) {
        self.view.endEditing(true)
        if let newVC = R.storyboard.popups.locationAlertVC() {
            newVC.delegate = self
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            self.present(newVC, animated: true)
        }
    }
    
    // Save Button Action
    @IBAction func saveButtonAction(_ sender: UIButton) {
        log.verbose("savePressed!!")
        self.updateMyChannel()
    }
    
    // MARK: - Helper Functions
    
    private func setupUI() {
        self.usernameTextField.text = AppInstance.instance.userProfile?.data?.username ?? ""
        self.fullNameTextField.text = AppInstance.instance.userProfile?.data?.name ?? ""
        self.emailTextField.text = AppInstance.instance.userProfile?.data?.email ?? ""
        self.aboutTextField.text = AppInstance.instance.userProfile?.data?.about ?? ""
        self.genderTextField.text = AppInstance.instance.userProfile?.data?.gender_text ?? ""
        if let age = AppInstance.instance.userProfile?.data?.age {
            self.age = "\(age)"
        }
        self.ageTextField.text = "\(AppInstance.instance.userProfile?.data?.age ?? 0)"
        if let country_id = AppInstance.instance.userProfile?.data?.country_id {
            self.country_id = "\(country_id)"
        }
        self.countryTextField.text = AppInstance.instance.userProfile?.data?.country_name ?? ""
        self.facebookTextField.text = AppInstance.instance.userProfile?.data?.facebook ?? ""
        self.twitterTextField.text = AppInstance.instance.userProfile?.data?.twitter ?? ""
        self.profileImage.sd_setImage(with: URL(string: AppInstance.instance.userProfile?.data?.avatar ?? ""))
        self.coverImage.sd_setImage(with: URL(string: AppInstance.instance.userProfile?.data?.cover ?? ""))
        log.verbose("Gender = \(AppInstance.instance.userProfile?.data?.gender_text ?? "")")
    }
    
    private func updateMyChannel() {
        self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
        let name = self.fullNameTextField.text ?? ""
        let gender = self.genderString  ?? ""
        let username = self.usernameTextField.text ?? ""
        let email = self.emailTextField.text ?? ""
        let about = self.aboutTextField.text ?? ""
        let facebook = self.facebookTextField.text ?? ""
        let twitter = self.twitterTextField.text ?? ""
        let userId = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        Async.background {
            if self.avatarImage == nil && self.coverImageVar == nil {
                MyChannelManager.instance.editMyChannel(user_id: userId, session_token: sessionID, username: username, name: name, email: email, about: about, gender: gender, age: self.age, country: self.country_id, facebook: facebook, twitter: twitter, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.debug("success = \(success?.message ?? "")")
                                self.view.makeToast(success?.message ?? "")
                                AppInstance.instance.fetchUserProfile { (success) in
                                    if (success) {
                                        self.navigationController?.popViewController(animated: true)
                                    } else{
                                        print("Profile Error")
                                    }
                                }
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                                self.view.makeToast(sessionError?.errors?.error_text ?? "")
                            }
                        }
                    } else {
                        Async.main {
                            self.dismissProgressDialog {
                                log.error("error = \(error?.localizedDescription ?? "")")
                                self.view.makeToast(error?.localizedDescription ?? "")
                            }
                        }
                    }
                })
            } else {
                if self.avatarImage != nil && self.coverImageVar == nil {
                    let avatarData = self.avatarImage?.jpegData(compressionQuality: 0.2)
                    MyChannelManager.instance.updateUserDataWithCandA(user_id: userId, session_token: sessionID, username: username, name: name, email: email, about: about, gender: gender, age: self.age, country: self.country_id, facebook: facebook, twitter: twitter, type: "avatar", avatar_data: avatarData, cover_data: nil, completionBlock: { (success, sessionError, error) in
                        if success != nil {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.debug("success = \(success?.message ?? "")")
                                    self.view.makeToast(success?.message ?? "")
                                    AppInstance.instance.fetchUserProfile { (success) in
                                        if (success){
                                            self.navigationController?.popViewController(animated: true)
                                        } else {
                                            print("Profile Error")
                                        }
                                    }
                                }
                            }
                        } else if sessionError != nil {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                                    self.view.makeToast(sessionError?.errors?.error_text ?? "")
                                }
                            }
                        } else {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.error("error = \(error?.localizedDescription ?? "")")
                                    self.view.makeToast(error?.localizedDescription ?? "")
                                }
                            }
                        }
                    })
                } else if self.avatarImage == nil && self.coverImageVar != nil {
                    let coverImageData = self.coverImageVar?.jpegData(compressionQuality: 0.2)
                    MyChannelManager.instance.updateUserDataWithCandA(user_id: userId, session_token: sessionID, username: username, name: name, email: email, about: about, gender: gender, age: self.age, country: self.country_id, facebook: facebook, twitter: twitter, type: "cover", avatar_data: nil, cover_data: coverImageData, completionBlock: { (success, sessionError, error) in
                        if success != nil {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.debug("success = \(success?.message ?? "")")
                                    self.view.makeToast(success?.message ?? "")
                                    AppInstance.instance.fetchUserProfile { (success) in
                                        if (success) {
                                            self.navigationController?.popViewController(animated: true)
                                        } else {
                                            print("Profile Error")
                                        }
                                    }
                                }
                            }
                        } else if sessionError != nil {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                                    self.view.makeToast(sessionError?.errors?.error_text ?? "")
                                }
                            }
                        } else {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.error("error = \(error?.localizedDescription ?? "")")
                                    self.view.makeToast(error?.localizedDescription ?? "")
                                }
                            }
                        }
                    })
                } else {
                    let avatarData = self.avatarImage?.jpegData(compressionQuality: 0.2)
                    let coverData = self.coverImageVar?.jpegData(compressionQuality: 0.2)
                    MyChannelManager.instance.updateUserDataWithCandA(user_id: userId, session_token: sessionID, username: username, name: name, email: email, about: about, gender: gender, age: self.age, country: self.country_id, facebook: facebook, twitter: twitter, type: "all", avatar_data: avatarData, cover_data: coverData, completionBlock: { (success, sessionError, error) in
                        if success != nil {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.debug("success = \(success?.message ?? "")")
                                    self.view.makeToast(success?.message ?? "")
                                    AppInstance.instance.fetchUserProfile { (success) in
                                        if (success) {
                                            self.navigationController?.popViewController(animated: true)
                                        } else {
                                            print("Profile Error")
                                        }
                                    }
                                }
                            }
                        } else if sessionError != nil {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                                    self.view.makeToast(sessionError?.errors?.error_text ?? "")
                                }
                            }
                        } else {
                            Async.main {
                                self.dismissProgressDialog {
                                    log.error("error = \(error?.localizedDescription ?? "")")
                                    self.view.makeToast(error?.localizedDescription ?? "")
                                }
                            }
                        }
                    })
                }
            }
        }
    }
    
}

// MARK: - Extensions

// MARK: ImageAlertVCDelegate Methods
extension EditChannelVC: ImageAlertVCDelegate {
    
    func handleImageGalleryTap(_ sender: UIButton) {
        self.setupImagePicker(imagePickerSourceType: .PHOTO_LIBRARY)
    }
    
    func handleTakePictureFromCameraTap(_ sender: UIButton) {
        self.setupImagePicker(imagePickerSourceType: .CAMERA)
    }
    
    func setupImagePicker(imagePickerSourceType: ImagePickerSourceType) {
        self.view.endEditing(true)
        self.imagePicker.delegate = self
        switch imagePickerSourceType {
        case .CAMERA:
            if UIImagePickerController.isSourceTypeAvailable(.camera) {
                self.imagePicker.sourceType = .camera
            } else {
                if let messageAlertVC = R.storyboard.popups.warningPopupVC() {
                    messageAlertVC.messageText = "You don't have camera"
                    self.present(messageAlertVC, animated: true, completion: nil)
                    messageAlertVC.okButton.isHidden = true
                    return
                }
            }
        case .PHOTO_LIBRARY:
            self.imagePicker.sourceType = .photoLibrary
        }
        self.imagePicker.mediaTypes = ["public.image"]
        self.imagePicker.allowsEditing = true
        self.present(self.imagePicker, animated: true, completion: nil)
    }
    
}

// MARK: UIImagePickerControllerDelegate Methods
extension EditChannelVC: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
        picker.dismiss(animated: true) {
            if let image = info[.originalImage] as? UIImage {
                if self.imageStatus {
                    self.coverImageVar = image
                    self.coverImage.image = image
                } else {
                    self.avatarImage  = image
                    self.profileImage.image = image
                }
            }
        }
    }
    
}

// MARK: GenderAlertVCDelegate Methods
extension EditChannelVC: GenderAlertVCDelegate {
    
    func handleGenderTap(gender: String) {
        self.genderString = gender.lowercased()
        self.genderTextField.text = gender
    }
    
}

// MARK: FavCategoryDelegate
extension EditChannelVC: FavCategoryDelegate {
    
    func fav_cat(id: [String], name: [String]) {
        var cat_names = ""
        for i in name {
            cat_names.append("\(i),")
        }
        self.favouriteCategoryTextField.text = cat_names
        self.cate_ids = id
    }
    
}

// MARK: LocationAlertVCDelegate Methods
extension EditChannelVC: LocationAlertVCDelegate {
    
    func handleCountrySelection(country: CountryModel) {
        self.country_id = country.id
        self.countryTextField.text = country.name
    }
    
}

// MARK: LocationAlertVCDelegate Methods
extension EditChannelVC: AgeAlertVCDelegate {
    
    func handleAgeSelection(age: String) {
        self.age = age
        self.ageTextField.text = age
    }
    
}
