import UIKit
import AVKit
import Async
import PlaytubeSDK
import Toast_Swift

class SettingVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    
    // MARK: - Properties
    
    var settings: [SettingModel] = []
    var pause_history = 0
    var picture_In_Picture = false
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    @IBAction func btnBack(_ sender: Any) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.tableViewSetup()
        self.picture_In_Picture = UserDefaults.standard.getPictureInPicture(Key: Local.GET_SETTINGS.picture_In_Picture)
        self.getPauseHistory()
        self.setSettingsData()
    }
    
    // TableView Setup
    func tableViewSetup() {
        self.tableView.separatorStyle = .none
        self.tableView.register(UINib(resource: R.nib.settingOneTableItem), forCellReuseIdentifier: R.reuseIdentifier.settingOneTableItem.identifier)
        self.tableView.register(UINib(resource: R.nib.settingThreeTableItem), forCellReuseIdentifier: R.reuseIdentifier.settingThreeTableItem.identifier)
    }
    
    func setSettingsData() {
        self.settings = []
        self.settings = [
            SettingModel(settingItem: [
                SettingItems(title: "Edit My Channel", image: "s_edit"),
                SettingItems(title: "Withdrawals", image: "s_withdrawal"),
                SettingItems(title: "Go Pro", image: "s_pro"),
                SettingItems(title: "Verification", image: "s_verification"),
                SettingItems(title: "Points", image: "points_new"),
                SettingItems(title: "Blocked Users", image: "s_block"),
                SettingItems(title: "Wallet", image: "wallet_new"),
                SettingItems(title: "Link with TV code", image: "tv_new")
            ]),
            SettingModel(settingItem: [
                SettingItems(title: "Password", image: "s_password"),
                SettingItems(title: "Two Factor Authentication", image: "s_two"),
                SettingItems(title: "Manage Sessions", image: "s_manage")
            ]),
            SettingModel(settingItem: [
                SettingItems(title: "Theme", image: "s_theme"),
                SettingItems(title: "Picture in picture", image: "s_pip")
            ]),
            SettingModel(settingItem: [
                SettingItems(title: "Clear watched history", image: "s_history"),
                SettingItems(title: "Pause watch history", image: "s_pause_history"),
                SettingItems(title: "Clear Cache", image: "s_cache")
            ]),
            SettingModel(settingItem: [
                SettingItems(title: "Rate our App", image: "s_rate"),
                SettingItems(title: "Invite Friends", image: "s_invite"),
                SettingItems(title: "About us", image: "s_about"),
                SettingItems(title: "Terms of use", image: "s_terms"),
                SettingItems(title: "Privacy Policy", image: "s_terms"),
                SettingItems(title: "Help", image: "s_help")
            ]),
            SettingModel(settingItem: [
                SettingItems(title: "Delete Account", image: "s_delete"),
                SettingItems(title: "Logout", image: "s_logout")
            ])
        ]
        if AppInstance.instance.userProfile?.data?.is_pro == 1 {
            for (index, value) in self.settings.enumerated() {
                for (sIndex, sValue) in (value.settingItem ?? []).enumerated() {
                    if sValue.title == "Go Pro" {
                        self.settings[index].settingItem?.remove(at: sIndex)
                    }
                }
            }
        }
        self.tableView.reloadData()
    }
    
    func getPauseHistory() {
        AppInstance.instance.fetchUserProfile { success in
            if success {
                self.pause_history = AppInstance.instance.userProfile?.data?.pause_history ?? 0
                self.tableView.reloadData()
            }
        }
    }
    
    private func clearWatchHistory() {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                MyChannelManager.instance.clearWatchHistory(User_id: userID, Session_Token: sessionID, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("SUCCESS = \(success?.message ?? "")")
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            self.view.makeToast(sessionError?.errors.error_text ?? "")
                            log.verbose("SessionError = \(sessionError?.errors.error_text ?? "")")
                        }
                    } else {
                        self.dismissProgressDialog {
                            self.view.makeToast(error?.localizedDescription ?? "")
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    private func updatePauseWatchHistory() {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            GetSettingsManager.instance.updatePauseWatchHistory() { (success, authError, error) in
                if (success != nil) {
                    self.dismissProgressDialog {
                        print(success?.message ?? "")
                        self.view.makeToast(success?.message ?? "")
                        self.getPauseHistory()
                    }
                } else if (authError != nil) {
                    self.dismissProgressDialog {
                        self.view.makeToast(authError?.errors?.error_text ?? "")
                    }
                } else {
                    self.dismissProgressDialog {
                        self.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
        
}

// MARK: - Extensions

// MARK: TableView Setup
extension SettingVC: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return self.settings.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.settings[section].settingItem?.count ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let rowPath = self.settings[indexPath.section].settingItem?[indexPath.item]
        if rowPath?.title == "Picture in picture" {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.settingThreeTableItem.identifier) as! SettingThreeTableItem
            cell.titleLabel.text = rowPath?.title ?? ""
            cell.pictureInPictureDelegate = self
            cell.type = "picture_in_picture"
            cell.imgLeft.image = UIImage(named: rowPath?.image ?? "")
            cell.`switch`.isOn = self.picture_In_Picture
            return cell
        } else if rowPath?.title == "Pause watch history" {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.settingThreeTableItem.identifier) as! SettingThreeTableItem
            cell.titleLabel.text = rowPath?.title ?? ""
            cell.pauseHistoryDelegate = self
            cell.type = "pause_history"
            cell.imgLeft.image = UIImage(named: rowPath?.image ?? "")
            cell.`switch`.isOn = self.pause_history == 1
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.settingOneTableItem.identifier) as! SettingOneTableItem
            cell.titleLabel.text = rowPath?.title ?? ""
            cell.imgLeft.image = UIImage(named: rowPath?.image ?? "")
            return cell
        }
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.deselectRow(at: indexPath, animated: true)
        let rowPath = self.settings[indexPath.section].settingItem?[indexPath.item]
        switch rowPath?.title {
        case "Edit My Channel":
            if let newVC = R.storyboard.settings.editChannelVC() {
                newVC.is_Setting = 1
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Withdrawals":
            if let newVC = R.storyboard.settings.withdrawalsVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Go Pro":
            if let newVC = R.storyboard.upgrade.upgradeVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Verification":
            if AppInstance.instance.userProfile?.data?.verified == 0{
                if let newVC = R.storyboard.settings.verificationVC() {
                    self.navigationController?.pushViewController(newVC, animated: true)
                }
            } else {
                if let newVC = R.storyboard.settings.verifiedVC() {
                    self.navigationController?.pushViewController(newVC, animated: true)
                }
            }
        case "Points":
            if let newVC = R.storyboard.settings.pointsVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Blocked Users":
            if let newVC = R.storyboard.settings.blockedUsersVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Wallet":
            if let newVC = R.storyboard.settings.walletVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Link with TV code":
            if let newVC = R.storyboard.settings.linkTvVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Password":
            if let newVC = R.storyboard.settings.changePasswordVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Two Factor Authentication":
            if let newVC = R.storyboard.settings.twoFactorController() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Manage Sessions":
            if let newVC = R.storyboard.settings.manageSessionVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Theme":
            if let newVC = R.storyboard.popups.themeOptionPopupVC() {
                newVC.modalPresentationStyle = .custom
                newVC.transitioningDelegate = self
                newVC.sheetHeight = 220
                self.present(newVC, animated: true)
            }
        case "Picture in picture":
            print("")
        case "Clear watched history":
            self.clearWatchHistory()
        case "Pause watch history":
            print("")
        case "Clear Cache":
            if let newVC = R.storyboard.popups.clearCachesPopupVC() {
                newVC.modalPresentationStyle = .custom
                newVC.transitioningDelegate = self
                self.present(newVC, animated: true)
            }
        case "Rate our App":
            print("Rate our App")
            self.view.makeToast("Coming Soon....")
        case "Invite Friends":
            if let newVC = R.storyboard.settings.phoneContactsVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "About us":
            if let newVC = R.storyboard.settings.settingsWebViewVC() {
                newVC.headerText = "About"
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Terms of use":
            if let newVC = R.storyboard.settings.settingsWebViewVC() {
                newVC.headerText = "Terms of use"
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Privacy Policy":
            if let newVC = R.storyboard.settings.settingsWebViewVC() {
                newVC.headerText = "Privacy Policy"
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Help":
            if let newVC = R.storyboard.settings.settingsWebViewVC() {
                newVC.headerText = "Help"
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Delete Account":
            if let newVC = R.storyboard.settings.deleteAccountVC() {
                self.navigationController?.pushViewController(newVC, animated: true)
            }
        case "Logout":
            if let newVC = R.storyboard.popups.logoutPopupVC() {
                newVC.parentVC = self
                newVC.modalPresentationStyle = .custom
                newVC.transitioningDelegate = self
                self.present(newVC, animated: true)
            }
        default:
            break
        }
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        if section != 0 {
            let view = UIView(frame: CGRect(x: 0, y: 0, width: tableView.frame.size.width, height: 20))
            view.backgroundColor = UIColor.clear
            let separatorView = UIView(frame: CGRect(x: 0, y: 0, width: view.frame.size.width, height: 1))
            separatorView.backgroundColor = UIColor(named: "Separator_Colors_Opaque")
            view.addSubview(separatorView)
            return view
        } else {
            return nil
        }
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if section == 0 {
            return 0
        } else {
            return 20
        }
    }
    
}

// MARK: UIViewControllerTransitioningDelegate Methods
extension SettingVC: UIViewControllerTransitioningDelegate {
    
    func presentationController(forPresented presented: UIViewController, presenting: UIViewController?, source: UIViewController) -> UIPresentationController? {
        BottomSheetPresentationController(presentedViewController: presented, presenting: presenting)
    }
    
}

// MARK: PauseHistoryDelegate Methods
extension SettingVC: PauseHistoryDelegate {
    
    func handlePauseHistorySwitchTap(_ sender: UISwitch) {
        self.updatePauseWatchHistory()
    }
    
}

// MARK: PictureInPictureDelegate Methods
extension SettingVC: PictureInPictureDelegate {
    
    func handlePictureInPictureSwitchTap(_ sender: UISwitch) {
        if AVPictureInPictureController.isPictureInPictureSupported() {
            self.picture_In_Picture = !self.picture_In_Picture
            UserDefaults.standard.setPictureInPicture(value: self.picture_In_Picture, ForKey: Local.GET_SETTINGS.picture_In_Picture)
            self.picture_In_Picture = UserDefaults.standard.getPictureInPicture(Key: Local.GET_SETTINGS.picture_In_Picture)
        } else {
            self.view.makeToast("Picture In Picture is not supported on this device")
            UserDefaults.standard.setPictureInPicture(value: AVPictureInPictureController.isPictureInPictureSupported(), ForKey: Local.GET_SETTINGS.picture_In_Picture)
            self.picture_In_Picture = UserDefaults.standard.getPictureInPicture(Key: Local.GET_SETTINGS.picture_In_Picture)
        }
        self.tableView.reloadData()
    }
    
}
