import UIKit
import PlaytubeSDK
import Async
import Alamofire
import Toast_Swift

class VerificationVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var firstNameTextField: UITextField!
    @IBOutlet weak var selectedImage: UIImageView!
    @IBOutlet weak var messageTextField: UITextField!
    @IBOutlet weak var lastNameTextField: UITextField!
    
    // MARK: - Properties
    
    private var imagePicker = UIImagePickerController()
    private var imageData: Data? = nil
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
    }
    
    // MARK: - Selectors
    
    // Submit Button Action
    @IBAction func submitButtonAction(_ sender: UIButton) {
        if (self.firstNameTextField.text ?? "").isEmptyStr {
            self.view.makeToast(NSLocalizedString("Please enter first name.", comment: "Please enter first name."))
        } else if (self.lastNameTextField.text ?? "").isEmptyStr {
            self.view.makeToast(NSLocalizedString("Please enter last name.", comment: "Please enter last name."))
        } else if (self.messageTextField.text ?? "").isEmptyStr {
            self.view.makeToast(NSLocalizedString("Please enter a message.", comment: "Please enter a message."))
        } else if imageData == nil {
            self.view.makeToast(NSLocalizedString("Please select recent picutre of your passport or id.", comment: "Please select recent picutre of your passport or id."))
        } else {
            let firstName = self.firstNameTextField.text ?? ""
            let lastName = self.lastNameTextField.text ?? ""
            let message = self.messageTextField.text ?? ""
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let imageData = self.imageData ?? Data()
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            Async.background {
                VerificationManager.instance.verifyUser(User_id: userID, Session_Token: sessionID, FirstName: firstName, LastName: lastName, Message: message, Identity: imageData, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.view.makeToast(success?.message)
                                log.debug("Success = \(success?.message ?? "")")
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.view.makeToast(sessionError?.errors.error_text)
                                log.error("SessionError = \(sessionError?.errors.error_text ?? "")")
                            }
                        }
                    } else {
                        Async.main {
                            self.dismissProgressDialog {
                                log.error("error = \(error?.localizedDescription ?? "")")
                                self.view.makeToast(error?.localizedDescription)
                            }
                        }
                    }
                })
            }
        }
    }
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
    
    // Select Picture Button Action
    @IBAction func selectPictureButtonAction(_ sender: UIButton) {
        if let newVC = R.storyboard.popups.imageAlertVC() {
            newVC.modalPresentationStyle = .overCurrentContext
            newVC.modalTransitionStyle = .crossDissolve
            newVC.delegate = self
            self.present(newVC, animated: true)
        }
    }
    
}

// MARK: - Extensions

// MARK: ImageAlertVCDelegate Methods
extension VerificationVC: ImageAlertVCDelegate {
    
    func handleImageGalleryTap(_ sender: UIButton) {
        self.setupImagePicker(imagePickerSourceType: .PHOTO_LIBRARY)
    }
    
    func handleTakePictureFromCameraTap(_ sender: UIButton) {
        self.setupImagePicker(imagePickerSourceType: .CAMERA)
    }
    
    func setupImagePicker(imagePickerSourceType: ImagePickerSourceType) {
        self.view.endEditing(true)
        self.imagePicker.delegate = self
        switch imagePickerSourceType {
        case .CAMERA:
            if UIImagePickerController.isSourceTypeAvailable(.camera) {
                self.imagePicker.sourceType = .camera
            } else {
                if let messageAlertVC = R.storyboard.popups.warningPopupVC() {
                    messageAlertVC.messageText = "You don't have camera"
                    self.present(messageAlertVC, animated: true, completion: nil)
                    messageAlertVC.okButton.isHidden = true
                    return
                }
            }
        case .PHOTO_LIBRARY:
            self.imagePicker.sourceType = .photoLibrary
        }
        self.imagePicker.mediaTypes = ["public.image"]
        self.imagePicker.allowsEditing = true
        self.present(self.imagePicker, animated: true, completion: nil)
    }
    
}

// MARK: UIImagePickerControllerDelegate Methods
extension VerificationVC: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
        picker.dismiss(animated: true) {
            if let image = info[.originalImage] as? UIImage {
                self.selectedImage.image = image
                self.imageData = image.jpegData(compressionQuality: 0.1)
            }
        }
    }
    
}
