import UIKit
import PlaytubeSDK
import Async
import Toast_Swift

class WithdrawalsVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var emailTextField: UITextField!
    @IBOutlet weak var amountText: UITextField!
    @IBOutlet weak var amountLabel: UILabel!
    @IBOutlet weak var myBalanceLabel: UILabel!
    
    // MARK: - Properties
    
    var email: String? = ""
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // Withdrawal Button Action
    @IBAction func withdrawalButtonAction(_ sender: UIButton) {
        let amountValue = self.amountText.text?.toInt(defaultValue: 0)
        let userID = AppInstance.instance.userId ?? 0
        let sessionID = AppInstance.instance.sessionId ?? ""
        if amountValue ?? 0 >= 50 && !(self.emailTextField.text?.isEmpty ?? false) {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let email = self.emailTextField.text ?? ""
            Async.background {
                MonetizationManager.instance.monetizeUser(User_id: userID, Session_Token: sessionID, Email: email, Ammount: amountValue ?? 0, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.debug("success = \(success?.message ?? "")")
                                self.view.makeToast(success?.message)
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.error("sessionError = \(sessionError?.errors.error_text ?? "")")
                                self.view.makeToast(sessionError?.errors.error_text)
                            }
                        }
                    } else {
                        Async.main {
                            self.dismissProgressDialog {
                                log.error("error = \(error?.localizedDescription ?? "")")
                                self.view.makeToast(error?.localizedDescription)
                            }
                        }
                    }
                })
            }
        } else if amountValue == nil {
            self.view.makeToast(NSLocalizedString("Please enter amount.", comment: "Please enter amount."))
        } else if self.emailTextField.text!.isEmpty {
            self.view.makeToast(NSLocalizedString("Please enter email.", comment: "Please enter email."))
        } else if !((self.emailTextField.text ?? "").isEmail) {
            self.view.makeToast(NSLocalizedString("Email is badly formatted.", comment: "Email is badly formatted."))
        } else {
            self.view.makeToast(NSLocalizedString("Amount shouldn't be less than 50.", comment: "Amount shouldn't be less than 50."))
        }
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.setData()
    }
    
    private func setData() {
        self.myBalanceLabel.text = NSLocalizedString("MY BALANCE", comment: "MY BALANCE")
        self.amountLabel.text = "$"+(AppInstance.instance.userProfile?.data?.balance ?? "")
    }
    
}
