import UIKit
import AVKit
import Async
import PlaytubeSDK
import FittedSheets
import Toast_Swift

class ShortsVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var activityIndicator: UIActivityIndicatorView!
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var pausePlayButton: UIButton!
    @IBOutlet weak var skipBackwardButton: UIButton!
    @IBOutlet weak var skipForwardButton: UIButton!
    @IBOutlet weak var backwardButton: UIButton!
    @IBOutlet weak var forwardButton: UIButton!
    
    // MARK: - Properties
    
    var shortsVideosArray: [VideoDetail] = []
    var isFromTabbar = false
    var currentIndexPath = IndexPath(item: 0, section: 0)
    fileprivate var isPlaying = false
    lazy var player: AVPlayer = {
        let player = AVPlayer()
        return player
    }()
    lazy var playerLayer: AVPlayerLayer = {
        let playerLayer = AVPlayerLayer()
        return playerLayer
    }()
    fileprivate var isScrubAble = false
    var currentPlayingIndex = 0
    var timer = Timer()
    var tapGesture = UITapGestureRecognizer()
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name("reloadShortData"), object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.reloadShortData(_:)), name: NSNotification.Name("reloadShortData"), object: nil)
        self.initialConfig()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewDidAppear(animated)
        self.player.pause()
    }
    
    // MARK: - Selectors
    
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
    
    // Play Pause Button Action
    @IBAction func playPauseButtonAction(_ sender: UIButton) {
        if self.player.isPlaying {
            self.player.pause()
        } else {
            self.player.play()
        }
        self.timer.invalidate()
        self.timer = Timer.scheduledTimer(timeInterval: 5, target: self, selector: #selector(self.timerAction), userInfo: nil, repeats: true)
        self.pausePlayButton.setImage(self.player.icon, for: .normal)
    }
    
    // Skip Backward Button Action
    @IBAction func skipBackwardButtonAction(_ sender: UIButton) {
        guard self.isScrubAble else { return }
        let currentTime = player.currentTime()
        let currentTimeInSecondsMinus10 =  CMTimeGetSeconds(currentTime).advanced(by: -10)
        let seekTime = CMTime(value: CMTimeValue(currentTimeInSecondsMinus10), timescale: 1)
        self.player.seek(to: seekTime)
    }
    
    // Skip Forward Button Action
    @IBAction func skipForwardButtonAction(_ sender: UIButton) {
        guard isScrubAble else { return }
        let currentTime = player.currentTime()
        let currentTimeInSecondsPlus10 =  CMTimeGetSeconds(currentTime).advanced(by: 10)
        let seekTime = CMTime(value: CMTimeValue(currentTimeInSecondsPlus10), timescale: 1)
        self.player.seek(to: seekTime)
    }
    
    // MARK: - Helper Functions -
    
    // InitialConfig
    func initialConfig() {
        self.setupCollectionView()
        self.handleToggleControls()
        if self.isFromTabbar {
            self.fetchShortsData(UserID: AppInstance.instance.userId ?? 0, SessionID: AppInstance.instance.sessionId ?? "")
        } else {
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                self.collectionView.scrollToItem(at: self.currentIndexPath, at: .centeredVertically, animated: false)
                DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                    guard let cell = self.collectionView.cellForItem(at: self.currentIndexPath) as? ShortsVideoCell else {return}
                    self.handleSetUpVideoPlayer(cell: cell, videoUrlString: self.shortsVideosArray[self.currentIndexPath.item].video_location ?? "")
                }
            }
        }
    }
    
    @objc func reloadShortData(_ notification: NotificationCenter) {
        self.fetchShortsData(UserID: AppInstance.instance.userId ?? 0, SessionID: AppInstance.instance.sessionId ?? "")
    }
    
    // Register Cell
    func setupCollectionView() {
        self.collectionView.delegate = self
        self.collectionView.dataSource = self
    }
    
    fileprivate func setUpTapGesture(cell: ShortsVideoCell) {
        self.tapGesture = UITapGestureRecognizer(target: self, action: #selector(self.handleTapGestureAction))
        cell.postImageView.isUserInteractionEnabled = true
        cell.postImageView.addGestureRecognizer(self.tapGesture)
    }
    
    @objc fileprivate func handleTapGestureAction() {
        guard self.isScrubAble else { return }
        self.handleToggleControls()
        DispatchQueue.main.asyncAfter(deadline: .now() + 5) {
            if self.pausePlayButton.isHidden == false{
                self.timer.invalidate()
                self.timer = Timer.scheduledTimer(timeInterval: 1, target: self, selector: #selector(self.timerAction), userInfo: nil, repeats: true)
            }
        }
    }
    
    fileprivate func handleToggleControls() {
        [self.pausePlayButton, self.skipBackwardButton, self.backwardButton, self.skipForwardButton, self.forwardButton].forEach { view in
            UIView.animate(withDuration: 0.4, delay: 0) {[weak view] in
                guard let view = view else { return }
                view.alpha = view.alpha == 0 ? 1 : 0
                self.isHidden(view.alpha == 0)
            }
        }
    }
    
    func isHidden(_ hide: Bool) {
        let view = [self.pausePlayButton, self.skipBackwardButton, self.backwardButton, self.skipForwardButton, self.forwardButton]
        view.forEach { view in
            view?.isHidden = hide
        }
    }
    
    // called every time interval from the timer
    @objc func timerAction() {
        self.timer.invalidate()
        self.handleToggleControls()
    }
    
    func getCurrentView() -> UIViewController? {
        if let window = UIApplication.shared.keyWindow, var currentView: UIViewController = window.rootViewController {
            while (currentView.presentedViewController != nil) {
                if let presented = currentView.presentedViewController {
                    currentView = presented
                }
            }
            if currentView is UITabBarController {
                if let visible = (currentView as! UITabBarController).selectedViewController {
                    currentView = visible;
                }
            }
            if currentView is UINavigationController {
                if let visible = (currentView as! UINavigationController).visibleViewController {
                    currentView = visible;
                }
            }
            return currentView
        }
        return nil
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension ShortsVC {
    
    func fetchShortsData(UserID: Int, SessionID: String) {
        if Connectivity.isConnectedToNetwork() {
            Async.background {
                ShortsManager.instance.getShortsData(User_id: UserID, Session_Token: SessionID, Limit: 10, Offset: "", completionBlock: { success, sessionError, error in
                    if success != nil {
                        DispatchQueue.main.async {
                            self.shortsVideosArray = []
                            if let data = success?.data {
                                self.shortsVideosArray = AppInstance.instance.getNotInterestedData(data: data)
                            }
                            self.collectionView.reloadData()
                            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                                guard let cell = self.collectionView.cellForItem(at: self.currentIndexPath) as? ShortsVideoCell else {return}
                                if self.getCurrentView() is ShortsVC  {
                                    self.handleSetUpVideoPlayer(cell: cell, videoUrlString: self.shortsVideosArray[self.currentIndexPath.item].video_location ?? "")
                                }
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            log.error("responseError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text)
                        }
                    } else {
                        log.error("error = \(error?.localizedDescription ?? "")")
                        self.view.makeToast(error?.localizedDescription ?? "")
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func fetchVideoDetails(video_id: String, cell: ShortsVideoCell) {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let videoID = video_id
            Async.background {
                PlayVideoManager.instance.getVideosDetailsByVideoId(User_id: userID, Session_Token: sessionID, VideoId: videoID, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            if let object = success?.data {
                                self.shortsVideosArray[self.currentIndexPath.item] = object
                                let rowPath = self.shortsVideosArray[self.currentIndexPath.item]
                                cell.object = rowPath
                                cell.setData(data: rowPath)
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            log.debug("sessionError = \(sessionError?.errors!.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors!.error_text ?? "")
                        }
                    } else {
                        Async.main {
                            log.debug("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func fetchUserProfile(Channel_id: Int) {
        if Connectivity.isConnectedToNetwork() {
            let userId = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                MyChannelManager.instance.getChannelInfo(User_id: userId, Session_Token: sessionID, Channel_id: Channel_id, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.shortsVideosArray[self.currentIndexPath.item].owner = success?.data
                            guard let cell = self.collectionView.cellForItem(at: self.currentIndexPath) as? ShortsVideoCell else { return }
                            let rowPath = self.shortsVideosArray[self.currentIndexPath.item]
                            cell.object = rowPath
                            cell.setData(data: rowPath)
                        }
                    } else if sessionError != nil {
                        Async.main {
                            log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        Async.main {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription)
                        }
                    }
                })
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
    func subscribeChannel() {
        if Connectivity.isConnectedToNetwork() {
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let channelID = self.shortsVideosArray[self.currentIndexPath.item].owner?.id ?? 0
            Async.background {
                PlayVideoManager.instance.subUnsubChannel(User_id: userID, Session_Token: sessionID, Channel_Id: channelID) { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            if success?.code == 0 {
                                self.view.makeToast("Unsubscribed Successfully")
                            } else {
                                self.view.makeToast(NSLocalizedString("Subscribed Successfully", comment: "Subscribed Successfully"))
                            }
                            self.fetchUserProfile(Channel_id: self.shortsVideosArray[self.currentIndexPath.item].owner?.id ?? 0)
                        }
                    } else if sessionError != nil {
                        log.verbose("sessionError = \(sessionError?.errors!.error_text ?? "")")
                        self.view.makeToast(sessionError?.errors!.error_text ?? "")
                    } else {
                        log.verbose("error = \(error?.localizedDescription ?? "")")
                        self.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            }
        } else {
            self.view.makeToast("InterNetError")
        }
    }
    
    func payUsingWalletApi(pay_Type: String, id: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                WalletManager.instance.payUsingWalletApi(user_id: userID, session_token: sessionID, pay_Type: pay_Type, id: id) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("success \(success?.success ?? "")")
                                self.fetchUserProfile(Channel_id: self.shortsVideosArray[self.currentIndexPath.item].owner?.id ?? 0)
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: Collection View Setup
extension ShortsVC: UICollectionViewDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return self.shortsVideosArray.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = self.collectionView.dequeueReusableCell(withReuseIdentifier: "ShortsVideoCell", for: indexPath) as! ShortsVideoCell
        let object = self.shortsVideosArray[indexPath.row]
        cell.object = object
        cell.commentButton.tag = indexPath.row
        cell.subscribeButton.tag = indexPath.row
        cell.delegate = self
        cell.setData(data: object)
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        return CGSize(width: self.collectionView.frame.width, height: self.collectionView.frame.height)
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return 0
    }
    
    func scrollViewWillEndDragging(_ scrollView: UIScrollView, withVelocity velocity: CGPoint, targetContentOffset: UnsafeMutablePointer<CGPoint>) {
        let pageNumber = Int(targetContentOffset.pointee.y / self.collectionView.frame.height)
        let indexPath = IndexPath(item: pageNumber, section: 0)
        self.currentIndexPath = indexPath
        guard let cell = collectionView.cellForItem(at: indexPath) as? ShortsVideoCell else {return}
        self.handleSetUpVideoPlayer(cell: cell, videoUrlString: self.shortsVideosArray[indexPath.item].video_location ?? "")
    }
    
    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        self.player.pause()
    }
    
}

extension ShortsVC {
    
    func handleSetUpVideoPlayer(cell: ShortsVideoCell, videoUrlString: String) {
        if self.currentPlayingIndex == self.currentIndexPath.row && self.currentPlayingIndex != 0 {
            self.player.play()
            return
        }
        self.activityIndicator.startAnimating()
        self.handleFetchVideoFromCachingManagerUsing(urlString: videoUrlString) {[weak self] (url) in
            guard let self = self, let urlUnwrapped = url else { return }
            self.initializeVideoPlayer(url: urlUnwrapped, cell: cell)
        }
        self.fetchVideoDetails(video_id: self.shortsVideosArray[self.currentIndexPath.item].video_id ?? "", cell: cell)
    }
    
    fileprivate func removePeriodicTimeObserver(cell: ShortsVideoCell) {
        self.isPlaying = false
        cell.postImageView.layer.sublayers?.removeAll(where: { $0.name == "AVPlayerLayer" })
        cell.postImageView.removeGestureRecognizer(self.tapGesture)
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name.AVPlayerItemDidPlayToEndTime, object: self.player.currentItem)
        if self.isScrubAble {
            self.isScrubAble = false
            self.player.removeObserver(self, forKeyPath: "currentItem.loadedTimeRanges")
        }
    }
    
    fileprivate func handleFetchVideoFromCachingManagerUsing(urlString: String, completion: @escaping (URL?) -> ()) {
        CacheManager.shared.getFileWith(stringUrl: urlString) { result in
            switch result {
            case .success(let url):
                // do some magic with path to saved video
                completion(url)
                break;
            case .failure(let error):
                // handle errror
                completion(nil)
                print(error, "failed to find value of key\(urlString) in cache and also synchroniously failed to fetch video from our remote server, most likely a network issue like lack of connectivity or database failure")
                break;
            }
        }
    }
    
    fileprivate  func initializeVideoPlayer(url: URL, cell: ShortsVideoCell) {
        self.removePeriodicTimeObserver(cell: cell)
        self.setUpTapGesture(cell: cell)
        self.player.replaceCurrentItem(with: nil)
        let player = AVPlayer(url: url)
        let playerLayer = AVPlayerLayer(player: player)
        playerLayer.frame = cell.bounds
        playerLayer.videoGravity = AVLayerVideoGravity.resizeAspect
        playerLayer.name = "AVPlayerLayer"
        self.player = player
        self.playerLayer = playerLayer
        cell.postImageView.layer.addSublayer(playerLayer)
        player.play()
        self.pausePlayButton.setImage(player.icon, for: .normal)
        self.isPlaying = true
        self.currentPlayingIndex = self.currentIndexPath.row
        player.addObserver(self, forKeyPath: "currentItem.loadedTimeRanges", options: .new, context: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.playerDidPlayToEndTime), name: NSNotification.Name.AVPlayerItemDidPlayToEndTime, object: player.currentItem)
    }
    
    @objc fileprivate func playerDidPlayToEndTime(notification: Notification) {
        self.player.seek(to: CMTime.zero)
        self.player.play()
        self.pausePlayButton.setImage(player.icon, for: .normal)
    }
    
    // MARK: - Player Observer
    override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {
        //This is when the player is ready and rendering frames
        if keyPath == "currentItem.loadedTimeRanges" {
            self.isScrubAble = true
            self.activityIndicator.stopAnimating()
        }
    }
    
}

// MARK: ShortsVideoCellDelegate
extension ShortsVC: ShortsVideoCellDelegate {
    
    func handleSubcribeButtonTap(_ sender: UIButton) {
        if self.shortsVideosArray[self.currentIndexPath.item].owner?.am_i_subscribed == 0 {
            if self.shortsVideosArray[self.currentIndexPath.item].owner?.subscriber_price != "0" {
                let warningPopupVC = R.storyboard.popups.warningPopupVC()
                warningPopupVC?.delegate = self
                warningPopupVC?.titleText  = "Purchase Required"
                warningPopupVC?.messageText = "This channel is paid, You must pay to subscribe"
                warningPopupVC?.okText = "PURCHASE"
                self.present(warningPopupVC!, animated: true, completion: nil)
                warningPopupVC?.okButton.tag = 1001
            } else {
                self.subscribeChannel()
            }
        } else {
            self.subscribeChannel()
        }
    }
    
    func handleCommentButtonTap(_ sender: UIButton) {
        let object = self.shortsVideosArray[sender.tag]
        let Storyboards  = UIStoryboard(name: "Player", bundle: nil)
        let playerCommentsVC = Storyboards.instantiateViewController(withIdentifier: "PlayerCommentVC") as! PlayerCommentVC
        playerCommentsVC.videoID = object.id ?? 0
        playerCommentsVC.delegate = self
        playerCommentsVC.index = sender.tag
        let sheet = SheetViewController(controller: playerCommentsVC, sizes: [.fixed(520), .intrinsic])
        sheet.shouldDismiss = { _ in
            // This is called just before the sheet is dismissed. Return false to prevent the build in dismiss events
            playerCommentsVC.handleRefreshCommentCount()
            return true
        }
        sheet.didDismiss = { _ in
            // This is called after the sheet is dismissed
        }
        self.present(sheet, animated: false, completion: nil)
    }
    
}

// MARK: WarningPopupVCDelegate
extension ShortsVC: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        if sender.tag == 1001 {
            if self.shortsVideosArray[self.currentIndexPath.item].owner?.am_i_subscribed == 0 && self.shortsVideosArray[self.currentIndexPath.item].owner?.subscriber_price != "0" {
                let amount = ((self.shortsVideosArray[self.currentIndexPath.item].owner?.subscriber_price ?? "") as NSString).doubleValue
                AppInstance.instance.fetchUserProfile { success in
                    if success {
                        let walletBalance = AppInstance.instance.userProfile?.data?.wallet?.rounded() ?? 0.0
                        if amount < walletBalance {
                            self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.shortsVideosArray[self.currentIndexPath.item].owner?.id ?? 0)
                        } else {
                            let warningPopupVC = R.storyboard.popups.warningPopupVC()
                            warningPopupVC?.delegate = self
                            warningPopupVC?.titleText  = "Wallet"
                            warningPopupVC?.messageText = "Sorry, You do not have enough money please top up your wallet"
                            warningPopupVC?.okText = "ADD WALLET"
                            self.present(warningPopupVC!, animated: true, completion: nil)
                            warningPopupVC?.okButton.tag = 1002
                        }
                    }
                }
            }
            
        }
        if sender.tag == 1002 {
            let newVC = R.storyboard.settings.walletVC()
            newVC?.isPurchase = true
            newVC?.delegate = self
            self.navigationController?.pushViewController(newVC!, animated: true)
        }
    }
    
}

// MARK: WalletVCDelegate
extension ShortsVC: WalletVCDelegate {
    
    func handleAddMoneyInWalletForPurchase() {
        self.payUsingWalletApi(pay_Type: API.WALLET_PAY_TYPE.subscribe, id: self.shortsVideosArray[self.currentIndexPath.item].owner?.id ?? 0)
    }
    
}

// MARK: PlayerCommentVCDelegate
extension ShortsVC: PlayerCommentVCDelegate {
    
    func handleRefreshCommentCount(index: Int?, count: Int?) {
        if self.shortsVideosArray.count > (index ?? 0) {
            self.shortsVideosArray[index ?? 0].comments_count = count
            guard let cell = self.collectionView.cellForItem(at: IndexPath(row: index ?? 0, section: 0)) as? ShortsVideoCell else { return }
            cell.lblCommentCount.text = "\(self.shortsVideosArray[index ?? 0].comments_count ?? 0)"
        }
    }
    
}
