import UIKit
import PlaytubeSDK
import Async
import Toast_Swift

class UpgradeVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var proPackagePriceLabel: UILabel!
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.navigationController?.popViewController(animated: true)
    }
    
    // Upgrade Now Button Action
    @IBAction func upgradeNowButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        let warningPopupVC = R.storyboard.popups.warningPopupVC()
        warningPopupVC?.delegate = self
        warningPopupVC?.titleText  = "Purchase Required"
        warningPopupVC?.messageText = "Go Pro"
        warningPopupVC?.okText = "PURCHASE"
        self.present(warningPopupVC!, animated: true, completion: nil)
        warningPopupVC?.okButton.tag = 1001
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.setData()
    }
    
    // Set Data
    func setData() {
        self.proPackagePriceLabel.text = "$" + (AppInstance.instance.siteSetting?.pro_pkg_price ?? "")
    }
    
}

// MARK: - Extensions

// MARK: Api Call
extension UpgradeVC {
    
    private func upgradeMemberShip() {
        if Connectivity.isConnectedToNetwork() {
            let sessionID = AppInstance.instance.sessionId ?? ""
            let userID = AppInstance.instance.userId ?? 0
            Async.background {
                UpgradeManager.instance.upgrade(User_id: userID, Session_Token: sessionID, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.debug("userList = \(success?.message ?? "")")
                                self.navigationController?.popViewController(animated: true)
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.view.makeToast(sessionError?.errors?.error_text ?? "")
                                log.error("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            }
                        }
                    } else {
                        Async.main {
                            self.dismissProgressDialog {
                                self.view.makeToast(error?.localizedDescription ?? "")
                                log.error("error = \(error?.localizedDescription ?? "")")
                            }
                        }
                    }
                })
            }
        } else {
            log.error("internetError = \(InterNetError)")
            self.view.makeToast(InterNetError)
        }
    }
    
    func payUsingWalletApi(pay_Type: String, id: Int) {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                WalletManager.instance.payUsingWalletApi(user_id: userID, session_token: sessionID, pay_Type: pay_Type, id: id) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.verbose("success \(success?.success ?? "")")
                                AppInstance.instance.fetchUserProfile { success in
                                    self.navigationController?.popViewController(animated: true)
                                }
                            }
                        }
                    } else if sessionError != nil {
                        self.dismissProgressDialog {
                            log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                            self.view.makeToast(sessionError?.errors?.error_text ?? "")
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.verbose("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                }
            }
        } else {
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: WarningPopupVCDelegate
extension UpgradeVC: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        if sender.tag == 1001 {
            if let newVC = R.storyboard.popups.proPurchaseAlertVC() {
                newVC.delegate = self
                newVC.modalPresentationStyle = .overCurrentContext
                newVC.modalTransitionStyle = .crossDissolve
                self.present(newVC, animated: true)
            }
        }
        if sender.tag == 1002 {
            let newVC = R.storyboard.settings.walletVC()
            newVC?.isPurchase = true
            newVC?.delegate = self
            self.navigationController?.pushViewController(newVC!, animated: true)
        }
    }
    
}

// MARK: WalletVCDelegate
extension UpgradeVC: WalletVCDelegate {
    
    func handleAddMoneyInWalletForPurchase() {
        self.payUsingWalletApi(pay_Type: "pro", id: AppInstance.instance.userId ?? 0)
    }
    
}

// MARK: ProPurchaseAlertVCDelegate
extension UpgradeVC: ProPurchaseAlertVCDelegate {
    
    func handlePurchaseTap(purchaseFrom: String) {
        let amount = ((AppInstance.instance.siteSetting?.pro_pkg_price ?? "") as NSString).doubleValue
        switch purchaseFrom {
        case "InApp":
            A4WPurchaseManager.buyProVersion { success in
                if success {
                    self.upgradeMemberShip()
                }
            }
        case "Wallet":
            AppInstance.instance.fetchUserProfile { success in
                if success {
                    let walletBalance = AppInstance.instance.userProfile?.data?.wallet?.rounded() ?? 0.0
                    if amount < walletBalance {
                        self.payUsingWalletApi(pay_Type: "pro", id: AppInstance.instance.userId ?? 0)
                    } else {
                        let warningPopupVC = R.storyboard.popups.warningPopupVC()
                        warningPopupVC?.delegate = self
                        warningPopupVC?.titleText  = "Wallet"
                        warningPopupVC?.messageText = "Sorry, You do not have enough money please top up your wallet"
                        warningPopupVC?.okText = "ADD WALLET"
                        self.present(warningPopupVC!, animated: true, completion: nil)
                        warningPopupVC?.okButton.tag = 1002
                    }
                }
            }
        default:
            break
        }
    }
    
}
