//
//  WebViewVC.swift
//  Playtube
//
//  Created by Macbook Pro on 04/04/2019.
//  Copyright © 2019 Muhammad Haris Butt. All rights reserved.
//

import UIKit
import WebKit
import PlaytubeSDK

class WebViewVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var headerTitle: UILabel!
    @IBOutlet weak var webView: WKWebView!
    
    // MARK: - Properties
    
    var article: String? = ""
    var mode = ""
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
        self.webView.navigationDelegate = self
        print("Cookie", AppInstance.instance.cookie ?? "")
        self.headerTitle.text = self.article
        if UserDefaults.standard.getSystemTheme(Key: "SystemTheme") {
            if appDelegate.window?.overrideUserInterfaceStyle == .dark {
                self.mode = "night"
            } else {
                self.mode = "day"
            }
        } else {
            if UserDefaults.standard.getDarkMode(Key: "DarkMode") {
                self.mode = "night"
            } else {
                self.mode = "day"
            }
        }
        if article == "Import" {
            webView.load(URLRequest(url: URL(string: API.Articles_Methods.IMPORT_VIDEO_API + "\(AppInstance.instance.cookie ?? "")&mode=\(self.mode)")!))
        } else if article == "Upload" || article == "Create a short" {
            webView.load(URLRequest(url: URL(string: API.Articles_Methods.UPLOAD_API + "\(AppInstance.instance.userId ?? 0)&cookie=\(AppInstance.instance.cookie ?? "")&mode=\(self.mode)")!))
        } else {
            webView.load(URLRequest(url: URL(string: API.Articles_Methods.CREATE_ARTICLE_API + "\(AppInstance.instance.cookie ?? "")&user_id=\(AppInstance.instance.userId ?? 0)&mode=\(self.mode)")!))
        }
    }
    
    // MARK: - Selectors
    
    // Back Button Action
    @IBAction func backButtonAction(_ sender: UIButton) {
        self.navigationController?.popViewController(animated: true)
    }
        
}

// MARK: - Extensions

// MARK: WKNavigationDelegate
extension WebViewVC: WKNavigationDelegate {
    
    func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        self.dismissProgressDialog {
            log.verbose("Dismissed")
            self.webView.evaluateJavaScript("var elements = document.getElementsByClassName('navbar navbar-findcond navbar-fixed-top header-layout');" + "$('.btn-fab-floating').hide();", completionHandler: nil)
        }
    }
    
    func webView(_ webView: WKWebView, didFail navigation: WKNavigation!, withError error: Error) {
        self.dismissProgressDialog {
            log.verbose("Dismissed")
        }
    }
    
    func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        guard let urlAsString = navigationAction.request.url?.absoluteString.lowercased() else { return }
        if urlAsString.range(of: "the url that the button redirects the webpage to") != nil {
            print(urlAsString)
        }
        if urlAsString.contains("watch") {
            NotificationCenter.default.post(name: Notification.Name("VideoUploded"), object: nil)
            self.navigationController?.popViewController(animated: true)
        }
        decisionHandler(.allow)
    }
    
}
