//
//  AgeAlertVC.swift
//  Playtube
//
//  Created by iMac on 23/12/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit

protocol AgeAlertVCDelegate {
    func handleAgeSelection(age: String)
}

class AgeAlertVC: UIViewController {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var tableViewHeight: NSLayoutConstraint!
    @IBOutlet weak var closeButton: UIButton!
    
    // MARK: - Properties
    
    var age_list: [String] = []
    var delegate: AgeAlertVCDelegate?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Close Button Action
    @IBAction func closeButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.dismiss(animated: true, completion: nil)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.view.backgroundColor = .black.withAlphaComponent(0.4)
        self.tableViewSetUp()
        self.setData()
    }
    
    // TableView SetUp
    func tableViewSetUp() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.register(UINib(resource: R.nib.ageAlertCell), forCellReuseIdentifier: R.reuseIdentifier.ageAlertCell.identifier)
        self.tableView.reloadData()
    }
    
    // Set Data
    func setData() {
        self.age_list = (1...99).map { String($0) }
        print(self.age_list)
        self.tableView.reloadData()
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.01) {
            if self.tableView.contentSize.height > (self.view.frame.height * 0.7) {
                self.tableViewHeight.constant = (self.view.frame.height * 0.7)
            } else {
                self.tableViewHeight.constant = self.tableView.contentSize.height
            }
        }
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension AgeAlertVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.age_list.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = self.tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.ageAlertCell.identifier, for: indexPath) as! AgeAlertCell
        cell.titleLabel.text = self.age_list[indexPath.row]
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let age = self.age_list[indexPath.row]
        self.dismiss(animated: true) {
            self.delegate?.handleAgeSelection(age: age)
        }
    }
    
}

