//
//  CategoryPopupVC.swift
//  Playtube
//
//  Created by Abdul Moid on 25/03/2021.
//  Copyright © 2021 Muhammad Haris Butt. All rights reserved.
//

import UIKit

protocol CategoryPopupVCDelegate {
    func handleCategorySelection(category: CategoryStruct)
}

class CategoryPopupVC: UIViewController {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var tableViewHeight: NSLayoutConstraint!
    @IBOutlet weak var closeButton: UIButton!
    
    // MARK: - Properties
    
    var categories: [CategoryStruct] = []
    var delegate: CategoryPopupVCDelegate?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Close Button Action
    @IBAction func closeButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.dismiss(animated: true, completion: nil)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.view.backgroundColor = .black.withAlphaComponent(0.4)
        self.tableViewSetUp()
    }
    
    // TableView SetUp
    func tableViewSetUp() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.register(UINib(resource: R.nib.categoryTableCell), forCellReuseIdentifier: R.reuseIdentifier.categoryTableCell.identifier)
        self.tableView.reloadData()
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.01) {
            if self.tableView.contentSize.height > (self.view.frame.height * 0.7) {
                self.tableViewHeight.constant = (self.view.frame.height * 0.7)
            } else {
                self.tableViewHeight.constant = self.tableView.contentSize.height
            }
        }
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension CategoryPopupVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.categories.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = self.tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.categoryTableCell.identifier, for: indexPath) as! CategoryTableCell
        cell.titleLabel.text = self.categories[indexPath.row].cat_Name
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let category = self.categories[indexPath.row]
        self.dismiss(animated: true) {
            self.delegate?.handleCategorySelection(category: category)
        }
    }
    
}
