//
//  ReleaseAlertVC.swift
//  Playtube
//
//  Created by Abdul Moid on 25/03/2021.
//  Copyright © 2021 Muhammad Haris Butt. All rights reserved.
//

import UIKit

protocol ReleaseAlertVCDelegate {
    func handleReleaseSelection(release: String)
}

class ReleaseAlertVC: UIViewController {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var tableViewHeight: NSLayoutConstraint!
    @IBOutlet weak var closeButton: UIButton!
    
    // MARK: - Properties
    
    var release_list: [String] = []
    var delegate: ReleaseAlertVCDelegate?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Close Button Action
    @IBAction func closeButtonAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.dismiss(animated: true, completion: nil)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.view.backgroundColor = .black.withAlphaComponent(0.4)
        self.tableViewSetUp()
        self.setData()
    }
    
    // TableView SetUp
    func tableViewSetUp() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.register(UINib(resource: R.nib.releaseAlertCell), forCellReuseIdentifier: R.reuseIdentifier.releaseAlertCell.identifier)
        self.tableView.reloadData()
    }
    
    // Set Data
    func setData() {
        self.release_list = (1970...2022).map { String($0) }
        print(self.release_list)
        self.tableView.reloadData()
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.01) {
            if self.tableView.contentSize.height > (self.view.frame.height * 0.7) {
                self.tableViewHeight.constant = (self.view.frame.height * 0.7)
            } else {
                self.tableViewHeight.constant = self.tableView.contentSize.height
            }
        }
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension ReleaseAlertVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.release_list.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = self.tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.releaseAlertCell.identifier, for: indexPath) as! ReleaseAlertCell
        cell.titleLabel.text = self.release_list[indexPath.row]
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let release = self.release_list[indexPath.row]
        self.dismiss(animated: true) {
            self.delegate?.handleReleaseSelection(release: release)
        }
    }
    
}
