import UIKit
import PlaytubeSDK
import Async
import Toast_Swift

protocol ReportVideoDelegate {
    func reportVideo(_ success: Bool)
    func helpAndFeedBack()
}

extension ReportVideoDelegate {
    func helpAndFeedBack() { }
}

class ReportVideoPopupVC: BaseVC {
    
    @IBOutlet weak var submitButton: UIButton!
    @IBOutlet weak var cancelButton: UIButton!
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var reportLabel: UILabel!
    @IBOutlet weak var reportTextField: TJTextField!
    
    var videoID: Int? = 0
    var delegate: ReportVideoDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.view.backgroundColor = .black.withAlphaComponent(0.4)
    }
    
    @IBAction func cancelPressed(_ sender: Any) {
        self.dismiss(animated: true, completion: nil)
    }
    
    @IBAction func submitPressed(_ sender: Any) {
        self.reportVideo()
    }
    
    func reportVideo() {
        if Connectivity.isConnectedToNetwork() {
            self.showProgressDialog(text: NSLocalizedString("Loading...", comment: "Loading..."))
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            let videoID = self.videoID ?? 0
            let reportText = self.reportTextField.text ?? ""
            Async.background {
                ReportManager.instance.reportVideo(User_id: userID, Session_Token: sessionID, id: videoID, text: reportText, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                self.dismiss(animated: true, completion: {
                                    self.view.makeToast(NSLocalizedString("Video Reported successfully!!", comment: "Video Reported successfully!!"))
                                    self.delegate?.reportVideo(true)
                                })
                            }
                        }
                    } else if sessionError != nil {
                        Async.main {
                            self.dismissProgressDialog {
                                log.error("responseError = \(sessionError?.errors!.error_text ?? "")")
                                self.view.makeToast(sessionError?.errors!.error_text)
                            }
                        }
                    } else {
                        self.dismissProgressDialog {
                            log.error("error = \(error?.localizedDescription ?? "")")
                            self.view.makeToast(error?.localizedDescription ?? "")
                        }
                    }
                })
            }
        } else {
            self.dismissProgressDialog {
                self.view.makeToast(InterNetError)
            }
        }
    }
    
}
