//
//  VideoOptionPopupVC.swift
//  Playtube
//
//  Created by iMac on 02/05/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit
import Toast_Swift
import PlaytubeSDK
import Async
import SwiftEventBus

class VideoOptionPopupVC: BaseVC {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var pullBar: UIView!
    @IBOutlet weak var borderView: UIView! {
        didSet {
            borderView.layer.cornerRadius = 3.5
        }
    }
    @IBOutlet weak var tableView: UITableView!
    
    // MARK: - Properties
    
    var sheetHeight: CGFloat = 0
    var sheetBackgroundColor = UIColor(named: "System_Backgrounds_Primary")
    var sheetCornerRadius: CGFloat = 22
    private var hasSetOriginPoint = false
    private var originPoint: CGPoint?
    var optionArray: [String] = []
    var object: VideoDetail?
    var tabBarControllers: TabbarController?
    
    // MARK: - View Life Cycles
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.tabBarControllers = ((obj_appDelegate.window?.rootViewController as? UINavigationController)?.viewControllers.filter({$0 is TabbarController}).first as? TabbarController)
        self.tabBarControllers?.videoDataObject = self.object
        self.initialConfig()
    }
    
    override func viewDidLayoutSubviews() {
        if !hasSetOriginPoint {
            hasSetOriginPoint = true
            originPoint = view.frame.origin
        }
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.setData()
        view.frame.size.height = sheetHeight
        view.isUserInteractionEnabled = true
        view.backgroundColor = sheetBackgroundColor
        view.roundCorners(corners: [.topLeft, .topRight], radius: sheetCornerRadius)
        self.registerCell()
        self.setPanGesture()
    }
    
    // Register Cell
    func registerCell() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.register(UINib(resource: R.nib.videoOptionCell), forCellReuseIdentifier: R.reuseIdentifier.videoOptionCell.identifier)
    }
    
    // Set Pan Gesture
    func setPanGesture() {
        let panGesture = UIPanGestureRecognizer(target: self, action: #selector(panGestureRecognizerAction))
        view.addGestureRecognizer(panGesture)
    }
    
    // Gesture Recognizer
    @objc func panGestureRecognizerAction(sender: UIPanGestureRecognizer) {
        let translation = sender.translation(in: view)
        // Not allowing the user to drag the view upward
        guard translation.y >= 0 else { return }
        view.frame.origin = CGPoint(
            x: 0,
            y: self.originPoint!.y + translation.y
        )
        if sender.state == .ended {
            self.dismiss(animated: true, completion: nil)
        }
    }
    
    // Set Data
    func setData() {
        if AppInstance.instance.getWatchLaterData().contains(where: { $0.id == object?.id }) {            
            self.optionArray.append("Remove from watch later")
        } else {
            self.optionArray.append("Save to watch later")
        }
        self.optionArray.append("Save to playlist")
        if object?.video_type == "video/mp4" {
            self.optionArray.append("Download video")
        }
        if object?.owner?.id == AppInstance.instance.userId {
            self.optionArray.append("Edit Video")
        }
        self.optionArray.append("Share")
        if object?.owner?.id != AppInstance.instance.userId {
            self.optionArray.append("Not interested")
            self.optionArray.append("Report")
        }
        self.sheetHeight = CGFloat((self.optionArray.count * 48)) + (32 + self.view.safeAreaBottom)
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension VideoOptionPopupVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.optionArray.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = self.tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.videoOptionCell.identifier, for: indexPath) as! VideoOptionCell
        cell.setData(content: self.optionArray[indexPath.row])
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.dismiss(animated: true) {
            switch self.optionArray[indexPath.row] {
            case "Save to watch later":
                if AppInstance.instance.getUserSession() {
                    if let object = self.object {
                        self.setWatchLater(object: object)
                    }
                } else {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.okText = "YES"
                    warningPopupVC?.cancelText = "NO"
                    warningPopupVC?.titleText  = "Warning"
                    warningPopupVC?.messageText = "Please sign in to add to WatchLater videos"
                    self.appDelegate.window?.rootViewController?.present(warningPopupVC!, animated: true, completion: nil)
                }
            case "Remove from watch later":
                if AppInstance.instance.getUserSession() {
                    if let object = self.object {
                        self.removeFromWatchLater(object: object)
                    }
                } else {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.okText = "YES"
                    warningPopupVC?.cancelText = "NO"
                    warningPopupVC?.titleText  = "Warning"
                    warningPopupVC?.messageText = "Please sign in to add to WatchLater videos"
                    self.appDelegate.window?.rootViewController?.present(warningPopupVC!, animated: true, completion: nil)
                }
            case "Save to playlist":
                if AppInstance.instance.getUserSession() {
                    self.getPlaylist()
                } else {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.okText = "YES"
                    warningPopupVC?.cancelText = "NO"
                    warningPopupVC?.titleText  = "Warning"
                    warningPopupVC?.messageText = "Please sign in to add to playlist videos"
                    self.appDelegate.window?.rootViewController?.present(warningPopupVC!, animated: true, completion: nil)
                }
            case "Download video":
                if getLocalVideoAdded(url: self.object?.video_location ?? "") {
                    obj_appDelegate.window?.rootViewController?.view.makeToast("Video Already added in offline.")
                    return
                }
                self.tabBarControllers?.downloadButton(UIButton())
            case "Edit Video":
                if let newVC = R.storyboard.loggedUser.editVideoVC() {
                    newVC.object = self.object
                    (self.appDelegate.window?.rootViewController as? UINavigationController)?.pushViewController(newVC, animated: true)
                }
            case "Share":
                if AppInstance.instance.getUserSession() {
                    self.shareVideo()
                }  else {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.okText = "YES"
                    warningPopupVC?.cancelText = "NO"
                    warningPopupVC?.titleText  = "Warning"
                    warningPopupVC?.messageText = "Please sign in to share video"
                    self.appDelegate.window?.rootViewController?.present(warningPopupVC!, animated: true, completion: nil)
                }
            case "Not interested":
                if AppInstance.instance.getUserSession() {
                    self.setNotInterested()
                }  else {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.okText = "YES"
                    warningPopupVC?.cancelText = "NO"
                    warningPopupVC?.titleText  = "Warning"
                    warningPopupVC?.messageText = "You ca  set not interested only after you log in"
                    self.appDelegate.window?.rootViewController?.present(warningPopupVC!, animated: true, completion: nil)
                }
            case "Report":
                if AppInstance.instance.getUserSession() {
                    let vc = R.storyboard.popups.reportVideoPopupVC()
                    vc?.videoID = self.object?.id ?? 0
                    self.appDelegate.window?.rootViewController?.present(vc!, animated: true, completion: nil)
                } else {
                    let warningPopupVC = R.storyboard.popups.warningPopupVC()
                    warningPopupVC?.delegate = self
                    warningPopupVC?.okText = "YES"
                    warningPopupVC?.cancelText = "NO"
                    warningPopupVC?.titleText  = "Warning"
                    warningPopupVC?.messageText = "You can add report only after you log in"
                    self.appDelegate.window?.rootViewController?.present(warningPopupVC!, animated: true, completion: nil)
                }
            default:
                break
            }
        }
    }
}

// MARK: Helper Functions
extension VideoOptionPopupVC {
    
    func setWatchLater(object: VideoDetail) {
        let objectToEncode = object
        let data = try! PropertyListEncoder().encode(objectToEncode)
        var watchLaterData = UserDefaults.standard.getWatchLater(Key: Local.WATCH_LATER.watch_Later)
        watchLaterData.append(data)
        UserDefaults.standard.setWatchLater(value: watchLaterData, ForKey: Local.WATCH_LATER.watch_Later)
        self.appDelegate.window?.rootViewController?.view.makeToast(NSLocalizedString("Added to watch later", comment: "Added to watch later"))
        SwiftEventBus.postToMainThread("RefreshWatchLater")
    }
    
    func removeFromWatchLater(object: VideoDetail) {
        var watchLaterData =  UserDefaults.standard.getSharedVideos(Key: Local.WATCH_LATER.watch_Later)
        for (index, data) in watchLaterData.enumerated() {
            let videoObject = try! PropertyListDecoder().decode(VideoDetail.self ,from: data)
            if videoObject.id == object.id {
                watchLaterData.remove(at: index)
            }
        }
        UserDefaults.standard.setWatchLater(value: watchLaterData, ForKey: Local.WATCH_LATER.watch_Later)
        self.appDelegate.window?.rootViewController?.view.makeToast(NSLocalizedString("Remove from watch later", comment: "Remove from watch later"))
        SwiftEventBus.postToMainThread("RefreshWatchLater")
    }
    
    func setNotInterested() {
        if object != nil {
            log.verbose("Check = \(UserDefaults.standard.getNotInterested(Key: Local.NOT_INTERESTED.not_interested))")
            let objectToEncode = object
            if Connectivity.isConnectedToNetwork() {
                NotInterestManager.instance.addNotInterestAPI(videoID: objectToEncode?.id ?? 0) { success, sessionError, error in
                    if success != nil {
                        Async.main {
                            log.debug("Success: \(success?.message ?? "")")
                            AppInstance.instance.getNotInterestedDataAPI()
                            self.appDelegate.window?.rootViewController?.view.makeToast(success?.message)
                        }
                    } else if sessionError != nil {
                        // self.dismissLoaderOnly(loader: self.activityInd)
                        log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        self.view.makeToast(sessionError?.errors?.error_text ?? "")
                    } else {
                        // self.dismissLoaderOnly(loader: self.activityInd)
                        log.verbose("Error = \(error?.localizedDescription ?? "")")
                        self.view.makeToast(error?.localizedDescription ?? "")
                    }
                }
            } else {
                // self.dismissLoaderOnly(loader: self.activityInd)
                self.view.makeToast(InterNetError)
            }
        }
    }
    
    func shareVideo() {
        // text to share
        let someText: String = "Share Video"
        let url = self.object?.url ?? ""
        print(url)
        // set up activity view controller
        let textToShare = [ someText, url ]
        let activityViewController = UIActivityViewController(activityItems: textToShare, applicationActivities: nil)
        activityViewController.popoverPresentationController?.sourceView = self.appDelegate.window?.rootViewController?.view
        // exclude some activity types from the list (optional,)
        activityViewController.excludedActivityTypes = [
            UIActivity.ActivityType.airDrop,
            UIActivity.ActivityType.postToFacebook,
            UIActivity.ActivityType.assignToContact,
            UIActivity.ActivityType.mail,
            UIActivity.ActivityType.postToTwitter,
            UIActivity.ActivityType.message,
            UIActivity.ActivityType.postToFlickr,
            UIActivity.ActivityType.postToVimeo,
            UIActivity.ActivityType.postToTencentWeibo,
            UIActivity.ActivityType.init(rawValue: "net.whatsapp.WhatsApp.ShareExtension"),
            UIActivity.ActivityType.init(rawValue: "com.google.Gmail.ShareExtension"),
            UIActivity.ActivityType.init(rawValue: "com.toyopagroup.picaboo.share"),
            UIActivity.ActivityType.init(rawValue: "com.tinyspeck.chatlyio.share")
        ]
        activityViewController.completionWithItemsHandler = { activity, completed, items, error in
            if !completed {
                return
            } else {
                if self.object != nil {
                    log.verbose("Check = \(UserDefaults.standard.getSharedVideos(Key: Local.SHARED_VIDEOS.shared_videos))")
                    let objectToEncode = self.object
                    let data = try? PropertyListEncoder().encode(objectToEncode)
                    var getSharedVideosData = UserDefaults.standard.getSharedVideos(Key: Local.SHARED_VIDEOS.shared_videos)
                    if UserDefaults.standard.getSharedVideos(Key: Local.SHARED_VIDEOS.shared_videos).contains(data!) {
                        self.appDelegate.window?.rootViewController?.view.makeToast(NSLocalizedString("Already added in shared videos", comment: "Already added in shared videos"))
                    } else {
                        getSharedVideosData.append(data!)
                        UserDefaults.standard.setSharedVideos(value: getSharedVideosData, ForKey: Local.SHARED_VIDEOS.shared_videos)
                        self.appDelegate.window?.rootViewController?.view.makeToast(NSLocalizedString("Added to shared videos", comment: "Added to shared videos"))
                    }
                }
                
            }
        }
        // present the view controller
        self.appDelegate.window?.rootViewController?.present(activityViewController, animated: true, completion: nil)
    }
    
    func getPlaylist() {
        if Connectivity.isConnectedToNetwork() {
            // self.showLoaderOnly(loader: self.activityInd)
            let userID = AppInstance.instance.userId ?? 0
            let sessionID = AppInstance.instance.sessionId ?? ""
            Async.background {
                PlaylistManager.instance.getPlaylist(User_id: userID, Session_Token: sessionID, completionBlock: { (success, sessionError, error) in
                    if success != nil {
                        Async.main {
                            // self.dismissLoaderOnly(loader: self.activityInd)
                            let vc = R.storyboard.popups.playlistsPopupVC()
                            vc?.videoID = (self.object?.id)
                            vc?.playlistArray = success?.myAllPlaylists ?? []
                            vc?.delegate = self
                            self.appDelegate.window?.rootViewController?.present(vc!, animated: true, completion: nil)
                        }
                    } else if sessionError != nil {
                        // self.dismissLoaderOnly(loader: self.activityInd)
                        log.verbose("sessionError = \(sessionError?.errors?.error_text ?? "")")
                        self.view.makeToast(sessionError?.errors?.error_text ?? "")
                    } else {
                        // self.dismissLoaderOnly(loader: self.activityInd)
                        log.verbose("Error = \(error?.localizedDescription ?? "")")
                        self.view.makeToast(error?.localizedDescription ?? "")
                    }
                })
            }
        } else {
            // self.dismissLoaderOnly(loader: self.activityInd)
            self.view.makeToast(InterNetError)
        }
    }
    
}

// MARK: WarningPopupVCDelegate Methods
extension VideoOptionPopupVC: WarningPopupVCDelegate {
    
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        let vc = R.storyboard.auth.loginVC()
        (self.appDelegate.window?.rootViewController as? UINavigationController)?.pushViewController(vc!, animated: true)
    }
    
}

// MARK: ShowCreatePlayListDelegate Methods
extension VideoOptionPopupVC: ShowCreatePlayListDelegate {
    
    func showCreatePlaylist(Status: Bool) {
        let vc = R.storyboard.playlist.createNewPlaylistVC()
        (self.appDelegate.window?.rootViewController as? UINavigationController)?.pushViewController(vc!, animated: true)
    }
    
}

protocol DownloadProgressUpdateDelegate {
    func downloadProgress(_ progress: Float)
    func downloadFinished(_ success: Bool,_ message: String?)
}

class VideoDownloader: NSObject, URLSessionDelegate, URLSessionDownloadDelegate {
    
    var delegate: DownloadProgressUpdateDelegate?
    var isDownloading = false
    var handler:((_ isDownloading: Bool) -> Void)?
    var downloadTask: URLSessionDownloadTask?
    lazy var backgroundSession: URLSession = {
        let backgroundSessionIdentifier = "com.example.app.backgroundsession"
        let configuration = URLSessionConfiguration.background(withIdentifier: backgroundSessionIdentifier)
        return URLSession(configuration: configuration, delegate: self, delegateQueue: nil)
    }()
    
    
    init(delegate: DownloadProgressUpdateDelegate?) {
        self.delegate = delegate
    }
    
    func downloadVideo(videoURLString: String) {
        guard let videoURL = URL(string: videoURLString) else {
            print("Invalid URL")
            return
        }
        self.downloadTask = backgroundSession.downloadTask(with: videoURL)
        self.isDownloading = true
        self.downloadTask?.resume()
    }
    
    // URLSessionDownloadDelegate methods
    func urlSession(_ session: URLSession, downloadTask: URLSessionDownloadTask, didFinishDownloadingTo location: URL) {
        let documentsURL = FileManager.default.urls(for: .cachesDirectory, in: .userDomainMask).first!
        let destinationURL = documentsURL.appendingPathComponent(downloadTask.originalRequest?.url?.lastPathComponent ?? "")
        do {
            try FileManager.default.copyItem(at: location, to: destinationURL)
            self.isDownloading = false
            self.downloadTask = nil
            self.delegate?.downloadFinished(true, nil)
            print("Video downloaded successfully. Saved at: \(destinationURL)")
        } catch {
            self.isDownloading = false
            self.delegate?.downloadFinished(false, error.localizedDescription)
            print("Error moving video to destination: \(error)")
        }
    }
    
    func urlSession(_ session: URLSession, task: URLSessionTask, didCompleteWithError error: Error?) {
        if let error = error {
            self.isDownloading = false
            self.delegate?.downloadFinished(false, error.localizedDescription)
            print("Error downloading video: \(error)")
        }
    }
    
    func urlSession(_ session: URLSession, downloadTask: URLSessionDownloadTask, didWriteData bytesWritten: Int64, totalBytesWritten: Int64, totalBytesExpectedToWrite: Int64) {
        let progress = Float(totalBytesWritten) / Float(totalBytesExpectedToWrite)
        self.delegate?.downloadProgress(progress)
    }
    
}
