//
//  VideoPlayerPopupVC.swift
//  Playtube
//
//  Created by iMac on 28/08/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit
import Toast_Swift
import PlaytubeSDK
import Async

class VideoPlayerPopupVC: BaseVC {
    
    @IBOutlet weak var lblQuality: UILabel!
    @IBOutlet weak var lblSpeed: UILabel!
    @IBOutlet weak var lblMakeOffline: UILabel!
    @IBOutlet weak var pullBar: UIView!
    @IBOutlet weak var borderView: UIView! {
        didSet {
            borderView.layer.cornerRadius = 2.5
        }
    }
    
    // MARK: - Properties
    var sheetHeight: CGFloat = 0
    var sheetBackgroundColor = UIColor(named: "System_Backgrounds_Primary")
    var sheetCornerRadius: CGFloat = 22
    private var hasSetOriginPoint = false
    private var originPoint: CGPoint?
    var object: VideoDetail?
    var selectedSpeedPlayBack: String = ""
    var selectedQuality: String = ""
    var qualityDelegate: VideoQualityDelegate?
    var reportDelegate: ReportVideoDelegate?
    var progressDelegate: DownloadProgressUpdateDelegate?
    var videoPlayerDelegate: VideoPlayerViewDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        initialConfig()
    }
    
    override func viewDidLayoutSubviews() {
        if !hasSetOriginPoint {
            hasSetOriginPoint = true
            originPoint = view.frame.origin
        }
    }
    
    // Initial Config
    func initialConfig() {
        self.sheetHeight = 290 + (32 + self.view.safeAreaBottom)
        self.lblQuality.text = "Quality - Auto (\(selectedQuality))"
        let speed = self.selectedSpeedPlayBack == "Normal" ? "Normal" : "\(selectedSpeedPlayBack)x"
        self.lblSpeed.text = "Playback Speed - \(speed)"
        view.frame.size.height = sheetHeight
        view.isUserInteractionEnabled = true
        view.backgroundColor = sheetBackgroundColor
        view.roundCorners(corners: [.topLeft, .topRight], radius: sheetCornerRadius)
        self.setPanGesture()
    }
    
    // Set Pan Gesture
    func setPanGesture() {
        let panGesture = UIPanGestureRecognizer(target: self, action: #selector(panGestureRecognizerAction))
        view.addGestureRecognizer(panGesture)
    }
    
    // Gesture Recognizer
    @objc func panGestureRecognizerAction(sender: UIPanGestureRecognizer) {
        let translation = sender.translation(in: view)
        // Not allowing the user to drag the view upward
        guard translation.y >= 0 else { return }
        view.frame.origin = CGPoint(
            x: 0,
            y: self.originPoint!.y + translation.y
        )
        if sender.state == .ended {
            self.dismiss(animated: true, completion: nil)
        }
    }
    
    @IBAction func reportBtnAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if AppInstance.instance.getUserSession() {
            let vc = R.storyboard.popups.reportVideoPopupVC()
            vc?.videoID = self.object?.id ?? 0
            vc?.delegate = self
            self.present(vc!, animated: true, completion: nil)
        } else {
            let warningPopupVC = R.storyboard.popups.warningPopupVC()
            warningPopupVC?.delegate = self
            warningPopupVC?.okText = "YES"
            warningPopupVC?.cancelText = "NO"
            warningPopupVC?.titleText  = "Warning"
            warningPopupVC?.messageText = "You can add report only after you log in"
            self.present(warningPopupVC!, animated: true, completion: nil)
        }
    }
    
    @IBAction func reportCopyrightBtnAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if AppInstance.instance.getUserSession() {
            let vc = R.storyboard.popups.reportCopyrightPopupVC()
            vc?.delegate = self
            vc?.videoID = self.object?.id ?? 0
            self.present(vc!, animated: true, completion: nil)
        } else {
            let warningPopupVC = R.storyboard.popups.warningPopupVC()
            warningPopupVC?.delegate = self
            warningPopupVC?.okText = "YES"
            warningPopupVC?.cancelText = "NO"
            warningPopupVC?.titleText  = "Warning"
            warningPopupVC?.messageText = "You can add report only after you log in"
            self.present(warningPopupVC!, animated: true, completion: nil)
        }
    }
    
    @IBAction func qualityBtnAction(_ sender: UIButton) {
        self.view.endEditing(true)
        guard let newVC = R.storyboard.popups.videoQualityPopupVC() else { return }
        newVC.object = self.object
        newVC.delegate = self
        newVC.isQuality = true
        newVC.selectedDefault = selectedQuality
        self.present(newVC, animated: true)
    }
    
    @IBAction func playbackBtnAction(_ sender: UIButton) {
        self.view.endEditing(true)
        guard let newVC = R.storyboard.popups.videoQualityPopupVC() else { return }
        newVC.object = self.object
        newVC.delegate = self
        newVC.selectedDefault = selectedSpeedPlayBack
        self.present(newVC, animated: true)
    }
    
    @IBAction func helpBtnAction(_ sender: UIButton) {
        self.view.endEditing(true)
        self.dismiss(animated: true) {
            self.reportDelegate?.helpAndFeedBack()
        }
    }
    
    @IBAction func makeOfflineBtnAction(_ sender: UIButton) {
        self.view.endEditing(true)
        if getLocalVideoAdded(url: self.object?.video_location ?? "") {
            obj_appDelegate.window?.rootViewController?.view.makeToast("Video Already added in offline.")
            return
        }
        
        self.dismiss(animated: true) {
            self.videoPlayerDelegate?.downloadButton(sender)
        }
    }
}

// MARK: WarningPopupVCDelegate Methods
extension VideoPlayerPopupVC: WarningPopupVCDelegate {
    func warningPopupOKButtonPressed(_ sender: UIButton) {
        let vc = R.storyboard.auth.loginVC()
        (self.appDelegate.window?.rootViewController as? UINavigationController)?.pushViewController(vc!, animated: true)
    }
}

// MARK: ReportVideoDelegate Methods
extension VideoPlayerPopupVC: ReportVideoDelegate {
    func reportVideo(_ success: Bool) {
        if success {
            self.dismiss(animated: true) {
                self.reportDelegate?.reportVideo(success)
            }
        }
    }
}

// MARK: VideoQualityDelegate Methods
extension VideoPlayerPopupVC: VideoQualityDelegate {
    func selectedVideoSpeed(_ selected: String) {
        self.dismiss(animated: true) {
            self.qualityDelegate?.selectedVideoSpeed(selected)
        }
    }
    
    func selectedQuality(_ selected: String) {
        self.dismiss(animated: true) {
            self.qualityDelegate?.selectedQuality(selected)
        }
    }
}
