//
//  VideoQualityPopupVC.swift
//  Playtube
//
//  Created by iMac on 28/08/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import UIKit

protocol VideoQualityDelegate {
    func selectedQuality(_ selected: String)
    func selectedVideoSpeed(_ selected: String)
}

class VideoQualityPopupVC: UIViewController {
    
    // MARK: - IBOutlets
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var tableViewHeight: NSLayoutConstraint!
    
    // MARK: - Properties
    
    var object: VideoDetail?
    var qualityArray: [String] = []
    var selectedDefault = "360p"
    var delegate: VideoQualityDelegate?
    var isQuality = false
        
    // MARK: - View Life Cycles

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.initialConfig()
    }
    
    // MARK: - Selectors
    
    // Close Button Action
    @IBAction func closeButtonAction(_ sender: UIButton) {
        self.dismiss(animated: true)
    }
    
    // MARK: - Helper Functions
    
    // Initial Config
    func initialConfig() {
        self.view.backgroundColor = .black.withAlphaComponent(0.4)
        self.registerCell()
        self.setTableViewDataAndHeight()
        if isQuality {
            if self.object?.the240p == 1 {
                self.qualityArray.append("240p")
            }
            if self.object?.the360p == 1 {
                self.qualityArray.append("360p")
            }
            if self.object?.the480p == 1 {
                self.qualityArray.append("480p")
            }
            if self.object?.the720p == 1 {
                self.qualityArray.append("720p")
            }
            if self.object?.the1080p == 1 {
                self.qualityArray.append("1080p")
            }
            if self.object?.the2048p == 1 {
                self.qualityArray.append("2048p")
            }
            if self.object?.the4096p == 1 {
                self.qualityArray.append("4096p")
            }
        } else {
            qualityArray = ["0.25", "0.5", "0.75", "Normal", "1.25", "1.5", "1.75", "2"]
        }
        self.tableView.reloadData()
    }
    
    // Register Cell
    func registerCell() {
        self.tableView.delegate = self
        self.tableView.dataSource = self
        self.tableView.register(UINib(resource: R.nib.paymentOptionCell), forCellReuseIdentifier: R.reuseIdentifier.paymentOptionCell.identifier)
    }
    
    // Set TableView Data And Height
    func setTableViewDataAndHeight() {
        self.tableView.reloadData()
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.01) {
            if self.tableView.contentSize.height > (self.view.frame.height * 0.6) {
                self.tableViewHeight.constant = (self.view.frame.height * 0.6)
            } else {
                self.tableViewHeight.constant = self.tableView.contentSize.height
            }
        }
    }
    
}

// MARK: - Extensions

// MARK: TableView Setup
extension VideoQualityPopupVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.qualityArray.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = self.tableView.dequeueReusableCell(withIdentifier: R.reuseIdentifier.paymentOptionCell.identifier, for: indexPath) as! PaymentOptionCell
        let object = self.qualityArray[indexPath.row]
        let speed = object == "Normal" ? object : "\(object)x"
        cell.titleLabel.text = self.isQuality ? object : speed
        cell.titleLabel.textColor = selectedDefault == object ? .primary_UI_Primary : UIColor(named: "Label_Colors_Secondary")
        cell.titleLabel.font = selectedDefault == object ? R.font.ttCommonsDemiBold.callAsFunction(size: 16.0)! : R.font.ttCommonsRegular.callAsFunction(size: 16.0)!
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.dismiss(animated: true) {
            let object = self.qualityArray[indexPath.row]
            if self.isQuality {
                self.delegate?.selectedQuality(object)
            } else {
                self.delegate?.selectedVideoSpeed(object)
            }
        }
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 0
    }
    
    func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        return 0
    }
    
}
