//
//  GoLiveManager.swift
//  Playtube
//
//  Created by Abdul Moid on 18/03/2021.
//  Copyright © 2021 Muhammad Haris Butt. All rights reserved.
//

import Foundation
import Alamofire
import PlaytubeSDK

class GoLiveManager: NSObject {
    
    static let instance = GoLiveManager()
    
    func createLiveStream(streamName: String, user_id: Int, type: String, session_token: String, completionBlock: @escaping (_ Success: CreateLiveModel?, _ SessionError: ErrorModel?, Error?) -> () ) {
        let params = [
            API.Params.user_id: "\(user_id)",
            API.Params.Comment_Type: type,
            API.Params.stream_name: streamName,
            API.Params.session_token: session_token,
            API.Params.ServerKey: API.SERVER_KEY.Server_Key
        ] as [String : Any]
        AF.request(API.LiveStreamMethod.LiveStream, method: .post, parameters: params, encoding:URLEncoding.default , headers: nil).responseJSON { (response) in
            if let res = response.value as? [String: Any] {
                guard let api_status = res["api_status"]  as? String else {return}
                if api_status == "200" {
                    let data = try! JSONSerialization.data(withJSONObject: res, options: [])
                    let result = try! JSONDecoder().decode(CreateLiveModel.self, from: data)
                    completionBlock(result, nil, nil)
                } else if api_status == "400" {
                    let data = try! JSONSerialization.data(withJSONObject: res, options: [])
                    let result = try! JSONDecoder().decode(ErrorModel.self, from: data)
                    completionBlock(nil, result, nil)
                }
            } else {
                completionBlock(nil, nil, response.error)
            }
        }
    }
    
    func endStream(postID: Int, Session_Token: String, type: String, userid: Int, completionBlock: @escaping (_ Success: DeleteLiveModel?, _ SessionError: ErrorModel?, Error?) -> () ) {
        let params = [
            API.Params.user_id: "\(userid)",
            API.Params.Comment_Type: type,
            API.Params.session_token: Session_Token,
            API.Params.Post_Id: postID,
            API.Params.ServerKey: API.SERVER_KEY.Server_Key
        ] as [String : Any]
        AF.request(API.LiveStreamMethod.LiveStream, method: .post, parameters: params, encoding:URLEncoding.default , headers: nil).responseJSON { (response) in
            if let res = response.value as? [String: Any] {
                guard let api_status = res["api_status"]  as? String else {return}
                if api_status == "200" {
                    let data = try! JSONSerialization.data(withJSONObject: res, options: [])
                    let result = try! JSONDecoder().decode(DeleteLiveModel.self, from: data)
                    completionBlock(result, nil, nil)
                } else if api_status == "400" {
                    let data = try! JSONSerialization.data(withJSONObject: res, options: [])
                    let result = try! JSONDecoder().decode(ErrorModel.self, from: data)
                    completionBlock(nil, result, nil)
                }
            } else {
                completionBlock(nil, nil, response.error)
            }
        }
    }
    
    func checkLiveComments(userid: Int, Session_Token: String, page: String, postID: Int, completionBlock: @escaping (_ Success: CheckLiveModel?, _ SessionError: ErrorModel?, Error?) -> () ) {
        let params = [
            API.Params.user_id: "\(userid)",
            API.Params.Comment_Type: API.Params.check_comments,
            API.Params.session_token : Session_Token,
            API.Params.Post_Id : postID,
            API.Params.ServerKey: API.SERVER_KEY.Server_Key,
            API.Params.page: page
        ] as [String : Any]
        AF.request(API.LiveStreamMethod.LiveStream, method: .post, parameters: params, encoding: URLEncoding.default, headers: nil).responseJSON { (response) in
            if let res = response.value as? [String: Any] {
                log.verbose("Result = \(res)")
                guard let api_status = res["api_status"] else { return }
                if (api_status as? Int) == 200 {
                    let data = try! JSONSerialization.data(withJSONObject: res, options: [])
                    let result = try! JSONDecoder().decode(CheckLiveModel.self, from: data)
                    completionBlock(result, nil, nil)
                } else if (api_status as? String) == "400" {
                    let data = try! JSONSerialization.data(withJSONObject: res, options: [])
                    let result = try! JSONDecoder().decode(ErrorModel.self, from: data)
                    completionBlock(nil, result, nil)
                }
            } else {
                log.error("error = \(response.error?.localizedDescription ?? "")")
                completionBlock(nil, nil, response.error)
            }
        }
    }
    
}
