//
//  VideoManager.swift
//  Playtube
//
//  Created by iMac on 13/12/23.
//  Copyright © 2023 ScriptSun. All rights reserved.
//

import Foundation
import PlaytubeSDK
import Alamofire

class VideoManager: NSObject {
    
    static let instance = VideoManager()
    
    func editVideoApi(user_id: Int, session_token: String, video_id: Int, title: String, description: String, tags: String, privacy: Int, category_id: String, age_restriction: Int, thumbnail: Data?, completionBlock: @escaping (_ Success: ManageVideoModel?, _ SessionError: ErrorModel?, Error?) -> () ) {
        let params = [
            API.Params.user_id: user_id,
            API.Params.session_token: session_token,
            API.Params.ServerKey: API.SERVER_KEY.Server_Key,
            API.Params.video_id: video_id,
            API.Params.title: title,
            API.Params.description: description,
            API.Params.tags: tags,
            API.Params.privacy: privacy,
            API.Params.category_id: category_id,
            API.Params.age_restriction: age_restriction
        ] as [String : Any]
        let headers: HTTPHeaders = [
            "Content-type": "multipart/form-data"
        ]
        print(params)
        let URL = API.Video_Methods.EDIT_VIDEO_API
        print("URL :=> ",URL)
        AF.upload(multipartFormData: { (multipartFormData) in
            for (key, value) in params {
                multipartFormData.append("\(value)".data(using: String.Encoding.utf8)!, withName: key as String)
            }
            if let data = thumbnail {
                multipartFormData.append(data, withName: "thumbnail", fileName: "thumbnail.jpg", mimeType: "image/png")
            }
        }, to: URL, usingThreshold: UInt64.init(), method: .post, headers: headers).responseJSON { (response) in
            if let Res = response.value as? [String: Any] {
                guard let api_status = Res["api_status"]  as? String else { return }
                if api_status == "200" {
                    let data = try? JSONSerialization.data(withJSONObject: Res, options: [])
                    let result = try! JSONDecoder().decode(ManageVideoModel.self, from: data!)
                    completionBlock(result, nil, nil)
                } else if api_status == "400" {
                    let data = try? JSONSerialization.data(withJSONObject: Res, options: [])
                    let result = try? JSONDecoder().decode(ErrorModel.self, from: data!)
                    completionBlock(nil, result, nil)
                }
            } else {
                completionBlock(nil, nil, response.error)
            }
        }
    }
    
    func deleteVideoApi(user_id: Int, session_token: String, id: Int, completionBlock: @escaping (_ Success: ManageVideoModel?, _ SessionError: ErrorModel?, Error?) -> () ) {
        let params = [
            API.Params.user_id : user_id,
            API.Params.session_token : session_token,
            API.Params.ServerKey : API.SERVER_KEY.Server_Key
        ] as [String : Any]
        print(params)
        let URL = API.Video_Methods.DELETE_VIDEO_API + "&id=\(id)"
        print("URL :=> ", URL)
        AF.request(URL, method: .post, parameters: params, encoding: URLEncoding.default, headers: nil).responseJSON { (response) in
            if let Res = response.value as? [String: Any] {
                guard let api_status = Res["api_status"]  as? String else { return }
                if api_status == "200" {
                    let data = try? JSONSerialization.data(withJSONObject: Res, options: [])
                    let result = try! JSONDecoder().decode(ManageVideoModel.self, from: data!)
                    completionBlock(result, nil, nil)
                } else if api_status == "400" {
                    let data = try? JSONSerialization.data(withJSONObject: Res, options: [])
                    let result = try? JSONDecoder().decode(ErrorModel.self, from: data!)
                    completionBlock(nil, result, nil)
                }
            } else {
                completionBlock(nil, nil, response.error)
            }
        }
    }
    
}
