//
//  EmojiTextField.swift
//  Playtube
//
//  Created by iMac on 30/01/24.
//  Copyright © 2024 ScriptSun. All rights reserved.
//

import Foundation
import UIKit

class EmojiTextField: UITextField {
    
    var isEmojiKeyboard = false
    
    // required for iOS 13
    override var textInputContextIdentifier: String? { "" }
    
    override var textInputMode: UITextInputMode? {
        if self.isEmojiKeyboard {
            for mode in UITextInputMode.activeInputModes {
                if mode.primaryLanguage == "emoji" {
                    return mode
                }
            }
        } else {
            return UITextInputMode.activeInputModes.first
        }
        return nil
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        commonInit()
    }
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
        
        commonInit()
    }
    
    func commonInit() {
        NotificationCenter.default.addObserver(self, selector: #selector(inputModeDidChange), name: UITextInputMode.currentInputModeDidChangeNotification, object: nil)
    }
    
    @objc func inputModeDidChange(_ notification: Notification) {
        guard isFirstResponder else {
            return
        }
        DispatchQueue.main.async { [weak self] in
            self?.reloadInputViews()
        }
    }
    
}
